/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.andami.ui.mdiManager;

import java.awt.Rectangle;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;


/**
 * DOCUMENT ME!
 *
 * @author Fernando Gonzlez Corts
 */
public class WindowInfo {
    /** DOCUMENT ME! */
    public static final int RESIZABLE = 1;

    /** DOCUMENT ME! */
    public static final int MAXIMIZABLE = 2;

    /** DOCUMENT ME! */
    public static final int ICONIFIABLE = 4;

    /** DOCUMENT ME! */
    public static final int MODALDIALOG = 8;
    public static final int MODELESSDIALOG = 16;
    public static final int PALETTE = 32;
    private PropertyChangeSupport support = new PropertyChangeSupport(this);

    /** DOCUMENT ME! */
    private boolean resizable = false;

    /** DOCUMENT ME! */
    private boolean maximizable = false;

    /** DOCUMENT ME! */
    private boolean iconifiable = false;

    /** DOCUMENT ME! */
    private boolean modal = false;
    private boolean modeless = false;

    /**
     * Se usa para poner una ventana de tipo paleta, por
     * encima de las dems. Equivale a poner usar
     * JDesktopPane.PALETTE_LAYER
     */
    private boolean palette = false;

    /** These properties store the dimension and position of the frame */
    private int width = -1;
    private int height = -1;
    private int x = 0;
    private int y = 0;
    /**
     * These properties store the position and dimension of the frame when it is not maximized
     * (so that it can be restored to its original size). They are equal to the not-normal properties
     * when the frame is not maximized, and different when the frame is maximized.
     */
    private int normalX = 0;
    private int normalY = 0;
    private int normalHeight = 0;
    private int normalWidth = 0;
    /* Whether the window is maximized */
    private boolean isMaximized = false;
    private boolean visible = true;
    /* Whether the window is closed */
    private boolean isClosed = false;


    /** DOCUMENT ME! */
    private String title;

    private int id;

    /**
     * ActionCommand del tool seleccionado. Lo usamos
     * para activar el tool que estaba seleccionado en
     * la vista, cuando volvemos a ella.
     */
    private String selectedTool;

    /**
     * DOCUMENT ME!
     *
     * @return Returns the x.
     */
    public int getX() {
        return x;
    }

    /**
     * DOCUMENT ME!
     *
     * @param x The x to set.
     */
    public void setX(int x) {
    	support.firePropertyChange("x", this.x, x);
    	this.x = x;
    }

    /**
     * DOCUMENT ME!
     *
     * @return Returns the y.
     */
    public int getY() {
        return y;
    }

    /**
     * DOCUMENT ME!
     *
     * @param y The y to set.
     */
    public void setY(int y) {
    	support.firePropertyChange("y", this.y, y);
        this.y = y;
    }

    /**
     * DOCUMENT ME!
     *
     * @param listener
     */
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        support.addPropertyChangeListener(listener);
    }

    /**
     * DOCUMENT ME!
     *
     * @param listener
     */
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        support.removePropertyChangeListener(listener);
    }

    /**
     * Establece las propiedades de la vista
     *
     * @param code Bit-or de las propiedades de la vista
     *
     * @throws IllegalStateException DOCUMENT ME!
     */
    public WindowInfo(int code) {
        resizable = (code % 2) > 0;
        code = code / 2;
        maximizable = (code % 2) > 0;
        code = code / 2;
        iconifiable = (code % 2) > 0;
        code = code / 2;
        modal = (code % 2) > 0;
        code = code / 2;
        modeless = (code % 2) > 0;
        code = code / 2;
        palette = (code % 2) > 0;

        if (modal && modeless) {
            throw new IllegalStateException("modal && modeless");
        }
    }

    public WindowInfo(){

    }

    /**
     * Si es iconificable
     *
     * @return
     */
    public boolean isIconifiable() {
        return iconifiable;
    }

    /**
     * Si es maximizable
     *
     * @return
     */
    public boolean isMaximizable() {
        return maximizable;
    }

    /**
     * Si es resizable
     *
     * @return
     */
    public boolean isResizable() {
        return resizable;
    }

    /**
     * Devuelve si el dilogo es modal
     *
     * @return
     */
    public boolean isModal() {
        return modal;
    }

    /**
     * Obtiene la altura de la vista
     *
     * @return
     */
    public int getHeight() {
        return height;
    }

    /**
     * Obtiene la anchura de la vista
     *
     * @return
     */
    public int getWidth() {
        return width;
    }

    /**
     * establece la propiedad altura
     *
     * @param i
     */
    public void setHeight(int i) {
    	support.firePropertyChange("height", this.height, i);
        height = i;
    }

    /**
     * establece la propiedad anchura
     *
     * @param i
     */
    public void setWidth(int i) {
    	support.firePropertyChange("width", this.width, i);
        width = i;
    }

    /**
     * obtiene la propiedad titulo
     *
     * @return
     */
    public String getTitle() {
        return title;
    }

    /**
     * establece la propiedad titulo
     *
     * @param string
     */
    public void setTitle(String string) {
    	support.firePropertyChange("title", this.title, string);
        title = string;
    }

    /**
     * DOCUMENT ME!
     *
     * @return
     */
    public boolean isModeless() {
        return modeless;
    }

    /**
     * DOCUMENT ME!
     *
     * @return
     */
    public boolean isVisible() {
        return visible;
    }

    public boolean isPalette()
    {
        return palette;
    }

    public void setId(int id){
    	this.id = id;
    }

    public int getId(){
    	return id;
    }

    public void setWindowInfo(WindowInfo vi){
    	this.resizable = vi.resizable;
    	this.maximizable = vi.maximizable;
    	this.iconifiable = vi.iconifiable;
    	this.modal = vi.modal;
    	this.modeless = vi.modeless;
    	this.width = vi.width;
    	this.height = vi.height;
    	this.x = vi.x;
    	this.y = vi.y;
    	this.visible = vi.visible;
    	this.title = vi.title;
    	this.id = vi.id;
    }
    public void toPalette(boolean b){
    	this.palette=b;
    }

    public String getSelectedTool() {
        return selectedTool;
    }

    public void setSelectedTool(String selectedTool) {
        if (selectedTool != null)
            this.selectedTool = selectedTool;
    }

    /* Finds out whether a view is open (showing) or closed */
    public boolean isClosed() {
    	return isClosed;
    }

    /* Specifies whether a view is open (showing) or closed */
    public void setClosed(boolean closed) {
    	this.isClosed = closed;
    }

    public void setNormalX(int normalX) {
    	this.normalX = normalX;
    }

    public void setNormalY(int normalY) {
    	this.normalY = normalY;
    }

    public void setNormalHeight(int normalHeight) {
    	this.normalHeight = normalHeight;
    }

    public void setNormalWidth(int normalWidth) {
    	this.normalWidth = normalWidth;
    }


    public void setMaximized(boolean maximized) {
    	this.isMaximized = maximized;
    }

    private void setBounds(int x, int y, int width, int height) {
    	this.x = x;
    	this.y = y;
    	this.width = width;
    	this.height = height;
    }

    public void setNormalBounds(int x, int y, int width, int height) {
    	this.normalX = x;
    	this.normalY = y;
    	this.normalWidth = width;
    	this.normalHeight = height;
    }

    public Rectangle getNormalBounds() {
    	return new Rectangle(getNormalX(), getNormalY(), getNormalWidth(), getNormalHeight());
    }

    public void setNormalBounds(Rectangle normalBounds) {
    	normalX = normalBounds.x;
    	normalY = normalBounds.y;
    	normalHeight = normalBounds.height;
    	normalWidth = normalBounds.width;
    }

    public void setBounds(Rectangle bounds) {
    	x = bounds.x;
    	y = bounds.y;
    	height = bounds.height;
    	width = bounds.width;
    }

    public Rectangle getBounds() {
    	return new Rectangle(x, y, width, height);
    }

    public int getNormalX() {
    	if (normalX!=0)
    		return this.normalX;
    	else
    		return x;
    }

    public int getNormalY() {
    	if (normalY!=0)
    		return this.normalY;
    	else
    		return y;
    }

    public int getNormalHeight() {
    	if (normalHeight!=0)
    		return this.normalHeight;
    	else
    		return height;
    }

    public int getNormalWidth() {
    	if (normalWidth!=0)
    		return this.normalWidth;
    	else
    		return width;
    }

    public boolean isMaximized() {
    	return this.isMaximized;
    }
}
