
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.catalogClient.protocols;
import com.k_int.IR.IRQuery;
import com.k_int.IR.InformationFragment;
import com.k_int.IR.InformationFragmentSource;
import com.k_int.IR.PresentException;
import com.k_int.IR.RecordFormatSpecification;
import com.k_int.IR.SearchException;
import com.k_int.IR.SearchTask;
import com.k_int.IR.Searchable;
import com.k_int.IR.TimeoutExceededException;
import com.k_int.gen.Z39_50_APDU_1995.InitializeResponse_type;
import es.gva.cit.catalogClient.metadataXML.XMLNode;
import es.gva.cit.catalogClient.metadataXML.XMLTree;
import es.gva.cit.catalogClient.metadataXML.XMLTreeNumberOfRecordsAnswer;
import es.gva.cit.catalogClient.utils.Strings;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.net.SocketException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * This class implements the z39.50 protocol communication client.
 * 
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 * @see http://www.loc.gov/z3950/agency/
 */
public class Z3950Protocol extends Z3950OriginBean implements IProtocols {

/**
 * 
 * 
 */
    public static final char ISO2709_RS = 035;

/**
 * 
 * 
 */
    public static final char ISO2709_FS = 036;

/**
 * 
 * 
 */
    public static final char ISO2709_IDFS = 037;

/**
 * 
 * 
 */
    private int auth_type = 0;
// 0=none, 1=anonymous, 2=open, 3=idpass

/**
 * 
 * 
 */
    private String principal = null;

/**
 * 
 * 
 */
    private String group = null;

/**
 * 
 * 
 */
    private String credentials = null;

/**
 * 
 * 
 * 
 * @return 
 * @param url 
 * @param object 
 * @param firstRecord 
 */
    public Collection doQuery(URL url, Object object, int firstRecord) {        
        String message = (String) object;
        XMLNode[] nodes;
        Collection col = new ArrayList();;
       //Search properties
        Properties props = new Properties();
        props.setProperty("ServiceHost", url.getHost());
        props.setProperty("ServicePort", String.valueOf(url.getPort()));
        //props.setProperty("service_short_name","");
        //props.setProperty("service_long_name","");
        //props.setProperty("default_record_syntax","xml");
        //props.setProperty("service_user_principal","");
        //props.setProperty("service_user_group","");
        //props.setProperty("service_user_credentials","");
        //Search Object
        Searchable zServer = new com.k_int.z3950.IRClient.Z3950Origin();
        zServer.init(props);
        //Create the query object
        IRQuery e = new IRQuery();
        e.collections = new Vector();
        e.collections.add(getDatabase(url));
        e.query = new com.k_int.IR.QueryModels.PrefixString(new String(message));
        //create the search task
        SearchTask st = (SearchTask) zServer.createTask(e, null);
        try {
            // time to wait
            st.evaluate(1000000);
            RecordFormatSpecification spec = new RecordFormatSpecification("xml",
                    null, "f");
            InformationFragmentSource ifs = st.getTaskResultSet();
            InformationFragment frag = null;
           
            int lastRecord;
            if (ifs.getFragmentCount() > 10) {
                nodes = new XMLNode[11];
                if ((ifs.getFragmentCount() - firstRecord) < 10) {
                    lastRecord = ifs.getFragmentCount();
                } else {
                    lastRecord = firstRecord + 9;
                }
            } else {
                nodes = new XMLNode[ifs.getFragmentCount() + 1];
                lastRecord = ifs.getFragmentCount();
            }
            nodes[0] = XMLTreeNumberOfRecordsAnswer.getNode(ifs.getFragmentCount(),
                    firstRecord, lastRecord);
            int index = 1;
            for (int i = firstRecord; i <= lastRecord; i++) {
                //Fragment
                frag = ifs.getFragment(i, spec);
                String cadena = frag.toString();
                //This is a strange character
                // cadena = Strings.replace(cadena, "", "");
                //Change the format.
                cadena = Strings.replace(cadena,
                        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>",
                        "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
                InputStream buffer = new ByteArrayInputStream(cadena.getBytes());
                nodes[index] = XMLTree.xmlToTree(buffer);
                index++;               
            }
           
        } catch (TimeoutExceededException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
            return null;
        } catch (SearchException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
            return null;
        } catch (PresentException e1) {
            e1.printStackTrace();
            
            return null;
        } 
        zServer.destroy();
        zServer = null;
        this.disconnect();
        
        for (int i=0 ; i<nodes.length ; i++){
            if (nodes[i] != null){
            	col.add(nodes[i]);
            }
        }
        return col;
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param url 
 */
    public static String getDatabase(URL url) {        
        StringTokenizer sti = new StringTokenizer(url.getPath(), "/");
        if (sti.countTokens() == 0) {
            return "geo";
        } else {
            return sti.nextToken();
        }
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param url 
 */
    public String openConnection(URL url) {        
        String hostname = url.getHost();
        String port = String.valueOf(url.getPort());
        String cadena = "";
        try {
            InitializeResponse_type resp = connect(hostname, port, auth_type,
                    principal, group, credentials);
            if (resp.result.booleanValue() == true) {
                if (resp.referenceId != null) {
                    cadena = "Reference ID : " + new String(resp.referenceId) +
                        "\n";
                }
                cadena = cadena + "Implementation ID : " +
                    resp.implementationId + "\n";
                cadena = cadena + "Implementation Name : " +
                    resp.implementationName + "\n";
                cadena = cadena + "Implementation Version : " +
                    resp.implementationVersion + "\n";
            } else {
                System.out.println(" Failed to establish association");
                return null;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
        return cadena;
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param url 
 */
    public boolean isProtocolSupported(URL url) {        
        try {
            //Search properties
            Properties props = new Properties();
            props.setProperty("ServiceHost", url.getHost());
            props.setProperty("ServicePort", String.valueOf(url.getPort()));
            //Search Object
            Searchable zServer = new com.k_int.z3950.IRClient.Z3950Origin();
            zServer.init(props);
            //Create the query object
            IRQuery e = new IRQuery();
            e.collections = new Vector();
            e.collections.add(url.getHost() + ":" +
                String.valueOf(url.getPort()));
            e.query = new com.k_int.IR.QueryModels.PrefixString(
                    "@attrset geo @attr 1=4 @attr 4=6 @attr 2=3 \"a\"");
            //create the search task
            SearchTask st = (SearchTask) zServer.createTask(e, null);
            st.evaluate(10000);
        } catch (Exception e) {
            System.err.println(e.toString());
            return false;
        }
        
        this.disconnect();
        return true;
    } 
 }
