/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig;

import java.awt.Dimension;
import java.io.IOException;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.instruction.EvaluationException;
import com.hardcode.gdbms.engine.instruction.SemanticException;
import com.hardcode.gdbms.parser.ParseException;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.plugins.Extension;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.cit.gvsig.fmap.edition.EditableAdapter;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.gui.Table;
import com.iver.cit.gvsig.gui.filter.ExpressionListener;
import com.iver.cit.gvsig.gui.tables.AndamiWizard;
import com.iver.cit.gvsig.gui.tables.FieldSelectionModel;
import com.iver.cit.gvsig.gui.tables.ObjectSelectionStep;
import com.iver.cit.gvsig.gui.tables.TableSelectionModel;
import com.iver.cit.gvsig.project.ProjectTable;
import com.iver.utiles.swing.objectSelection.SelectionException;
import com.iver.utiles.swing.wizard.WizardControl;
import com.iver.utiles.swing.wizard.WizardEvent;
import com.iver.utiles.swing.wizard.WizardListener;


/**
 * Extensin que controla las operaciones realizadas sobre las tablas.
 *
 * @author Fernando Gonzlez Corts
 */
public class TableOperations extends Extension implements ExpressionListener {
	private SelectableDataSource dataSource = null;
	//private Table vista;

	/**
	 * @see com.iver.mdiApp.plugins.IExtension#updateUI(java.lang.String)
	 */
	public void execute(String actionCommand) {
		if ("JOIN".equals(actionCommand)) {
			ProjectExtension pe = (ProjectExtension) PluginServices.getExtension(ProjectExtension.class);
			ProjectTable[] pts = (ProjectTable[]) pe.getProject().getTables()
													.toArray(new ProjectTable[0]);

			try {
				final ObjectSelectionStep sourceTable = new ObjectSelectionStep();
				sourceTable.setModel(new TableSelectionModel(pts,
				        PluginServices.getText(this, "seleccione_tabla_origen")));

				final ObjectSelectionStep targetTable = new ObjectSelectionStep();
				targetTable.setModel(new TableSelectionModel(pts,
				        PluginServices.getText(this, "seleccione_tabla_a_enlazar")));

				final ObjectSelectionStep firstTableField = new ObjectSelectionStep();
				final ObjectSelectionStep secondTableField = new ObjectSelectionStep();
				final AndamiWizard wiz = new AndamiWizard(PluginServices.getText(this, "back"), PluginServices.getText(this, "next"), PluginServices.getText(this, "finish"), PluginServices.getText(this, "cancel"));
				wiz.setSize(new Dimension(450,200));
				wiz.addStep(sourceTable);
				wiz.addStep(firstTableField);
				wiz.addStep(targetTable);
				wiz.addStep(secondTableField);

				wiz.addWizardListener(new WizardListener() {
						public void cancel(WizardEvent w) {
							PluginServices.getMDIManager().closeWindow(wiz);
						}

						public void finished(WizardEvent w) {
							PluginServices.getMDIManager().closeWindow(wiz);

							ProjectTable sourceProjectTable = (ProjectTable) sourceTable.getSelected();
							SelectableDataSource sds=null;
							try {
								sds =(SelectableDataSource) sourceProjectTable.getModelo().getRecordset();
							} catch (DriverLoadException e1) {
								// TODO Auto-generated catch block
								e1.printStackTrace();
							}
							String tableName1 = sds.getName();

							ProjectTable targetProjectTable = (ProjectTable) targetTable.getSelected();
							try {
								sds = (SelectableDataSource)targetProjectTable.getModelo().getRecordset();
							} catch (DriverLoadException e1) {
								// TODO Auto-generated catch block
								e1.printStackTrace();
							}

							String tableName2 = sds.getName();

							String field1 = (String) firstTableField.getSelected();
							String field2 = (String) secondTableField.getSelected();

							String sql =
								"custom com_iver_cit_gvsig_arcjoin tables '" +
								tableName1 + "', '" + tableName2 + "' values(" +
								field1 + ", " + field2 + ");";
							System.out.println(sql);

							try {
								SelectableDataSource result = new SelectableDataSource(LayerFactory.getDataSourceFactory()
																								   .executeSQL(sql,
											DataSourceFactory.AUTOMATIC_OPENING));
								ProjectTable projectTable = (ProjectTable) sourceTable.getSelected();
								EditableAdapter auxea=new EditableAdapter();
								auxea.setOriginalDataSource(result);
								projectTable.replaceDataSource(auxea);
							} catch (ParseException e) {
								throw new RuntimeException(e);
							} catch (DriverLoadException e) {
								NotificationManager.addError("Error con la carga de drivers",
									e);
							} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
								NotificationManager.addError("Error leyendo del driver",
									e);
							} catch (SemanticException e) {
								throw new RuntimeException(e);
							} catch (IOException e) {
								NotificationManager.addError("Error de entrada/salida",
									e);
							} catch (EvaluationException e) {
								NotificationManager.addError("Error de evaluacin de la expresin",
										e);
                            }
						}

						public void next(WizardEvent w) {
							WizardControl wiz = w.wizard;
							wiz.enableBack(true);
							wiz.enableNext(((ObjectSelectionStep) wiz.getCurrentStep()).getSelectedItem() != null);

							if (w.currentStep == 1) {
								ProjectTable pt = (ProjectTable) sourceTable.getSelected();

								try {
									firstTableField.setModel(new FieldSelectionModel(
											pt.getModelo().getRecordset(),
											PluginServices.getText(this, "seleccione_campo_enlace"),
											-1));
								} catch (SelectionException e) {
									NotificationManager.addError("Error obteniendo los campos de la tabla",
										e);
								} catch (DriverLoadException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}
							} else if (w.currentStep == 3) {
								try {
									//tabla
									ProjectTable pt = (ProjectTable) sourceTable.getSelected();

									//ndice del campo
									SelectableDataSource sds = (SelectableDataSource)pt.getModelo().getRecordset();
									String fieldName = (String) firstTableField.getSelected();
									int fieldIndex = sds.getFieldIndexByName(fieldName);
									int type = sds.getFieldType(fieldIndex);

									secondTableField.setModel(new FieldSelectionModel(
											((ProjectTable) targetTable.getSelected()).getModelo().getRecordset(),
											PluginServices.getText(this, "seleccione_campo_enlace"),
											type));
								} catch (SelectionException e) {
									NotificationManager.addError("Error obteniendo los campos de la tabla",
										e);
								} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
									NotificationManager.addError("Error obteniendo los campos de la tabla",
										e);
								} catch (DriverLoadException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}
							}
						}

						public void back(WizardEvent w) {
							WizardControl wiz = w.wizard;
							wiz.enableBack(true);
							wiz.enableNext(((ObjectSelectionStep) wiz.getCurrentStep()).getSelectedItem() != null);
						}
					});
				PluginServices.getMDIManager().addWindow(wiz);
			} catch (SelectionException e) {
				NotificationManager.addError("Error abriendo el asistente", e);
			}
		}else if ("LINK".equals(actionCommand)) {
			ProjectExtension pe = (ProjectExtension) PluginServices.getExtension(ProjectExtension.class);
			ProjectTable[] pts = (ProjectTable[]) pe.getProject().getTables()
													.toArray(new ProjectTable[0]);

			try {
				final ObjectSelectionStep sourceTable = new ObjectSelectionStep();
				sourceTable.setModel(new TableSelectionModel(pts,
						PluginServices.getText(this, "seleccione_tabla_origen")));

				final ObjectSelectionStep targetTable = new ObjectSelectionStep();
				targetTable.setModel(new TableSelectionModel(pts,
				        PluginServices.getText(this, "seleccione_tabla_a_enlazar")));

				final ObjectSelectionStep firstTableField = new ObjectSelectionStep();
				final ObjectSelectionStep secondTableField = new ObjectSelectionStep();
				final AndamiWizard wiz = new AndamiWizard(PluginServices.getText(this, "back"), PluginServices.getText(this, "next"), PluginServices.getText(this, "finish"), PluginServices.getText(this, "cancel"));
				wiz.setSize(new Dimension(450,200));
				wiz.addStep(sourceTable);
				wiz.addStep(firstTableField);
				wiz.addStep(targetTable);
				wiz.addStep(secondTableField);

				wiz.addWizardListener(new WizardListener() {
						public void cancel(WizardEvent w) {
							PluginServices.getMDIManager().closeWindow(wiz);
						}

						public void finished(WizardEvent w) {
							PluginServices.getMDIManager().closeWindow(wiz);

							ProjectTable sourceProjectTable = (ProjectTable) sourceTable.getSelected();
							SelectableDataSource sds1=null;;
							try {
								sds1 = (SelectableDataSource)sourceProjectTable.getModelo().getRecordset();
							} catch (DriverLoadException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
							String tableName1 = sds1.getName();

							ProjectTable targetProjectTable = (ProjectTable) targetTable.getSelected();
							SelectableDataSource sds2=null;
							try {
								sds2 = (SelectableDataSource)targetProjectTable.getModelo().getRecordset();
							} catch (DriverLoadException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}

							String tableName2 = sds2.getName();

							String field1 = (String) firstTableField.getSelected();
							String field2 = (String) secondTableField.getSelected();
							sourceProjectTable.setLinkTable(sds2.getName(),field1,field2);
							((ProjectExtension)PluginServices.getExtension(ProjectExtension.class)).getProject().setLinkTable();
						}

						public void next(WizardEvent w) {
							WizardControl wiz = w.wizard;
							wiz.enableBack(true);
							wiz.enableNext(((ObjectSelectionStep) wiz.getCurrentStep()).getSelectedItem() != null);

							if (w.currentStep == 1) {
								ProjectTable pt = (ProjectTable) sourceTable.getSelected();

								try {
									firstTableField.setModel(new FieldSelectionModel(
											pt.getModelo().getRecordset(),
											PluginServices.getText(this, "seleccione_campo_enlace"),
											-1));
								} catch (SelectionException e) {
									NotificationManager.addError("Error obteniendo los campos de la tabla",
										e);
								} catch (DriverLoadException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}
							} else if (w.currentStep == 3) {
								try {
									//tabla
									ProjectTable pt = (ProjectTable) sourceTable.getSelected();

									//ndice del campo
									SelectableDataSource sds = (SelectableDataSource)pt.getModelo().getRecordset();
									String fieldName = (String) firstTableField.getSelected();
									int fieldIndex = sds.getFieldIndexByName(fieldName);
									int type = sds.getFieldType(fieldIndex);

									secondTableField.setModel(new FieldSelectionModel(
											((ProjectTable) targetTable.getSelected()).getModelo().getRecordset(),
											PluginServices.getText(this, "seleccione_campo_enlace"),
											type));
								} catch (SelectionException e) {
									NotificationManager.addError("Error obteniendo los campos de la tabla",
										e);
								} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
									NotificationManager.addError("Error obteniendo los campos de la tabla",
										e);
								} catch (DriverLoadException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}
							}
						}

						public void back(WizardEvent w) {
							WizardControl wiz = w.wizard;
							wiz.enableBack(true);
							wiz.enableNext(((ObjectSelectionStep) wiz.getCurrentStep()).getSelectedItem() != null);
						}
					});
				PluginServices.getMDIManager().addWindow(wiz);
			} catch (SelectionException e) {
				NotificationManager.addError("Error abriendo el asistente", e);
			}
      	}
	}

	/**
	 * @see com.iver.cit.gvsig.gui.filter.ExpressionListener#newSet(java.lang.String)
	 */
	public void newSet(String expression) {
		long[] sel = doSet(expression);

		if (sel == null) {
			throw new RuntimeException("Not a 'where' clause?");
		}

		FBitSet selection = new FBitSet();

		for (int i = 0; i < sel.length; i++) {
			selection.set((int) sel[i]);
		}

		dataSource.clearSelection();
		dataSource.setSelection(selection);
	}

	/**
	 * @see com.iver.cit.gvsig.gui.filter.ExpressionListener#newSet(java.lang.String)
	 */
	private long[] doSet(String expression) {
		try {
			DataSource ds = LayerFactory.getDataSourceFactory().executeSQL(expression,
					DataSourceFactory.MANUAL_OPENING);

			return ds.getWhereFilter();
		} catch (DriverLoadException e) {
			NotificationManager.addError("Error cargando el driver", e);
		} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
			NotificationManager.addError("Error accediendo al driver", e);
		} catch (ParseException e) {
			NotificationManager.addError("Parse error", e);
		} catch (SemanticException e) {
			NotificationManager.addError(e.getMessage(), e);
		} catch (IOException e) {
			NotificationManager.addError("GDBMS internal error", e);
		} catch (EvaluationException e) {
			NotificationManager.addError("Error con la expresin", e);
        }

		return null;
	}

	/**
	 * @see com.iver.cit.gvsig.gui.filter.ExpressionListener#addToSet(java.lang.String)
	 */
	public void addToSet(String expression) {
		long[] sel = doSet(expression);

		if (sel == null) {
			throw new RuntimeException("Not a 'where' clause?");
		}

		FBitSet selection = new FBitSet();

		for (int i = 0; i < sel.length; i++) {
			selection.set((int) sel[i]);
		}

		FBitSet fbs = dataSource.getSelection();
		fbs.or(selection);
		dataSource.setSelection(fbs);
	}

	/**
	 * @see com.iver.cit.gvsig.gui.filter.ExpressionListener#fromSet(java.lang.String)
	 */
	public void fromSet(String expression) {
		long[] sel = doSet(expression);

		if (sel == null) {
			throw new RuntimeException("Not a 'where' clause?");
		}

		FBitSet selection = new FBitSet();

		for (int i = 0; i < sel.length; i++) {
			selection.set((int) sel[i]);
		}

		FBitSet fbs = dataSource.getSelection();
		fbs.and(selection);
		dataSource.setSelection(fbs);
	}

	/**
	 * @see com.iver.mdiApp.plugins.IExtension#isVisible()
	 */
	public boolean isVisible() {
		IWindow v = PluginServices.getMDIManager().getActiveWindow();

		if (v == null) {
			return false;
		}

		if (v instanceof Table) {
			return true;
		} /*else {
			if (v instanceof com.iver.cit.gvsig.gui.View) {
				com.iver.cit.gvsig.gui.View view = (com.iver.cit.gvsig.gui.View) v;
				ProjectView pv = view.getModel();
				FLayer[] seleccionadas = pv.getMapContext().getLayers()
										   .getActives();

				if (seleccionadas.length == 1) {
					if (seleccionadas[0] instanceof AlphanumericData) {
						return true;
					}
				}
			}
*/
			return false;
		//}
	}

	/**
	 * @see com.iver.andami.plugins.IExtension#initialize()
	 */
	public void initialize() {
	}

	/**
	 * @see com.iver.andami.plugins.IExtension#isEnabled()
	 */
	public boolean isEnabled() {
		return true;
	}


}
