/*
 * Created on 22-jul-2004
 *
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.layout.dialogs;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.gui.layout.Layout;
import com.iver.cit.gvsig.gui.layout.commands.EditableFeatureSource;
import com.iver.cit.gvsig.gui.layout.fframes.IFFrame;
import com.iver.cit.gvsig.gui.layout.fframes.IFFrameViewDependence;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Iterator;
import java.util.TreeMap;


/**
 * Clase que hace de Listener de FAlignDialog.
 *
 * @author Vicente Caballero Navarro
 */
public class EventsFAlign implements ActionListener {
	private Layout m_layout;
	private boolean inLayout = false;

	/**
	 * Crea un nuevo FAlign.
	 *
	 * @param layout Referencia al Layout.
	 */
	public EventsFAlign(Layout layout) {
		m_layout = layout;
	}

	/**
	 * Desplaza los fframes seleccionados a la izquierda del fframe ms
	 * occidental.
	 */
	private void alignLeft() {
		double xmin = Double.MAX_VALUE;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != IFFrame.NOSELECT) {
				if (xmin > fframe.getBoundBox().getMinX()) {
					xmin = fframe.getBoundBox().getMinX();
				}
			}
		}
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = fframes.length-1; i>=0; i--) {
			IFFrame fframe = (IFFrame) fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != IFFrame.NOSELECT) {
				fframe.getBoundBox().x = xmin;
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"align_left"));
	}

	/**
	 * Desplaza los fframes seleccionados a la izquierda del Layout.
	 */
	private void alignLeftL() {
		double xmin = 0;
		IFFrame[] fframes=m_layout.getFFrames();
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().x = xmin;
				efs.modifyFFrame(fframes[i],fframe);
			}

		}
		efs.endComplexCommand(PluginServices.getText(this,"align_to_layout_left"));
	}

	/**
	 * Desplaza los fframes seleccionados al centro del fframe mas ancho de
	 * forma horizontal.
	 */
	private void alignCenterV() {
		double xcenter = 0;
		double w = Double.MIN_VALUE;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				if (w < fframe.getBoundBox().getWidth()) {
					w = fframe.getBoundBox().getWidth();
					xcenter = fframe.getBoundBox().getCenterX();
				}
			}
		}
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().x = xcenter -
					(fframe.getBoundBox().width / 2);
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"align_center"));
	}

	/**
	 * Desplaza los fframes seleccionados al centro del Layout horizontalmente.
	 */
	private void alignCenterVL() {
		double xcenter = 0;
		xcenter = m_layout.getAtributes().m_sizePaper.getAncho() / 2;
		IFFrame[] fframes=m_layout.getFFrames();
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().x = xcenter -
					(fframe.getBoundBox().width / 2);
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"align_to_layout_center"));
	}

	/**
	 * Desplaza los fframes seleccionados a la parte derecha del fframe ms
	 * oriental.
	 */
	private void alignRight() {
		double xmax = Double.MIN_VALUE;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != IFFrame.NOSELECT) {
				if (xmax < fframe.getBoundBox().getMaxX()) {
					xmax = fframe.getBoundBox().getMaxX();
				}
			}
		}
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != IFFrame.NOSELECT) {
				fframe.getBoundBox().x = xmax -
					fframes[i].getBoundBox().width;
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"align_right"));
	}

	/**
	 * Desplaza los fframes seleccionados a la parte derecha del Layout.
	 */
	private void alignRightL() {
		double xmax = 0;
		xmax = m_layout.getAtributes().m_sizePaper.getAncho();
		IFFrame[] fframes=m_layout.getFFrames();
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe =fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().x = xmax -
					fframes[i].getBoundBox().width;
				efs.modifyFFrame(fframes[i], fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"align_to_layout_right"));
	}

	/**
	 * Desplaza los fframes seleccionados a la parte inferior del fframe ms
	 * hacia abajo.
	 */
	private void alignDown() {
		double ymax = Double.MIN_VALUE;
		IFFrame[] fframes=m_layout.getFFrames();

		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				if (ymax < fframe.getBoundBox().getMaxY()) {
					ymax = fframe.getBoundBox().getMaxY();
				}
			}
		}
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().y = ymax - fframe.getBoundBox().height;
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"align_down"));
	}

	/**
	 * Desplaza los fframes seleccionados a la parte inferior del Layout.
	 */
	private void alignDownL() {
		double ymax = 0;
		ymax = m_layout.getAtributes().m_sizePaper.getAlto();
		IFFrame[] fframes=m_layout.getFFrames();
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().y = ymax - fframe.getBoundBox().height;
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"align_to_layout_down"));
	}

	/**
	 * Desplaza los fframes seleccionados a la parte superior del fframe que
	 * ms arriba este colocado.
	 */
	private void alignUp() {
		double ymin = Double.MAX_VALUE;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				if (ymin > fframe.getBoundBox().getMinY()) {
					ymin = fframe.getBoundBox().getMinY();
				}
			}
		}
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().y = ymin;
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"align_up"));
	}

	/**
	 * Desplaza los fframes seleccionados a la parte superior del Layout.
	 */
	private void alignUpL() {
		double ymin = 0;
		IFFrame[] fframes=m_layout.getFFrames();
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().y = ymin;
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"align_to_layout_up"));
	}

	/**
	 * Desplaza los fframes seleccionados al centro del fframe ms alto
	 * verticalmente.
	 */
	private void alignCenterH() {
		double ycenter = 0;
		double h = Double.MIN_VALUE;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				if (h < fframe.getBoundBox().getHeight()) {
					h = fframe.getBoundBox().getHeight();
					ycenter = fframe.getBoundBox().getCenterY();
				}
			}
		}
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().y = ycenter -
					(fframe.getBoundBox().height / 2);
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"align_vertical_center"));
	}

	/**
	 * Desplaza los fframes seleccionados en el Layout al centro verticalmente.
	 */
	private void alignCenterHL() {
		double ycenter = 0;
		ycenter = m_layout.getAtributes().m_sizePaper.getAlto() / 2;
		IFFrame[] fframes=m_layout.getFFrames();
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().y = ycenter -
					(fframe.getBoundBox().height / 2);
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"align_to_layout_vertical_center"));
	}

	/**
	 * Distribuye los fframes seleccionados de forma equidistante y vertical,
	 * de izquierda a derecha.
	 */
	private void distLeft() {
		double xmin = Double.MAX_VALUE;
		double xmax = Double.MIN_VALUE;
		int num = 0;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				if (xmin > fframe.getBoundBox().getMinX()) {
					xmin = fframe.getBoundBox().getMinX();
				}

				if (xmax < fframe.getBoundBox().getMaxX()) {
					xmax = fframe.getBoundBox().getMaxX();
				}

				num++;
			}
		}

		double dif = xmax - xmin;
		double dist = dif / num;
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().x = xmin + (dist * i);
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"distributes_left"));
	}

	/**
	 * Distribuye los fframes seleccionados en el Layout de forma equidistante
	 * y vertical, de izquierda a derecha.
	 */
	private void distLeftL() {
		double xmin = 0;
		double xmax = 0;
		xmax = m_layout.getAtributes().m_sizePaper.getAncho();

		int num = 0;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				num++;
			}
		}

		double dif = xmax - xmin;
		double dist = dif / num;
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().x = xmin + (dist * i);
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"distributes_to_layout_left"));
	}

	/**
	 * Distribuye los fframes seleccionados de forma equidistante y vertical,
	 * de derecha a izquierda.
	 */
	private void distRight() {
		double xmin = Double.MAX_VALUE;
		double xmax = Double.MIN_VALUE;
		int num = 0;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				if (xmin > fframe.getBoundBox().getMinX()) {
					xmin = fframe.getBoundBox().getMinX();
				}

				if (xmax < fframe.getBoundBox().getMaxX()) {
					xmax = fframe.getBoundBox().getMaxX();
				}

				num++;
			}
		}

		double dif = xmax - xmin;
		double dist = dif / num;
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().x = xmax - (dist * i) -
					fframe.getBoundBox().width;
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"distributes_right"));
	}

	/**
	 * Distribuye los fframes seleccionados en el Layout de forma equidistante
	 * y vertical, de derecha a izquierda.
	 */
	private void distRightL() {
		double xmin = 0;
		double xmax = 0;
		xmax = m_layout.getAtributes().m_sizePaper.getAncho();

		int num = 0;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				num++;
			}
		}

		double dif = xmax - xmin;
		double dist = dif / num;
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().x = xmax - (dist * i) -
					fframe.getBoundBox().width;
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"distributes_to_layout_right"));
	}

	/**
	 * Distribuye los fframes seleccionados de forma equidistante y vertical,
	 * desde arriba hacia abajo.
	 */
	private void distUp() {
		double ymin = Double.MAX_VALUE;
		double ymax = Double.MIN_VALUE;
		int num = 0;
		IFFrame[] fframes=m_layout.getFFrames();

		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				if (ymin > fframe.getBoundBox().getMinY()) {
					ymin = fframe.getBoundBox().getMinY();
				}

				if (ymax < fframe.getBoundBox().getMaxY()) {
					ymax = fframe.getBoundBox().getMaxY();
				}

				num++;
			}
		}

		double dif = ymax - ymin;
		double dist = dif / num;
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i <fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().y = ymin + (dist * i);
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"distributes_up"));
	}

	/**
	 * Distribuye los fframes seleccionados en el Layout de forma equidistante
	 * y vertical, desde arriba hacia abajo.
	 */
	private void distUpL() {
		double ymin = 0;
		double ymax = 0;
		ymax = m_layout.getAtributes().m_sizePaper.getAlto();

		int num = 0;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				num++;
			}
		}

		double dif = ymax - ymin;
		double dist = dif / num;
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().y = ymin + (dist * i);
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"distributes_to_layout_up"));
	}

	/**
	 * Distribuye los fframes seleccionados de forma equidistante y vertical,
	 * desde bajo hacia arriba.
	 */
	private void distDown() {
		double ymin = Double.MAX_VALUE;
		double ymax = Double.MIN_VALUE;
		int num = 0;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				if (ymin > fframe.getBoundBox().getMinY()) {
					ymin = fframe.getBoundBox().getMinY();
				}

				if (ymax < fframe.getBoundBox().getMaxY()) {
					ymax = fframe.getBoundBox().getMaxY();
				}

				num++;
			}
		}

		double dif = ymax - ymin;
		double dist = dif / num;
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().y = ymax - (dist * i) -
					fframe.getBoundBox().height;
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"distributes_down"));
	}

	/**
	 * Distribuye los fframes seleccionados en el Layout de forma equidistante
	 * y vertical, desde bajo hacia arriba.
	 */
	private void distDownL() {
		double ymin = 0;
		double ymax = 0;
		ymax = m_layout.getAtributes().m_sizePaper.getAlto();

		int num = 0;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				num++;
			}
		}

		double dif = ymax - ymin;
		double dist = dif / num;
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = fframes.length - 1; i >= 0; i--) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().y = ymax - (dist * i) -
					fframe.getBoundBox().height;
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"distributes_to_layout_down"));
	}

	/**
	 * Distribuye los fframes seleccionados de forma equidistante y vertical.
	 */
	private void distCenterH() {
		double xmin = Double.MAX_VALUE;
		double xmax = Double.MIN_VALUE;
		int num = 0;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				if (xmin > fframe.getBoundBox().getMinX()) {
					xmin = fframe.getBoundBox().getMinX();
				}

				if (xmax < fframe.getBoundBox().getMaxX()) {
					xmax = fframe.getBoundBox().getMaxX();
				}

				num++;
			}
		}

		double dif = xmax - xmin;
		double dist = dif / num;
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().x = (xmin +
					(((dist) * (i + 1)) - (dist / 2))) -
					(fframe.getBoundBox().width / 2);
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"distributes_vertical"));
	}

	/**
	 * Distribuye los fframes seleccionados en el Layout de forma equidistante
	 * y vertical.
	 */
	private void distCenterHL() {
		double xmin = 0;
		double xmax = 0;
		xmax = m_layout.getAtributes().m_sizePaper.getAncho();

		int num = 0;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				num++;
			}
		}

		double dif = xmax - xmin;
		double dist = dif / num;
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().x = (xmin +
					(((dist) * (i + 1)) - (dist / 2))) -
					(fframe.getBoundBox().width / 2);
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"distributes_to_layout_vertical"));
	}

	/**
	 * Distribuye los fframes seleccionados de forma equidistante y horizontal.
	 */
	private void distCenterV() {
		double ymin = Double.MAX_VALUE;
		double ymax = Double.MIN_VALUE;
		int num = 0;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				if (ymin > fframe.getBoundBox().getMinY()) {
					ymin = fframe.getBoundBox().getMinY();
				}

				if (ymax < fframe.getBoundBox().getMaxY()) {
					ymax = fframe.getBoundBox().getMaxY();
				}

				num++;
			}
		}

		double dif = ymax - ymin;
		double dist = dif / num;
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().y = (ymin +
					(((dist) * (i + 1)) - (dist / 2))) -
					(fframe.getBoundBox().height / 2);
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"distributes_horizontal"));
	}

	/**
	 * Distribuye los fframes seleccionados en el Layout de forma equidistante
	 * y horizontal.
	 */
	private void distCenterVL() {
		double ymin = 0;
		double ymax = 0;
		ymax = m_layout.getAtributes().m_sizePaper.getAlto();

		int num = 0;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				num++;
			}
		}

		double dif = ymax - ymin;
		double dist = dif / num;
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().y = (ymin +
					(((dist) * (i + 1)) - (dist / 2))) -
					(fframe.getBoundBox().height / 2);
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"distributes_to_layout_horizontal"));
	}

	/**
	 * Cambia la anchura de los fframes seleccionados por la anchura del ms
	 * ancho.
	 */
	private void sizeWidth() {
		double wmax = Double.MIN_VALUE;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				if (wmax < fframe.getBoundBox().getWidth()) {
					wmax = fframe.getBoundBox().getWidth();
				}
			}
		}
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().width = wmax;
			efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"change_width"));
	}

	/**
	 * Cambia la altura de los fframes seleccionados por la altura del mas
	 * alto.
	 */
	private void sizeHeight() {
		double hmax = Double.MIN_VALUE;
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				if (hmax < fframe.getBoundBox().getHeight()) {
					hmax = fframe.getBoundBox().getHeight();
				}
			}
		}
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i].cloneFFrame(m_layout);

			if (fframe.getSelected() != 0) {
				fframe.getBoundBox().height = hmax;
				efs.modifyFFrame(fframes[i],fframe);
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"change_height"));
	}

	/**
	 * Distribuye horizontalmente los fframes dejando como espacio entre ellos
	 * la media de todos los espacios.
	 */
	private void spaceRight() {
		double total = 0;
		double num = 0;
		double xmin = Double.MAX_VALUE;
		double xmax = Double.MIN_VALUE;

		TreeMap treemap = new TreeMap();
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				treemap.put((Object) new Double(fframe.getBoundBox().getMinX()),
					(Object) fframe);
				num++;
			}
		}

		Iterator j = treemap.keySet().iterator();

		if (j.hasNext()) {
			IFFrame fframeA = (IFFrame) treemap.get(j.next());
			IFFrame fframeS = null;
			xmin = fframeA.getBoundBox().x;

			while (j.hasNext()) {
				fframeS = (IFFrame) treemap.get(j.next());
				total += (fframeS.getBoundBox().getMinX() -
				fframeA.getBoundBox().getMaxX());
				fframeA = fframeS;
			}

			if (fframeS != null) {
				xmax = fframeS.getBoundBox().getMaxX();
			}
		}

		if (inLayout) {
			total += (xmin +
			(m_layout.getAtributes().m_sizePaper.getAncho() - xmax));
			num += 2;
		}

		double dist = total / (num - 1);
		Iterator k = treemap.keySet().iterator();
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		if (k.hasNext()) {
			IFFrame fframe=(IFFrame) treemap.get(k.next());
			IFFrame fframeA = fframe.cloneFFrame(m_layout);

			if (inLayout) {
				fframeA.getBoundBox().x = dist;
				efs.modifyFFrame(fframe,fframeA);
			}

			while (k.hasNext()) {
				IFFrame fframeAux=(IFFrame) treemap.get(k.next());
				IFFrame fframeS = fframeAux.cloneFFrame(m_layout);
				fframeS.getBoundBox().x = fframeA.getBoundBox().getMaxX() +
					dist;
				efs.modifyFFrame(fframeAux,fframeS);
				fframeA = fframeS;
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"horizontal_space"));
	}

	/**
	 * Distribuye verticalmente los fframes dejando como espacio entre ellos la
	 * media de todos los espacios.
	 */
	private void spaceDown() {
		double total = 0;
		double num = 0;
		double ymin = Double.MAX_VALUE;
		double ymax = Double.MIN_VALUE;

		TreeMap treemap = new TreeMap();
		IFFrame[] fframes=m_layout.getFFrames();
		for (int i = 0; i < fframes.length; i++) {
			IFFrame fframe = fframes[i];

			if (fframe.getSelected() != 0) {
				treemap.put((Object) new Double(fframe.getBoundBox().getMinY()),
					(Object) fframe);
				num++;
			}
		}

		Iterator j = treemap.keySet().iterator();

		if (j.hasNext()) {
			IFFrame fframeA = (IFFrame) treemap.get(j.next());
			ymin = fframeA.getBoundBox().y;

			IFFrame fframeS = null;

			while (j.hasNext()) {
				fframeS = (IFFrame) treemap.get(j.next());
				total += (fframeS.getBoundBox().getMinY() -
				fframeA.getBoundBox().getMaxY());
				fframeA = fframeS;
			}

			if (fframeS != null) {
				ymax = fframeS.getBoundBox().getMaxY();
			}
		}

		if (inLayout) {
			total += (ymin +
			(m_layout.getAtributes().m_sizePaper.getAlto() - ymax));
			num += 2;
		}

		double dist = total / (num - 1);
		Iterator k = treemap.keySet().iterator();
		EditableFeatureSource efs=m_layout.getEFS();
		efs.startComplexCommand();
		if (k.hasNext()) {
			IFFrame fframe=(IFFrame) treemap.get(k.next());
			IFFrame fframeA = fframe.cloneFFrame(m_layout) ;

			if (inLayout) {
				fframeA.getBoundBox().y = dist;
				efs.modifyFFrame(fframe,fframeA);
			}

			while (k.hasNext()) {
				IFFrame fframeAux= (IFFrame) treemap.get(k.next());
				IFFrame fframeS = fframeAux.cloneFFrame(m_layout);
				fframeS.getBoundBox().y = fframeA.getBoundBox().getMaxY() +
					dist;
				efs.modifyFFrame(fframeAux,fframeS);
				fframeA = fframeS;
			}
		}
		efs.endComplexCommand(PluginServices.getText(this,"vertical_space"));
	}

	//  class Listener implements ActionListener {

	/**
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		m_layout.updateFFrames();
		if (!inLayout) {
			if (e.getActionCommand() == "LEFT") {
				alignLeft();
			} else if (e.getActionCommand() == "CENTERV") {
				alignCenterV();
			} else if (e.getActionCommand() == "RIGHT") {
				alignRight();
			} else if (e.getActionCommand() == "UP") {
				alignUp();
			} else if (e.getActionCommand() == "CENTERH") {
				alignCenterH();
			} else if (e.getActionCommand() == "DOWN") {
				alignDown();
			} else if (e.getActionCommand() == "DISTUP") {
				distUp();
			} else if (e.getActionCommand() == "DISTCENTERV") {
				distCenterV();
			} else if (e.getActionCommand() == "DISTDOWN") {
				distDown();
			} else if (e.getActionCommand() == "DISTLEFT") {
				distLeft();
			} else if (e.getActionCommand() == "DISTCENTERH") {
				distCenterH();
			} else if (e.getActionCommand() == "DISTRIGHT") {
				distRight();
			} else if (e.getActionCommand() == "SIZECENTERV") {
				sizeWidth();
			} else if (e.getActionCommand() == "SIZECENTERH") {
				sizeHeight();
			} else if (e.getActionCommand() == "SIZEOTHER") {
				sizeWidth();
				m_layout.updateFFrames();
				sizeHeight();
			} else if (e.getActionCommand() == "SPACERIGHT") {
				spaceRight();
			} else if (e.getActionCommand() == "SPACEDOWN") {
				spaceDown();
			} else if (e.getActionCommand() == "INLAYOUT") {
				inLayout = true;
			}
		} else {
			if (e.getActionCommand() == "LEFT") {
				alignLeftL();
			} else if (e.getActionCommand() == "CENTERV") {
				alignCenterVL();
			} else if (e.getActionCommand() == "RIGHT") {
				alignRightL();
			} else if (e.getActionCommand() == "UP") {
				alignUpL();
			} else if (e.getActionCommand() == "CENTERH") {
				alignCenterHL();
			} else if (e.getActionCommand() == "DOWN") {
				alignDownL();
			} else if (e.getActionCommand() == "DISTUP") {
				distUpL();
			} else if (e.getActionCommand() == "DISTCENTERV") {
				distCenterVL();
			} else if (e.getActionCommand() == "DISTDOWN") {
				distDownL();
			} else if (e.getActionCommand() == "DISTLEFT") {
				distLeftL();
			} else if (e.getActionCommand() == "DISTCENTERH") {
				distCenterHL();
			} else if (e.getActionCommand() == "DISTRIGHT") {
				distRightL();
			} else if (e.getActionCommand() == "SIZECENTERV") {
				sizeWidth();
			} else if (e.getActionCommand() == "SIZECENTERH") {
				sizeHeight();
			} else if (e.getActionCommand() == "SIZEOTHER") {
				sizeWidth();
				m_layout.updateFFrames();
				sizeHeight();
			} else if (e.getActionCommand() == "SPACERIGHT") {
				spaceRight();
			} else if (e.getActionCommand() == "SPACEDOWN") {
				spaceDown();
			} else if (e.getActionCommand() == "INLAYOUT") {
				inLayout = false;
			}
		}
		m_layout.updateFFrames();
		m_layout.refresh();
	}
}
