/*
 * Created on 20-feb-2004
 *
 */
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.gui.layout.fframes;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.layers.XMLException;
import com.iver.cit.gvsig.gui.layout.Layout;
import com.iver.cit.gvsig.gui.project.SaveException;

import com.iver.utiles.XMLEntity;

import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;


/**
 * Interface que implementa FFrame.
 *
 * @author Vicente Caballero Navarro
 */
public interface IFFrame {
	public static final int N = 1;
    public static final int NE = 2;
    public static final int E = 3;
    public static final int SE = 4;
    public static final int S = 5;
    public static final int SO = 6;
    public static final int O = 7;
    public static final int NO = 8;
    public static final int RECT = 9;
    /** FFrame no selecccionado. */
    public static final int NOSELECT = 0;

    /**
     * Devuelve el boundingBox del fframe en funcin de la transformada afn
     * que se pasa como parmetro. Si se pasa como parmetro null, devuelve el
     * ltimo boundingbox que se calcul.
     *
     * @param at Transformada afn
     *
     * @return Rectngulo que representa el BoundingBox del fframe.
     */
    public Rectangle2D.Double getBoundingBox(AffineTransform at);

    /**
     * Devuelve el rectngulo que representa el fframe en centmetros.
     *
     * @return Rectngulo en centimetros.
     */
    public Rectangle2D.Double getBoundBox();

    /**
     * Rellena con el rectngulo que se pasa como parmetro el boundBox(en
     * centmetros) del fframe del cual con una transformacin se podr
     * calcular el BoundingBox (en pixels).
     *
     * @param rect Rectngulo en centmetros.
     */
    public void setBoundBox(Rectangle2D rect);

    /**
     * Este mtodo se implementa en cada una de las fframe, ya que cada una se
     * dibuja de una forma diferente sobre el graphics. Mtodo que dibuja
     * sobre el graphics que se le pasa como parmetro, segn la transformada
     * afin que se debe de aplicar y el rectngulo que se debe de dibujar.
     *
     * @param g Graphics
     * @param at Transformada afin.
     * @param r rectngulo sobre el que hacer un clip.
     * @param imgBase Imagen para acelerar el dibujado.
     *
     * @throws DriverException
     */
    public void draw(Graphics2D g, AffineTransform at, Rectangle2D r,
        BufferedImage imgBase) throws DriverException;

    /**
     * Implementado para imprimir.
     *
     * @param g Graphics2D de la impresora sobre el que dibujar.
     * @param at DOCUMENT ME!
     *
     * @throws DriverException
     */
    public void print(Graphics2D g, AffineTransform at)
        throws DriverException;

    /**
     * Devuelve el nombre que representa al fframe.
     *
     * @return String nombre del FFrame.
     */
    public String getName();

    /**
     * Rellena el String que representa al nombre del fframe.
     *
     * @param n nombre del fframe.
     */
    public void setName(String n);

    /**
     * Devuelve true, si el punto que se pasa como parmetro esta contenido
     * dentro del boundingbox del fframe.
     *
     * @param p punto a comprobar.
     *
     * @return true si el punto esta dentro del boundingbox.
     */
    public boolean contains(Point2D p);

    /**
     * Dibuja los handlers sobre el boundingBox en el graphics que se pasa como
     * parmetro.
     *
     * @param g Graphics sobre el que dibujar.
     */
    public void drawHandlers(Graphics2D g);

    /**
     * Pasando como parmetro true,  se toma como que est seleccionado el
     * fframe  y si es false como que esta sin seleccionar, de esta forma se
     * selecciona un fframe directamente sin comprobar si un punto est
     * contenido en l.
     *
     * @param b true si se quiere seleccionar y false si se quiere
     *        deseleccionar.
     */
    public void setSelected(Point2D b, MouseEvent e);

    /**
     * Establece que tipo de seleccin se realiza sobre el fframe.
     *
     * @param b punto sobre el que se debe de establecer si se selecciona o no
     *        el fframe.
     */
    public void setSelected(boolean b);

    /**
     * Devuelve un entero que representa el tipo de seleccin que se ha
     * realizado sobre el fframe.
     *
     * @return tipo de seleccin que se ha realizado.
     */
    public int getSelected();

    /**
     * Devuelve un entero que representa donde esta contenido el punto que se
     * pasa como parmetro.
     *
     * @param p punto a comparar.
     *
     * @return entero que representa como esta contenido el punto.
     */
    public int getContains(Point2D p);

    /**
     * Devuelve el Cursor adecuado segn como est contenido el punto, si es
     * para desplazamiento, o cambio de tamao.
     *
     * @param p punto a comprobar.
     *
     * @return Cursor adecuado a la posicin.
     */
    public Cursor getMapCursor(Point2D p);

    /**
     * Devuelve el rectngulo a partir del desplazamiento en el eje x y el
     * desplazamiento en el eje y.
     *
     * @param difx desplazamiento sobre el eje x.
     * @param dify desplazamiento sobre el eje y.
     *
     * @return rectngulo modificado en funcin del desplazamiento realizado.
     */
    public Rectangle2D getMovieRect(int difx, int dify);

    /**
     * Devuelve un Objeto XMLEntity con la informacin los atributos necesarios
     * para poder despus volver a crear el objeto original.
     *
     * @return XMLEntity.
     *
     * @throws XMLException
     */
    public XMLEntity getXMLEntity() throws SaveException;

    /**
     * Dibuja un rectngulo de color gris,  con el nombre en el centro y
     * escalado en forma de borrador.
     *
     * @param g Graphics sobre el que se dibuja.
     */
    public void drawDraft(Graphics2D g);

    /**
     * Actualiza el BoundBox del FFrame a partir de su rectngulo en pixels y
     * la matriz de transformacin.
     *
     * @param r Rectngulo.
     * @param at Matriz de transformacin.
     */
    public void updateRect(Rectangle2D r, AffineTransform at);

    /**
     * Devuelve true si el rectngulo primero es null o si es distinto de null
     * e intersecta.
     *
     * @param rv Rectngulo
     * @param r Rectngulo
     *
     * @return True si intersecta o es null.
     */
    public boolean intersects(Rectangle2D rv, Rectangle2D r);

    /**
     * Introduce el nmero de FFrame en el que de que se trata.
     *
     * @param i nmero de FFrame
     */
    public void setNum(int i);

    /**
     * A partir del xml y de Project inicia el objeto.
     *
     * @param xml XMLEntity
     */
    public void setXMLEntity(XMLEntity xml);

    /**
     * A partir del xml y de Project inicia el objeto.
     *
     * @param xml XMLEntity
     * @param l Project
     */
    public void setXMLEntity03(XMLEntity xml, Layout l);

    /**
     * Devuelve el nombre que representa al tipo de FFrame.
     *
     * @return nombre del elemento.
     */
    public String getNameFFrame();

    /**
     * Abre la ventana para modificar el tag.
     */
    public void openTag();

    /**
     * Rellena el tag.
     *
     * @param s valor del tag.
     */
    public void setTag(String s);

    /**
     * Devuelve el valor del tag.
     *
     * @return String del valor del tag.
     */
    public String getTag();

    /**
     * Dibuja el smbolo que indica que el FFrame contiene un tag.
     *
     * @param g Graphics sobre el que dibujar.
     */
    public void drawSymbolTag(Graphics2D g);

    /**
     * Rellena la rotacin del BoundingBox.
     *
     * @param rotation rotacin que se quiere aplicar.
     */
    public void setRotation(double rotation);

    /**
     * Devuelve la rotacin del BoundingBox.
     *
     * @return Rotacin del BoundingBox.
     */
    public double getRotation();

    /**
     * Devuelve el nivel al que se encuentra el FFrame en el Layout.
     *
     * @return nivel.
     */
    public int getLevel();

    /**
     * Inserta el nivel de dibujado del FFrame respecto del resto de fframes
     * del Layout.
     *
     * @param l nivel.
     */
    public void setLevel(int l);
    public IFFrame cloneFFrame(Layout layout);
	public Rectangle2D getLastMoveRect();
}
