/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.project;

import java.awt.Color;
import java.awt.Component;
import java.lang.reflect.Field;
import java.text.DateFormat;
import java.util.Date;

import javax.swing.JOptionPane;

import org.cresques.cts.IProjection;

import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.WindowInfo;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.ErrorEvent;
import com.iver.cit.gvsig.fmap.ErrorListener;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.layers.XMLException;
import com.iver.cit.gvsig.gui.project.OpenException;
import com.iver.cit.gvsig.gui.project.SaveException;
import com.iver.utiles.XMLEntity;


/**
 * Clase que representa una vista del proyecto
 *
 * @author Fernando Gonzlez Corts
 */
public class ProjectView extends ProjectElement implements ErrorListener {
	public static int numViews = 0;

	//public static int METROS = 0;
	//public static int KILOMETROS = 1;
	//public static int[] unidades = new int[] { METROS, KILOMETROS };
	///private Color backgroundColor = new Color(255, 255, 255);
	private MapContext mapOverViewContext;
	private MapContext mapContext;
	private int m_typeLink = 0;
	private String m_extLink;
	private String m_selectedField = null;

	/**
	 * Gets the FMap's contexts of the main map in the view.
	 *
	 * @return
	 */
	public MapContext getMapContext() {
		return mapContext;
	}

	/**
	 * Gets the FMap's context from the locator, which is the
	 * small map in the left-bottom corner of the View.
	 *
	 * @return
	 */
	public MapContext getMapOverViewContext() {
		return mapOverViewContext;
	}

	/**
	 * Crea una ProjectView, que es el modelo que hay detrs de una vista.
	 *
	 * @param viewName
	 *
	 * @return
	 */
	public static ProjectView createView(String viewName) {
		ProjectView v = new ProjectView();
		v.mapContext = new MapContext(new ViewPort(Project.getProjection()));
		v.mapOverViewContext = new MapContext(null);
		v.mapOverViewContext.setProjection(v.mapContext.getProjection());

		//v.mapOverViewContext = null;
		v.setName(viewName);
		v.setCreationDate(DateFormat.getInstance().format(new Date()));

		return v;
	}

	/**
	 * @see com.iver.cit.gvsig.project.ProjectView#setMapContext(com.iver.cit.gvsig.fmap.MapContext)
	 */
	public void setMapContext(MapContext fmap) {
		mapContext = fmap;
		fmap.addErrorListener(this);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param fmap DOCUMENT ME!
	 */
	public void setMapOverViewContext(MapContext fmap) {
		mapOverViewContext = fmap;
		mapOverViewContext.setProjection(mapContext.getProjection());
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 * @throws XMLException
	 * @throws SaveException
	 */
	public XMLEntity getXMLEntity() throws SaveException {
		XMLEntity xml = super.getXMLEntity();
		//xml.putProperty("nameClass", this.getClass().getName());
		try{
		xml.putProperty("numViews", numViews);
		xml.putProperty("m_selectedField", m_selectedField);
		xml.putProperty("m_typeLink", m_typeLink);
		xml.putProperty("m_extLink", m_extLink);
		xml.addChild(mapContext.getXMLEntity());

		if (mapOverViewContext != null) {
			if (mapOverViewContext.getViewPort() != null) {
				xml.putProperty("mapOverView", true);
				xml.addChild(mapOverViewContext.getXMLEntity());
			} else {
				xml.putProperty("mapOverView", false);
			}
		} else {
			xml.putProperty("mapOverView", false);
		}
		XMLEntity viewProperties = this.getViewInfoXMLEntity();
		if (viewProperties!=null) { //store the properties of the window
			xml.addChild(viewProperties);
		}
		}catch (Exception e) {
			throw new SaveException(e,this.getClass().getName());
		}
		return xml;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param xml DOCUMENT ME!
	 * @param p DOCUMENT ME!
	 * @throws XMLException
	 * @throws DriverException
	 * @throws DriverIOException
	 *
	 * @see com.iver.cit.gvsig.project.ProjectElement#setXMLEntity(com.iver.utiles.XMLEntity)
	 */
	public void setXMLEntity03(XMLEntity xml, Project p)
		throws XMLException, DriverException, DriverIOException {
		numViews = xml.getIntProperty("numViews");
		m_selectedField = xml.getStringProperty("m_selectedField");
		m_typeLink = xml.getIntProperty("m_typeLink");
		m_extLink = xml.getStringProperty("m_extLink");
		setMapContext(MapContext.createFromXML03(xml.getChild(0)));

		if (xml.getBooleanProperty("mapOverView")) {
			setMapOverViewContext(MapContext.createFromXML03(xml.getChild(1)));
		}

	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param xml DOCUMENT ME!
	 * @param p DOCUMENT ME!
	 * @throws XMLException
	 * @throws DriverException
	 * @throws DriverIOException
	 * @throws OpenException
	 *
	 * @see com.iver.cit.gvsig.project.ProjectElement#setXMLEntity(com.iver.utiles.XMLEntity)
	 */
	public void setXMLEntity(XMLEntity xml, Project p)
		throws XMLException, DriverException, DriverIOException, OpenException {
		try{
			int currentChild=0;
			numViews = xml.getIntProperty("numViews");
			m_selectedField = xml.getStringProperty("m_selectedField");
			m_typeLink = xml.getIntProperty("m_typeLink");
			m_extLink = xml.getStringProperty("m_extLink");

			setMapContext(MapContext.createFromXML(xml.getChild(currentChild)));
			currentChild++;
			if (xml.getBooleanProperty("mapOverView")) {
				setMapOverViewContext(MapContext.createFromXML(xml.getChild(currentChild)));
				currentChild++;
			}
			if (currentChild<xml.getChildrenCount()) {
				XMLEntity child = xml.getChild(currentChild);
				if (child.contains("className") && child.getStringProperty("className").equals(this.getClass().getName()) && child.contains("name") && child.getStringProperty("name").equals("ViewInfoProperties")) {
					seedViewInfo = createViewInfoFromXMLEntity(child);
					//currentChild++;
				}
			}
			showErrors();
		}catch (Exception e) {
			throw new OpenException(e,this.getClass().getName());
		}
	}

	public void showErrors(){
		if (mapContext.getLayersError().size()>0){
			String layersError="";
			for (int i=0;i<mapContext.getLayersError().size();i++){
				layersError=layersError+"\n"+(String)mapContext.getLayersError().get(i);
			}
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this,"fallo_capas")+" : \n"+
					layersError);
		}
	}
	/**
	 * DOCUMENT ME!
	 *
	 * @param p DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 * @throws XMLException
	 * @throws DriverException
	 * @throws DriverIOException
	 * @throws OpenException
	 */
	/*public ProjectView cloneProjectView(Project p)
		throws XMLException, DriverException, DriverIOException, OpenException {
		return (ProjectView) createFromXML(getXMLEntity(), p);
	}
*/
	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public IProjection getProjection() {
		return mapContext.getProjection();
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public IProjection getOverViewProjection() {
		return mapOverViewContext.getProjection();
	}

	public void setProjection (IProjection proj) {
		mapContext.setProjection(proj);
		mapOverViewContext.setProjection(proj);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getExtLink() {
		return m_extLink;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public int getTypeLink() {
		return m_typeLink;
	}

	/**
     * Se selecciona la extensin para realizar cuando se quiera el link.
     *
     * @param s nombre del campo.
     */
	public void setExtLink(String s) {
		m_extLink = s;
	}

	/**
     * Se selecciona el tipo de fichero para realizar cuando se quiera el link.
     *
     * @param i tipo de fichero.
     */
	public void setTypeLink(int i) {
		m_typeLink = i;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param c DOCUMENT ME!
	 */
	public void setBackColor(Color c) {
//		getMapContext().getViewPort().addViewPortListener(getMapContext()
	//													   .getEventBuffer());
		getMapContext().getViewPort().setBackColor(c);
		//getMapContext().getViewPort().removeViewPortListener(getMapContext()
			//												  .getEventBuffer());
	}

	/**
	 * Se selecciona el nombre del campo para realizar cuando se quiera el
	 * link.
	 *
	 * @param s nombre del campo.
	 */
	public void setSelectedField(String s) {
		m_selectedField = s;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getSelectedField() {
		return m_selectedField;
	}

	public void errorThrown(ErrorEvent e) {
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this,"fallo_capas")+" : \n"+
					e.getMessage());

	}

//	public int computeSignature() {
//		int result = 17;
//
//		Class clazz = getClass();
//		Field[] fields = clazz.getDeclaredFields();
//		for (int i = 0; i < fields.length; i++) {
//			try {
//				String type = fields[i].getType().getName();
//				if (type.equals("boolean")) {
//					result += 37 + ((fields[i].getBoolean(this)) ? 1 : 0);
//				} else if (type.equals("java.lang.String")) {
//					Object v = fields[i].get(this);
//					if (v == null) {
//						result += 37;
//						continue;
//					}
//					char[] chars = ((String) v).toCharArray();
//					for (int j = 0; j < chars.length; j++) {
//						result += 37 + (int) chars[i];
//					}
//				} else if (type.equals("byte")) {
//					result += 37 + (int) fields[i].getByte(this);
//				} else if (type.equals("char")) {
//					result += 37 + (int) fields[i].getChar(this);
//				} else if (type.equals("short")) {
//					result += 37 + (int) fields[i].getShort(this);
//				} else if (type.equals("int")) {
//					result += 37 + fields[i].getInt(this);
//				} else if (type.equals("long")) {
//					long f = fields[i].getLong(this) ;
//					result += 37 + (f ^ (f >>> 32));
//				} else if (type.equals("float")) {
//					result += 37 + Float.floatToIntBits(fields[i].getFloat(this));
//				} else if (type.equals("double")) {
//					long f = Double.doubleToLongBits(fields[i].getDouble(this));
//					result += 37 + (f ^ (f >>> 32));
//				} else {
//					Object obj = fields[i].get(this);
//					result += 37 + ((obj != null)? obj.hashCode() : 0);
//				}
//			} catch (Exception e) { e.printStackTrace(); }
//
//		}
//		return result;
//	}
}
