package es.prodevelop.cit.gvsig.arcims.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;

import es.prodevelop.cit.gvsig.arcims.fmap.layers.FRasterLyrArcIMS;
import es.prodevelop.cit.gvsig.arcims.gui.panels.LayerScaleDrawPanel;

import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.WindowInfo;

/**
 * This class shows the ArcIMS layer's scale limits status.
 * 
 * @author jldominguez
 */
public class LayerScaleDialog extends JPanel implements ActionListener, IWindow {

	private JPanel button_p;
	private JButton b;
	private LayerScaleDrawPanel dp;
	private FRasterLyrArcIMS layer;
	private WindowInfo theViewInfo;
	private JScrollPane sp;
	private static final long serialVersionUID = 0;


	/**
	 * Needs the layer as parameter.
	 * 
	 * @param lyr the layer
	 */
	public LayerScaleDialog(FRasterLyrArcIMS lyr) {
		super();
		layer = lyr;
		lyr.addActionlistener(this);
		Vector infoV = layer.getLayerScaleInfoVector();
		int spHeight = 201 + 15 * infoV.size();
		if (spHeight > 500) spHeight = 500;
		
		setLayout(new BorderLayout());
		setSize(140 + 680, spHeight + 10);
		setMinimumSize(new Dimension(410, 226));
		
		b = new JButton(PluginServices.getText(this, "close"));
		b.setBounds(10, 10, 90, 25);
		b.addActionListener(this);
		
		button_p = new JPanel();
		button_p.add(b);
		
		dp = new LayerScaleDrawPanel(infoV, this);
		dp.setDpi(lyr.getArcimsStatus().getServiceInfo().getScreen_dpi());
		dp.setCurrentScale(1.0 * layer.getMapContext().getScaleView());
		
		dp.setPreferredSize(new Dimension(300, 200 + 15 * infoV.size()));

		sp = new JScrollPane(dp);
		sp.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		sp.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		
		add(BorderLayout.CENTER, sp);
		add(BorderLayout.SOUTH, button_p);
	}
	

	public void actionPerformed(ActionEvent arg0) {
		
		if (arg0.getActionCommand().compareToIgnoreCase(FRasterLyrArcIMS.SCALE_CHANGED_COMMAND) == 0) {
			dp.setCurrentScale(1.0 * layer.getScale());
			dp.repaint();
			return;
		}
		
		if (arg0.getSource() == b) {
			this.close();
		}

	}
	
	public void close() {
		PluginServices.getMDIManager().closeWindow(this);
	}

	public WindowInfo getWindowInfo() {
		if (theViewInfo == null) {
			theViewInfo = new WindowInfo(
					0 + // palette
					16 + // no modal
					0 + // modal
					4 + // iconifiable
					2 + // maximizable
					1 // resizable
					);
			theViewInfo.setTitle(
					PluginServices.getText(this, "layer") +
					": " + 
					layer.getName() + 
					" - " + 
					PluginServices.getText(this, "layer_scale_status")
					);
			theViewInfo.setWidth(getInitialWidth());
			theViewInfo.setHeight(getInitialHeight());
		}
		return theViewInfo;
	}

	public FRasterLyrArcIMS getLayer() {
		return layer;
	}

	public void setLayer(FRasterLyrArcIMS layer) {
		this.layer = layer;
	}
	

	private int getInitialWidth() {
		return 365;
	}

	private int getInitialHeight() {
		return 290;
	}

}
