/*
 * Created on 29-jun-2006
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
*
* $Id: UnitUtils.java,v 1.3 2006/08/29 07:56:30 cesar Exp $
* $Log: UnitUtils.java,v $
* Revision 1.3  2006/08/29 07:56:30  cesar
* Rename the *View* family of classes to *Window* (ie: SingletonView to SingletonWindow, ViewInfo to WindowInfo, etc)
*
* Revision 1.2  2006/08/29 07:21:09  cesar
* Rename com.iver.cit.gvsig.fmap.Fmap class to com.iver.cit.gvsig.fmap.MapContext
*
* Revision 1.1  2006/06/29 17:30:36  azabala
* first version in cvs
*
*
*/
package com.iver.cit.gvsig.geoprocess.core.util;

import org.cresques.cts.IProjection;
import org.cresques.cts.gt2.CSUTM;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.MapContext;
import com.iver.cit.gvsig.gui.View;

/**
 * Utility methods to manage distances
 * (conversion from user entries to internal units,
 * verifying if projection is planar or we are 
 * working with geographic coordinates, etc.)
 * @author azabala
 *
 */
public class UnitUtils {
	
	public static final double EARTH_RADIUS = 6378137d;

	/**
	 * For computing with geodetic coordinates:
	 * returns the angular measure (in radians)
	 * for a distance over the earth along a 
	 * meridiam.
	 * Because this consideration is an approximation,
	 * we consideer the eart like an sphere (not an
	 * ellipsoid)
	 * @param dist distance in meters
	 * @return arc of meridian whose length is the specified
	 * distance
	 */
	public static double toSexaAngularMeasure(double dist){
		/*
		 *  dist = 6378km(terrestrial radius) -> 2PI
		 *  passed distance -> incognite
		 */
		return Math.toDegrees((2 * Math.PI * dist)/EARTH_RADIUS);
	}
	
	
	public static double getInInternalUnits(double userEntryDistance){
		int mapUnits = ((View)PluginServices.
				getMDIManager().
				getActiveWindow()).getMapControl().getViewPort().getMapUnits();		
		double distInInternalUnits = userEntryDistance * MapContext.CHANGEM[mapUnits];
		/*
		 * if layer's projections is in geographics
		coords, pass distance to a angular measure
		*/
		IProjection proj =  ((View)PluginServices.
								getMDIManager().
								getActiveWindow()).
								getMapControl().
								getViewPort().
								getProjection();
		if( (proj != null) && !(proj instanceof CSUTM)){
			distInInternalUnits =
				UnitUtils.toSexaAngularMeasure(distInInternalUnits);	
		}
		return distInInternalUnits;
	}
	
	
	
	
	
	
}

