/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing;

import java.util.logging.Logger;


import com.iver.andami.PluginServices;
import com.iver.andami.plugins.Extension;
import com.iver.cit.gvsig.LayoutControls;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrGeoRaster;
import com.iver.cit.gvsig.gui.View;
import com.iver.cit.gvsig.gui.layout.Layout;


/**
 * Extensin encargada de cargar la herramientas de redimensin, movimiento 
 * de imagen, movimiento de puntos y cargar extent anterior. 
 *
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class GeoreferencingToolsModule extends Extension {
	
	//**********************Vars****************************************
	private static Logger 		logger = Logger.getLogger(LayoutControls.class.getName());
	private Layout 				layout = null;
	public static boolean 		enabled = true;
	//**********************End Vars************************************
	
	//**********************Methods*************************************
	/**
	 * El control est activo siempre que haya una vista abierta
	 */
	public boolean isEnabled() {
		if(enabled)
			return true;
		else
			return false;
	}

	/**
	 * El control es visible siempre que haya una vista abierta
	 */
	public boolean isVisible() {
		com.iver.andami.ui.mdiManager.IWindow f = null;
		try{
			f = PluginServices.getMDIManager().getActiveWindow();
		}catch(ClassCastException exc){
			return false;
		}
				
		if (f == null) {
			return false;
		}
		
		try{
			for(int i=0;i<((View)f).getMapControl().getMapContext().getLayers().getLayersCount();i++){
				FLayer lyr = ((View)f).getMapControl().getMapContext().getLayers().getLayer(i);
				if(	lyr instanceof FLyrGeoRaster &&
					lyr.getName().startsWith("*") &&
					lyr.isActive())
					return true;
			}
		}catch(ClassCastException ex){
			return false;
		}
		
		return false;
	}

	/**
	 * Obtiene la capa de georreferenciacin
	 * @param theView Vista de gvSIG
	 * @return capa de georreferenciacin
	 */
	private FLyrGeoRaster getLayer(View theView){
		FLyrGeoRaster lyrGeoRaster = null;
		for(int i=0;i<theView.getMapControl().getMapContext().getLayers().getLayersCount();i++){
			FLayer lyr = theView.getMapControl().getMapContext().getLayers().getLayer(i);
			if(lyr instanceof FLyrGeoRaster && lyr.getName().startsWith("*"))
				lyrGeoRaster = (FLyrGeoRaster)lyr;
		}
		return lyrGeoRaster;
	}
	
	/**
	 * Cuando ejecutamos alguna herramienta de georeferenciacin seleccionamos
	 * la que ha sido activada.
	 */
	public void execute(String s) {
		View theView = null;
		try{
			theView = (View) PluginServices.getMDIManager().getActiveWindow();
		}catch(ClassCastException exc){
			return;
		}
		MapControl mapCtrl = theView.getMapControl();
		
		if (s.compareTo("GEO_PAN") == 0) {
			mapCtrl.setTool("geoPan");
		} else if (s.compareTo("GEO_ZOOM") == 0) {
			mapCtrl.setTool("geoZoom");
		} else if (s.compareTo("GEO_ZOOM_PREV") == 0) {
			//Obtenemos la capa
			FLyrGeoRaster lyrGeoRaster = getLayer(theView);
			
			//Asignamos el ltimo extent
			if(lyrGeoRaster != null){
				lyrGeoRaster.setLastExtent();
				mapCtrl.getMapContext().invalidate();
			}
		} else if (s.compareTo("GEO_MOVE_POINT") == 0){ 
			mapCtrl.setTool("geoMovePoint");
		} else if (s.compareTo("GEO_ZOOM_NEXT") == 0) {			
			//Obtenemos la capa
			FLyrGeoRaster lyrGeoRaster = getLayer(theView);
						
			//Asignamos el extent anterior
			if(lyrGeoRaster != null){
				lyrGeoRaster.setNextExtent();
				mapCtrl.getMapContext().invalidate();
			}
		}
	}

	/**
	 * @see com.iver.andami.plugins.IExtension#initialize()
	 */
	public void initialize() {
		
	}
	
	/**
	 * Activa o desactiva la barra de herramientas de georreferenciacin
	 * @param enabled true para activar y false para desactivar
	 */
	public static void setEnabled(boolean enabled){
		if(PluginServices.getExtension(GeoreferencingToolsModule.class) != null){
			GeoreferencingToolsModule.enabled = enabled;
			PluginServices.getMainFrame().enableControls();
		}
	}
	//**********************End Methods**********************************
}