/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.gui.options;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JTable;

import org.gvsig.georeferencing.gui.dialog.GeoreferencingDialog;
import org.gvsig.georeferencing.gui.options.TableOptions.TableModelPoint;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.layers.FLyrPoints;
import com.iver.cit.gvsig.gui.View;
import java.awt.FlowLayout;

/**
 * Panel que contiene opciones de georreferenciacin a elegir por
 * el usuario. Esta clase no maneja eventos.
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class GeorefTableOptionsPanel extends JPanel implements ActionListener{

	private GeoreferencingDialog grd = null;
	
	private JPanel 				jPanel = null;
	private TableOptions 	tableOptions = null;
	
	/**
	 * This is the default constructor
	 */
	public GeorefTableOptionsPanel(GeoreferencingDialog grd) {
		super();
		this.grd = grd;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
        this.setLayout(new FlowLayout());
        this.setPreferredSize(new java.awt.Dimension(380,160));
        this.setBounds(0, 0, 380, 160);
        this.add(getJTableOptions(), java.awt.BorderLayout.NORTH);
	}

		
	public TableOptions getJTableOptions(){
		if(tableOptions == null){
			tableOptions = new TableOptions();
			tableOptions.setPreferredSize(new java.awt.Dimension(380,160));
			tableOptions.setEntry(true, PluginServices.getText(null, "mostrar_puntos"));
			tableOptions.setEntry(false, PluginServices.getText(null, "crear_worldfile"));
			tableOptions.setEntry(true, PluginServices.getText(null, "errores_csv"));
			tableOptions.setEntry(true, PluginServices.getText(null, "desactivar_capa"));
			tableOptions.setEntry(true, PluginServices.getText(null, "centrar_punto"));
			tableOptions.setEntry(true, PluginServices.getText(null, "desactivar_puntos"));
		}
		return tableOptions;
	}
	
	/**
	 * Accin cuando se pulsa el botn de aceptar en el dialogo.
	 * <UL>
	 * <LI>Cambiamos el nombre a la capa georraster</LI>
	 * <LI>A la capa Georraster le asignamos la capa de puntos para poder recuperarla</LI>
	 * <LI>Cerramos la ventana</LI>
	 * </UL>
	 */
	public void actionPerformed(java.awt.event.ActionEvent e) {
		if(e.getSource() instanceof JCheckBox){
			if(((JCheckBox)e.getSource()).getName().equals("mostrar_puntos")){
				FLyrPoints lyrPoints = grd.getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel().getLyrPoints();
				View theView = null;
				try{
					theView = (View) PluginServices.getMDIManager().getActiveWindow();
				}catch(ClassCastException exc){
					return;
				}
								
				if(((JCheckBox)e.getSource()).isSelected()){ //Mostramos el nmero de punto
					grd.getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel().getLyrPoints().setShowNumber(true);
					grd.getConectorPanel().getAdjustGeorefPanel().getZoomLeft().draw();
					grd.getConectorPanel().getAdjustGeorefPanel().getZoomRight().draw();
					theView.getMapControl().getMapContext().invalidate();
				}else{ //Ocultamos el nmero de punto
					grd.getConectorPanel().getDataPointsTabPanel().getSelectPointsPanel().getLyrPoints().setShowNumber(false);
					grd.getConectorPanel().getAdjustGeorefPanel().getZoomLeft().draw();
					grd.getConectorPanel().getAdjustGeorefPanel().getZoomRight().draw();
					theView.getMapControl().getMapContext().invalidate();
				}
			}
			
			if(((JCheckBox)e.getSource()).getName().equals("crear_worldfile")){
				if(((JCheckBox)e.getSource()).isSelected())
					grd.setCreateWorldFile(true);
				else
					grd.setCreateWorldFile(false);
			}
			
			if(((JCheckBox)e.getSource()).getName().equals("errores_csv")){
				if(((JCheckBox)e.getSource()).isSelected())
					grd.setErrorCSV(true);
				else
					grd.setErrorCSV(false);
			}
			
			if(((JCheckBox)e.getSource()).getName().equals("desactivar_capa")){
				if(((JCheckBox)e.getSource()).isSelected())
					grd.setOffLayer(true);
				else
					grd.setOffLayer(false);
			}
			
			if(((JCheckBox)e.getSource()).getName().equals("centrar_punto")){
				if(((JCheckBox)e.getSource()).isSelected())
					grd.setCenterPoint(true);
				else
					grd.setCenterPoint(false);
					
			}
			
			if(((JCheckBox)e.getSource()).getName().equals("desactivar_puntos")){
				if(((JCheckBox)e.getSource()).isSelected())
					grd.setOffPoints(true);
				else
					grd.setOffPoints(false);
					
			}
		}
	}

	
	/*private JPanel getJPanel11() {
		if (jPanel11 == null) {
			jPanel11 =  new TableOptionsPanel(this.grd, "mostrar_puntos");
			((TableOptionsPanel)jPanel11).getCbCheckBox().addActionListener(this);
		}
		return jPanel11;
	}

	private JPanel getJPanel12() {
		if (jPanel12 == null) {
			jPanel12 =  new TableOptionsPanel(this.grd, "crear_worldfile");
			((TableOptionsPanel)jPanel12).getCbCheckBox().setSelected(false);
			((TableOptionsPanel)jPanel12).getCbCheckBox().addActionListener(this);
		}
		return jPanel12;
	}

	private JPanel getJPanel13() {
		if (jPanel13 == null) {
			jPanel13 =  new TableOptionsPanel(this.grd, "errores_csv");
			((TableOptionsPanel)jPanel13).getCbCheckBox().setSelected(true);
			((TableOptionsPanel)jPanel13).getCbCheckBox().addActionListener(this);
		}
		return jPanel13;
	}

	private JPanel getJPanel14() {
		if (jPanel14 == null) {
			jPanel14 =  new TableOptionsPanel(this.grd, "desactivar_capa");
			((TableOptionsPanel)jPanel14).getCbCheckBox().setSelected(true);
			((TableOptionsPanel)jPanel14).getCbCheckBox().addActionListener(this);
		}
		return jPanel14;
	}
	
	private JPanel getJPanel15() {
		if (jPanel15 == null) {
			jPanel15 =  new TableOptionsPanel(this.grd, "centrar_punto");
			((TableOptionsPanel)jPanel15).getCbCheckBox().setSelected(true);
			((TableOptionsPanel)jPanel15).getCbCheckBox().addActionListener(this);
		}
		return jPanel15;
	}
	
	private JPanel getJPanel16() {
		if (jPanel16 == null) {
			jPanel16 =  new TableOptionsPanel(this.grd, "desactivar_puntos");
			((TableOptionsPanel)jPanel16).getCbCheckBox().setSelected(true);
			((TableOptionsPanel)jPanel16).getCbCheckBox().addActionListener(this);
		}
		return jPanel16;
	}*/
 }
