/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.gui.selectPoints;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.iver.andami.PluginServices;

/**
 * Panel que contiene la informacin de cada punto. 
 * No contiene eventos, estos deben ser manejados desde la 
 * clase que lo llame.
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class DataPointPanel extends JPanel{
	
	//**********************Vars**********************************
	private JPanel pInfoPoint = null;
	private JPanel pX = null;
	private JPanel pY = null;
	private JPanel pLatitud = null;
	private JPanel pLongitud = null;
	private JTextField tX = null;
	private JLabel lX = null;
	private JLabel lY = null;
	private JTextField tY = null;
	private JLabel lLongitud = null;
	private JLabel lLatitud = null;
	private JTextField tLatitud = null;
	private JTextField tLongitud = null;
	//**********************End Vars******************************
	private JPanel jPanel = null;
	private JCheckBox cbActive = null;
	
	//**********************Methods*******************************
	/**
	 * This is the default constructor
	 */
	public DataPointPanel() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {                
		GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
		gridBagConstraints6.insets = new java.awt.Insets(0,0,0,0);
		gridBagConstraints6.gridy = 4;
		gridBagConstraints6.gridx = 0;
		GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
		gridBagConstraints5.insets = new java.awt.Insets(0,0,0,0);
		gridBagConstraints5.gridy = 3;
		gridBagConstraints5.weighty = 0.0D;
		gridBagConstraints5.gridx = 0;
		GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
		gridBagConstraints4.insets = new java.awt.Insets(0,0,0,0);
		gridBagConstraints4.gridy = 2;
		gridBagConstraints4.weighty = 0.0D;
		gridBagConstraints4.gridx = 0;
		GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
		gridBagConstraints3.insets = new java.awt.Insets(0,0,0,0);
		gridBagConstraints3.gridy = 1;
		gridBagConstraints3.ipadx = 0;
		gridBagConstraints3.gridx = 0;
		
		this.setLayout(new GridBagLayout());
		this.setBorder(javax.swing.BorderFactory.createTitledBorder(null, PluginServices.getText(this, "punto"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
		this.setSize(new java.awt.Dimension(190,125));
		this.setPreferredSize(new java.awt.Dimension(190,125));
		this.add(getPX(), gridBagConstraints3);
		this.add(getPY(), gridBagConstraints4);
		this.add(getPLatitud(), gridBagConstraints5);
		this.add(getPLongitud(), gridBagConstraints6);
		this.add(getJPanel(), new GridBagConstraints());
	}
	
	/**
	 * Esta funcin resetea los controles del panel de info de un punto.
	 */
	public void resetControls(String value){
		String data = "";
		if(value != null)
			data = value;
		getTX().setText(data);
		getTY().setText(data);
		getLatitud().setText(data);
		getLongitud().setText(data);
		getCbActive().setSelected(true);
	}
	//**********************End Methods***************************
	
	//**********************Setters & Getters*********************
	/**
	 * This method initializes pX	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPX() {
		if (pX == null) {
			lX = new JLabel();
			lX.setText("X: ");
			FlowLayout flowLayout2 = new FlowLayout();
			flowLayout2.setAlignment(java.awt.FlowLayout.RIGHT);
			pX = new JPanel();
			pX.setLayout(flowLayout2);
			pX.setPreferredSize(new java.awt.Dimension(170,20));
			flowLayout2.setVgap(1);
			flowLayout2.setHgap(0);
			pX.add(lX, null);
			pX.add(getTX(), null);
		}
		return pX;
	}

	/**
	 * This method initializes pY	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPY() {
		if (pY == null) {
			lY = new JLabel();
			lY.setText("Y:");
			FlowLayout flowLayout1 = new FlowLayout();
			flowLayout1.setAlignment(java.awt.FlowLayout.RIGHT);
			pY = new JPanel();
			pY.setLayout(flowLayout1);
			pY.setPreferredSize(new java.awt.Dimension(170,20));
			flowLayout1.setHgap(0);
			flowLayout1.setVgap(1);
			pY.add(lY, null);
			pY.add(getTY(), null);
		}
		return pY;
	}

	/**
	 * This method initializes pLatitud	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPLatitud() {
		if (pLatitud == null) {
			lLongitud = new JLabel();
			lLongitud.setText("longitud:");
			FlowLayout flowLayout3 = new FlowLayout();
			flowLayout3.setAlignment(java.awt.FlowLayout.RIGHT);
			pLatitud = new JPanel();
			pLatitud.setLayout(flowLayout3);
			pLatitud.setPreferredSize(new java.awt.Dimension(170,20));
			flowLayout3.setHgap(0);
			flowLayout3.setVgap(1);
			pLatitud.add(lLongitud, null);
			pLatitud.add(getLongitud(), null);
		}
		return pLatitud;
	}

	/**
	 * This method initializes pLongitud	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPLongitud() {
		if (pLongitud == null) {
			lLatitud = new JLabel();
			lLatitud.setText(PluginServices.getText(this, "latitud")+":");
			FlowLayout flowLayout4 = new FlowLayout();
			flowLayout4.setAlignment(java.awt.FlowLayout.RIGHT);
			pLongitud = new JPanel();
			pLongitud.setLayout(flowLayout4);
			pLongitud.setPreferredSize(new java.awt.Dimension(170,20));
			flowLayout4.setHgap(0);
			flowLayout4.setVgap(1);
			pLongitud.add(lLatitud, null);
			pLongitud.add(getLatitud(), null);
		}
		return pLongitud;
	}
	
	/**
	 * Este mtodo inicializa el campo de texto que tiene la coordenada en X 
	 * de la vista. Controla que le sean introducidos valores numericos y
	 * salva el valor que contiene en la capa.	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	public JTextField getTX() {
		if (tX == null) {
			tX = new JTextField();
			tX.setPreferredSize(new java.awt.Dimension(90,18));
			tX.setHorizontalAlignment(javax.swing.JTextField.LEADING);
		}
		return tX;
	}

	/**
	 * Este mtodo inicializa el campo de texto que tiene la coordenada en Y 
	 * de la vista. Controla que le sean introducidos valores numericos y
	 * salva el valor que contiene en la capa.	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	public JTextField getTY() {
		if (tY == null) {
			tY = new JTextField();
			tY.setPreferredSize(new java.awt.Dimension(90,18));
		}
		return tY;
	}

	/**
	 * Este mtodo inicializa el campo de texto que tiene la coordenada en Y 
	 * del mundo real. Controla que le sean introducidos valores numericos y
	 * salva el valor que contiene en la capa.	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	public JTextField getLatitud() {
		if (tLatitud == null) {
			tLatitud = new JTextField();
			tLatitud.setPreferredSize(new java.awt.Dimension(111,18));
			tLatitud.setHorizontalAlignment(javax.swing.JTextField.LEFT);
		}
		return tLatitud;
	}

	/**
	 * Este mtodo inicializa el campo de texto que tiene la coordenada en X 
	 * del mundo real. Controla que le sean introducidos valores numericos y
	 * salva el valor que contiene en la capa.
	 * 	
	 * @return javax.swing.JTextField	
	 */
	public JTextField getLongitud() {
		if (tLongitud == null) {
			tLongitud = new JTextField();
			tLongitud.setPreferredSize(new java.awt.Dimension(111,18));
			tLongitud.setHorizontalAlignment(javax.swing.JTextField.LEFT);
		}
		return tLongitud;
	}	
	//**********************End Setters & Getters*****************

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			FlowLayout flowLayout = new FlowLayout();
			flowLayout.setVgap(0);
			flowLayout.setAlignment(java.awt.FlowLayout.LEFT);
			flowLayout.setHgap(0);
			jPanel = new JPanel();
			jPanel.setName("activo");
			jPanel.setLayout(flowLayout);
			jPanel.setPreferredSize(new java.awt.Dimension(170,19));
			jPanel.add(getCbActive(), null);
		}
		return jPanel;
	}

	/**
	 * This method initializes jCheckBox	
	 * 	
	 * @return javax.swing.JCheckBox	
	 */
	public JCheckBox getCbActive() {
		if (cbActive == null) {
			cbActive = new JCheckBox();
			cbActive.setText("activo");
			cbActive.setSelected(true);
		}
		return cbActive;
	}
	
	/**
     * Activa o desactiva este panel y todos los que lo componen
     * @param enabled variable booleana para la activacin y/o desactivacin
     */
    public void setEnabled(boolean enabled){
    	Color color = Color.WHITE;
    	if(!enabled)
    		color = this.getBackground();
    	this.getTX().setBackground(color);
    	this.getTX().setEnabled(enabled);
    	this.getTY().setBackground(color);
    	this.getTY().setEnabled(enabled);
    	this.getLatitud().setBackground(color);
    	this.getLatitud().setEnabled(enabled);
    	this.getLongitud().setBackground(color);
    	this.getLongitud().setEnabled(enabled);
    }
}
