/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.gvsig.rasterTools.saveRaster.ui.main;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.geom.Rectangle2D;
import java.io.File;

import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.cresques.cts.IProjection;
import org.cresques.io.GeoRasterWriter;
import org.cresques.ui.DefaultDialogPanel;
import org.gvsig.rasterTools.saveRaster.ui.listener.DataInputListener;
import org.gvsig.rasterTools.saveRaster.ui.properties.ProgressSaveRasterDialog;
import org.gvsig.rasterTools.saveRaster.ui.properties.SaveRasterPropsDialog;
import org.gvsig.rasterTools.saveRaster.util.DriverProperties;

import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.WindowInfo;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.layers.FLayers;


/**
 * Panel que contiene los botones de Aceptar, Cancelar y Aplicar heredando de
 * DefaultDialogPanel. Dentro de este estar el panel con los controles de
 * salvar a raster.
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class SaveRasterDefaultPanel extends DefaultDialogPanel implements IWindow {
    final private static long 			serialVersionUID = -3370601314380922368L;
    private DataInputListener			listener =  null;
    private GeoRasterWriter				writer=null;
	private IProjection 				currentProjection = null;
	private SaveRasterPropsDialog 		frameSaveRasterProps = null;
	private int 						compresion = 1;
	private int 						sizeBlock = 64;
	public DriverProperties				driverProps = null;
	private FLayers				 		layers = null;
	private MapControl					mapCtrl = null;

	/**
	 * Constructor de la ventana de dialogo para gvSIG.
	 */
	public SaveRasterDefaultPanel(FLayers layers, MapControl mapCtrl) {
		super(false);
		initialize();
		driverProps = new DriverProperties();
		this.layers = layers;
		this.mapCtrl = mapCtrl;
		init();
	}
	
	/**
	 * Constructor generico para poder visualizar la ventana desde
	 * una funcin main.
	 */
    public SaveRasterDefaultPanel() {
        super(false);
		initialize();
        init();
    }

    
	/**
     * Inicializacin del panel.
     * @return void
     */
    public void init() {
    	super.initialize();
    	//this.setSize(410, 380);
        this.setSize(new java.awt.Dimension(410,390));
        this.pButton.setPreferredSize(new java.awt.Dimension(390, 25));
    	this.getListener();
    	
        //Ocultamos el botn de aplicar
        this.setLocation(new java.awt.Point(0,0));
        this.getApplyButton().setVisible(false);
        this.getAcceptButton().setEnabled(false);
        
        setName("saveRaster");
		
	 	//this.getAcceptButton().setEnabled(false);
	 	this.getAcceptButton().addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	if(acceptButtonActionPerformed(evt))
                	closeJDialog();
            }
        });
	 	this.getCancelButton().addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeJDialog();
            }
        });
	 	this.getApplyButton().addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	acceptButtonActionPerformed(evt);
             }
        });
		
		//Captura de eventos para el botn de propiedades
		
		((SaveRasterControlsPanel)this.getContentPanel()).getBProperties().addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	propsButtonActionPerformed(evt);
             }
        });
		
    }

    /**
     * Cierra esta ventana a travs de andami 
     */
    public void closeJDialog() {
		this.hide();
		for(int i=0;i<layers.getLayersCount();i++){
			layers.getLayer(i).getMapContext().invalidate();
		}
		PluginServices.getMDIManager().closeWindow(SaveRasterDefaultPanel.this);
	}
    
    /**
     * Funcin a ejecutar cuando se pulsa el botn de propiedades.
     * @param e ActionEvent
     */
    private void propsButtonActionPerformed(ActionEvent e){
    	String name = this.getListener().getFileName();
				
		//Si no se ha seleccionado ningn fichero salimos
		if(name == null || name.equals(""))
			return;
	
		GeoRasterWriter writer = GeoRasterWriter.getWriter(name);
		String ext = name.toLowerCase().substring(name.lastIndexOf(".") + 1);
		if(driverProps.getProperties(ext)!=null)
			writer.setProps(driverProps.getProperties(ext));
		
		frameSaveRasterProps = new SaveRasterPropsDialog(writer);
		frameSaveRasterProps.setParentDialog(this);
		frameSaveRasterProps.show();
	}
	
	/**
	 * 
	 * @param e
	 */
    private boolean acceptButtonActionPerformed(ActionEvent e) {
    	SaveRasterControlsPanel dialog = (SaveRasterControlsPanel)this.getContentPanel();
    	    	
		String fName = this.getListener().getFileName();
		Dimension dimension = new Dimension(	this.getListener().getWidthInPixels(), 
											this.getListener().getHeightInPixels() );
		
		//Comprobamos que el en la ruta de destino tengamos permisos de escritura
		/*File f = new File(fName);
		if(!f.exists()){
			if(!checkFileExists(fName))
				return false;
		}
		f = new File(fName.substring(0, fName.lastIndexOf(File.separator)));
		if(f.exists() && f.isDirectory() && !f.canWrite()){
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_file_not_writable"));
			return false;
		}*/
		
		double lrX = Double.parseDouble(dialog.getTInfDerX().getText());
		double lrY = Double.parseDouble(dialog.getTInfDerY().getText());
		double ulX = Double.parseDouble(dialog.getTSupIzqX().getText());
		double ulY = Double.parseDouble(dialog.getTSupIzqY().getText());
		double width = 0;
		if(ulX>lrX)width = (double)(ulX-lrX);
			else width = (double)(lrX-ulX);
		double height = 0;
		if(ulY>lrY)height = (double)(ulY-lrY);
			else height = (double)(lrY-ulY);
		Rectangle2D ext = null;
		
		if(ulY<lrY)
			ext = new Rectangle2D.Double(ulX, ulY, width, height);
		else
			ext = new Rectangle2D.Double(ulX, lrY, width, height);
		
		ProgressSaveRasterDialog progressBar = 
			new ProgressSaveRasterDialog(mapCtrl, this.layers, this.currentProjection,
										dimension, ext, this);
		
		progressBar.setProps(driverProps);
		progressBar.setWriter(writer);
		progressBar.setFName(fName);
		progressBar.setBlockSize(sizeBlock);
		progressBar.setCompresion(compresion);
		progressBar.showWindow();
				
		for(int i=0;i<layers.getLayersCount();i++){
			layers.getLayer(i).getMapContext().invalidate();
		}
		return true;
    }
    
    /**
	 * Comprueba si existe el fichero pasado por parmetro existe solicitando confirmacin
	 * de sobreescritura. Si el usuario acepta devolver true y si no acepta devuelve false.
	 * Si se acepta la sobreescritura el fichero es borrado. 
	 * @param file Cadena con el nombre del fichero a comprobar su existencia.
	 * @return Devuelve true si el fichero existe y va a ser sobreescrito
	 */
	private boolean checkFileExists(String file){
		File f = new File(file);
		if(f.exists()){
			String string1 = PluginServices.getText(this, "yes");
			String string2 = PluginServices.getText(this, "no");
			Object[] options = {string1, string2};
			int n = JOptionPane.showOptionDialog((Component)PluginServices.getMainFrame(),
					PluginServices.getText(this, "error_file_exists"),
					PluginServices.getText(this, "confirmacion"),
					JOptionPane.YES_NO_OPTION,
					JOptionPane.QUESTION_MESSAGE,
					null,     
					options,  
					string1); 
			if (n == JOptionPane.YES_OPTION){ 
				f.delete();
				return true;
			}else
				return false;
		}
		return true;
	}
	
    /**
     * Obtiene el panel con los controles de Salvar a Raster.
     * @return
     */
    public JPanel getContentPanel() {
        if (contentPane == null) {
        	contentPane = new SaveRasterControlsPanel();
            contentPane.setPreferredSize(new java.awt.Dimension(379, 317));
        }
        return contentPane;
    }

	public DataInputListener getListener() {
		if(listener == null){
			listener = new DataInputListener((SaveRasterControlsPanel)this.getContentPanel());
			listener.setDialogPanel(this);
		}
		return listener;
	}
	
	/**
	 * This method initializes jContentPane	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	public Container getContentPane() {
		return this;
	}
	
	/**
	 * 
	 * @param prj
	 */
	
	public void setProjection(IProjection prj) {
		this.currentProjection = prj;
	}
	
	/**
	 * Asigna la lista de capas
	 * @param layers
	 */
	public void setLayerList(FLayers layers){
		this.layers = layers;
	}
	
	/**
	 * @see com.iver.mdiApp.ui.MDIManager.IWindow#getWindowInfo()
	 */
	public WindowInfo getWindowInfo() {
		WindowInfo m_viewinfo=new WindowInfo(WindowInfo.MODALDIALOG);
    		m_viewinfo.setTitle(PluginServices.getText(this, "salvar_raster"));
		return m_viewinfo;
	}
}
