package com.iver.cit.gvsig.fmap.layers.gml;

import java.awt.geom.Rectangle2D;
import java.io.File;
import java.util.ArrayList;

import junit.framework.TestCase;

import org.cresques.cts.IProjection;
import org.cresques.cts.ProjectionPool;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.FPolyline2D;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.GeneralPathX;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.drivers.ConcreteMemoryDriver;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.FieldDescription;
import com.iver.cit.gvsig.fmap.drivers.SHPLayerDefinition;
import com.iver.cit.gvsig.fmap.drivers.VectorialFileDriver;
import com.iver.cit.gvsig.fmap.edition.DefaultRowEdited;
import com.iver.cit.gvsig.fmap.edition.EditionException;
import com.iver.cit.gvsig.fmap.edition.IRowEdited;
import com.iver.cit.gvsig.fmap.edition.writers.gml.GMLWriter;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.ReadableVectorial;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.VectorialFileAdapter;
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GMLWriterTest.java,v 1.1 2006/07/24 07:30:33 jorpiell Exp $
 * $Log: GMLWriterTest.java,v $
 * Revision 1.1  2006/07/24 07:30:33  jorpiell
 * Se han eliminado las partes duplicadas y se est usando el parser de GML de FMAP.
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class GMLWriterTest extends TestCase {
	private String gmlFile = "MUNIC.gml";
	
	protected void setUp(){
		LayerFactory.setDriversPath("../_fwAndami/gvSIG/extensiones/com.iver.cit.gvsig/drivers");
		LayerFactory.setWritersPath("../_fwAndami/gvSIG/extensiones/com.iver.cit.gvsig/drivers");
		System.out.println("****************READERS****************");
		for (int i=0 ; i<LayerFactory.getDM().getDriverNames().length ; i++){
			System.out.println(LayerFactory.getDM().getDriverNames()[i]);
		}
		System.out.println("****************WRITERS****************");
		for (int i=0 ; i<LayerFactory.getWM().getWriterNames().length ; i++){
			System.out.println(LayerFactory.getWM().getWriterNames()[i]);
		}
		System.out.println("****************COMENZANDO****************");
	}
	
	public void test1() throws DriverLoadException, DriverException, EditionException, DriverIOException, com.hardcode.gdbms.engine.data.driver.DriverException{
        // DRIVER DEFINITION (SHAPE TYPE AND FIELDS)
        ConcreteMemoryDriver driver = new ConcreteMemoryDriver();
        driver.setShapeType(FShape.LINE);
        
        ArrayList arrayFields = new ArrayList();
        arrayFields.add("ID");
        Value[] auxRow = new Value[1];
  
        driver.getTableModel().setColumnIdentifiers(arrayFields.toArray());
                
        //GEOMETRY DEFINITION
        GeneralPathX gp = new GeneralPathX();
        
        gp.moveTo(20,80);
        gp.lineTo(80,60);
        gp.lineTo(100, 140);
        
        FShape shp = new FPolyline2D(gp);
        
        
        // ATRIBUTES
        auxRow[0] = ValueFactory.createValue(0);
        
        // ADD RECORD
        driver.addShape(shp, auxRow);
         
        //GEOMETRY DEFINITION
        gp = new GeneralPathX();
        
        gp.moveTo(20,80);
        gp.lineTo(80,60);
        gp.lineTo(100, 140);
        
        shp = new FPolyline2D(gp);
        
        
        // ATRIBUTES
        auxRow[0] = ValueFactory.createValue(1);
        
        // ADD RECORD
        driver.addShape(shp, auxRow);

        // CREATE AND ADD LAYER
        FLyrVect lyr;
        String layerName = "Example";
       	lyr = (FLyrVect) LayerFactory.createLayer(layerName,driver, ProjectionPool.get("EPSG:4326"));           	
		
       	       	
       	SHPLayerDefinition lyrDef = new SHPLayerDefinition();
		SelectableDataSource sds = lyr.getRecordset();
		FieldDescription[] fieldsDescrip = sds.getFieldsDescription();
		lyrDef.setFieldsDesc(fieldsDescrip);
		lyrDef.setName(lyr.getName());
       	
		File newFile = new File(gmlFile);
		
		GMLWriter writer = (GMLWriter)LayerFactory.getWM().getWriter("GML Writer");
		writer.setFile(new File(gmlFile));
		writer.setSchema(lyrDef);		
		Rectangle2D rectangle = new Rectangle2D.Double(0,
				0,
				100,
				100);		
		writer.setBoundedBy(rectangle,ProjectionPool.get("EPSG:23030"));
		
		writer.preProcess();
		
		ReadableVectorial adapter = lyr.getSource();
		for (int i=0; i < adapter.getShapeCount(); i++){
			IFeature feat = adapter.getFeature(i);
			IRowEdited editFeat = new DefaultRowEdited(feat, IRowEdited.STATUS_MODIFIED, i);
			writer.process(editFeat);
		}
		writer.postProcess();		
	}
	

	
	private static FLayer createLayer(String layerName, VectorialFileDriver d,
			File f, IProjection proj) throws DriverException {
			//TODO Comprobar si hay un adaptador ya
			VectorialFileAdapter adapter = new VectorialFileAdapter(f);
			adapter.setDriver(d);
			FLyrVect capa = new FLyrVect();
			capa.setName(layerName);

			capa.setSource(adapter);
			capa.setProjection(proj);

			return capa;
		}
}
