package com.iver.cit.gvsig.fmap.drivers.geotools;

import java.awt.Component;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.util.ArrayList;
import java.util.Hashtable;

import javax.swing.JOptionPane;

import org.geotools.feature.AttributeType;
import org.gvsig.remoteClient.wfs.WFSStatus;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.layers.WFSLayerNode;


/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSFeatureLayer.java,v 1.5 2006/07/24 07:30:33 jorpiell Exp $
 * $Log: WFSFeatureLayer.java,v $
 * Revision 1.5  2006/07/24 07:30:33  jorpiell
 * Se han eliminado las partes duplicadas y se est usando el parser de GML de FMAP.
 *
 * Revision 1.4  2006/07/05 12:05:41  jorpiell
 * Se ha modificado para que avise si se han recuperado las mismas features que marca el campo buffer
 *
 * Revision 1.3  2006/06/21 12:35:45  jorpiell
 * Se ha aadido la ventana de propiedades. Esto implica aadir listeners por todos los paneles. Adems no se muestra la geomatra en la lista de atributos y se muestran nicamnete los que se van a descargar
 *
 * Revision 1.2  2006/06/15 10:34:12  jorpiell
 * Ya se pueden ver los atributos de las tablas
 *
 * Revision 1.1  2006/06/14 07:57:19  jorpiell
 * Ya no se usa la estrategia ni se usa geotools para hacer el getFeature. Ahora se usa nicamente para el parseo de GML
 *
 * Revision 1.6  2006/05/25 10:32:21  jorpiell
 * Se ha renombrado la clase WFSFields por WFSAttributes porque era algo confusa
 *
 * Revision 1.5  2006/05/24 07:21:38  jorpiell
 * Si no ha dado tiempo a inicializar el resultset se debe devolver nulo
 *
 * Revision 1.4  2006/05/23 08:07:37  jorpiell
 * Cambio probocado por el renombramiento de la clase DataStore por WFSDataStore
 *
 * Revision 1.3  2006/05/22 10:32:01  jorpiell
 * El connect debe de tener en cuanta los valores del buffer y el timeout
 *
 * Revision 1.2  2006/05/22 10:10:46  jorpiell
 * El driver recorre las features la primera vez e inicializa el vector
 *
 * Revision 1.1  2006/05/19 12:48:21  jorpiell
 * Driver que implementa la accesibilidad a geotools
 *
 *
 */
/**
 *
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class WFSFeatureLayer {
	private ArrayList features = new ArrayList();
	private Rectangle2D fullExtent = null;
	private Hashtable hashRelate = new Hashtable();
	private WFSStatus currentStatus = null;
	private int numReg = -1;
	private int numField = -1;
	private AttributeType[] attributes;
	private WFSFeaturesIterator iterator = null;
	
	public WFSFeatureLayer(File file,WFSLayerNode[] schemas,WFSStatus status) throws Exception{
		WFSLayerNode schema = null;
		currentStatus = status;
		for (int i=0 ; i<schemas.length ; i++){
			if (schemas[i].getName().equals(status.getFeatureName())){
				schema = schemas[i];
				attributes = WFSTypesConversion.attributteTypeGeometryfromGvsigToGeotools(schema);
				numField = attributes.length;
			}
		}
		iterator = WFSGetFeatureParser.getFeatureReader(file,
				schema,
				status);
		initialize();
	}
	
	/**
	 * Reloads the layer
	 *
	 */
	public void reload(){
		initialize();
	}
	
	/**
	 * Sets the retrieved features
	 * @param transformation
	 * @throws DriverException 
	 * @throws DriverException 
	 */
	private void initialize() {
		int index = 0;
		while (iterator.hasNext()){
			IFeature feature;
			try {
				feature = iterator.next();
				try{
					features.add(feature);
					Rectangle2D rect = feature.getGeometry().getBounds2D();
					
					if (fullExtent == null) {
						fullExtent = new Rectangle2D.Double(rect.getMinX(),
								rect.getMinY(), rect.getWidth(), rect.getHeight());
					} else {
						fullExtent.add(new Rectangle2D.Double(rect.getMinX(),
								rect.getMinY(), rect.getWidth(), rect.getHeight()));
					}
					
					hashRelate.put(feature.getID(), new Integer(index));
				}catch(NullPointerException e){
					//
				}
			} catch (DriverException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}			
			index++;
		}
		numReg = index;
		if (numReg == currentStatus.getBuffer()){
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
				 	PluginServices.getText(this,"maxFeatures_aviso"));	
		}
	}

	/**
	 * @return Returns the features.
	 */
	public ArrayList getFeatures() {
		return features;
	}

	/**
	 * @return Returns the hashRelate.
	 */
	public Hashtable getHashRelate() {
		return hashRelate;
	}

	/**
	 * @return Returns the fullExtent.
	 */
	public Rectangle2D getFullExtent() {
		return fullExtent;
	}

	/**
	 * @return Returns the numReg.
	 */
	public int getNumReg() {
		return numReg;
	}

	/**
	 * @return Returns the attributes.
	 */
	public AttributeType[] getAttributes() {
		return attributes;
	}

	/**
	 * @return Returns the numField.
	 */
	public int getNumField() {
		return numField;
	}

	/**
	 * @return Returns the iterator.
	 */
	public WFSFeaturesIterator getIterator() {
			if (iterator == null){
				return null;
			}
			iterator.startIteration();
			return iterator;		
	}
}
