package com.iver.cit.gvsig.fmap.drivers.geotools;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Vector;

import org.geotools.data.FeatureReader;
import org.geotools.data.wfs.WFSFeatureReader;
import org.geotools.data.wfs.WFSTransactionState;
import org.geotools.factory.FactoryConfigurationError;
import org.geotools.feature.AttributeType;
import org.geotools.feature.AttributeTypeFactory;
import org.geotools.feature.FeatureType;
import org.geotools.feature.FeatureTypeBuilder;
import org.geotools.feature.SchemaException;
import org.gvsig.remoteClient.wfs.WFSAttribute;
import org.gvsig.remoteClient.wfs.WFSStatus;
import org.xml.sax.SAXException;

import com.iver.cit.gvsig.fmap.drivers.WFSException;
import com.iver.cit.gvsig.fmap.drivers.gml.GMLFeaturesParser;
import com.iver.cit.gvsig.fmap.layers.WFSLayerNode;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSGetFeatureParser.java,v 1.7 2006/07/24 07:30:33 jorpiell Exp $
 * $Log: WFSGetFeatureParser.java,v $
 * Revision 1.7  2006/07/24 07:30:33  jorpiell
 * Se han eliminado las partes duplicadas y se est usando el parser de GML de FMAP.
 *
 * Revision 1.6  2006/07/05 12:05:41  jorpiell
 * Se ha modificado para que avise si se han recuperado las mismas features que marca el campo buffer
 *
 * Revision 1.5  2006/06/21 12:35:45  jorpiell
 * Se ha aadido la ventana de propiedades. Esto implica aadir listeners por todos los paneles. Adems no se muestra la geomatra en la lista de atributos y se muestran nicamnete los que se van a descargar
 *
 * Revision 1.4  2006/06/15 11:17:06  jorpiell
 * Se ha encontrado la forma de comprobar cuando se prodicia un error de parseo al hacer un hasnext (en la feature). Se atrapa y se lanza la excepcin hacia arriba
 *
 * Revision 1.3  2006/06/15 10:34:12  jorpiell
 * Ya se pueden ver los atributos de las tablas
 *
 * Revision 1.2  2006/06/15 07:50:58  jorpiell
 * Aadida la funcionalidad de reproyectar y hechos algunos cambios en la interfaz
 *
 * Revision 1.1  2006/06/14 07:57:19  jorpiell
 * Ya no se usa la estrategia ni se usa geotools para hacer el getFeature. Ahora se usa nicamente para el parseo de GML
 *
 * Revision 1.2  2006/05/23 13:21:12  jorpiell
 * Modificado el datasource para capturar las excepciones de parseo
 *
 * Revision 1.1  2006/05/23 08:06:40  jorpiell
 * Clase que antiguamente se llamaba DataStore
 *
 * Revision 1.2  2006/05/19 12:47:21  jorpiell
 * DataStore que usa geotools. La clase datastore de geotools es abstracta, por lo que tenemos que crear esta clase que hereda de ella
 *
 * Revision 1.1  2006/05/19 12:45:47  jorpiell
 * DataStore que usa geotools. La clase datastore de geotools es abstracta, por lo que tenemos que crear esta clase que hereda de ella
 *
 *
 */
/**
 * It parses a getFeature request file
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class WFSGetFeatureParser extends GMLFeaturesParser{

	/**
	 * Return an iterrator of features
	 * @param file
	 * File to parse
	 * @param schemas
	 * Schemas to retrieve
	 * @param status
	 * Serviced status
	 * @return
	 * Iterator
	 * @throws Exception
	 */
	public static WFSFeaturesIterator getFeatureReader(File file,WFSLayerNode schema,WFSStatus status) throws Exception{
		FileInputStream is = new FileInputStream(file);
		
		Vector vSrs = schema.getSrs();	
		String srs = null;
		if (vSrs.size() > 0){
			srs = (String)vSrs.get(0);
		}
		FeatureReader featureReader = getFeatureReader(is,
				10000,
				status.getTimeout(),
				null,
				WFSTypesConversion.featureTypefromGeotoolsToGvSIG(schema));
		if (!(featureReader.hasNext())){
			throw new WFSException();
		}
		return new WFSFeaturesIterator(featureReader,srs);
	}


	
}
