package com.iver.cit.gvsig.fmap.drivers.geotools;

import java.util.Vector;

import org.geotools.factory.FactoryConfigurationError;
import org.geotools.feature.AttributeType;
import org.geotools.feature.AttributeTypeFactory;
import org.geotools.feature.FeatureType;
import org.geotools.feature.FeatureTypeBuilder;
import org.geotools.feature.SchemaException;
import org.gvsig.remoteClient.wfs.WFSAttribute;

import com.iver.cit.gvsig.fmap.drivers.gml.GMLTypesConversor;
import com.iver.cit.gvsig.fmap.layers.WFSLayerNode;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSTypesConversion.java,v 1.1 2006/07/24 07:30:33 jorpiell Exp $
 * $Log: WFSTypesConversion.java,v $
 * Revision 1.1  2006/07/24 07:30:33  jorpiell
 * Se han eliminado las partes duplicadas y se est usando el parser de GML de FMAP.
 *
 *
 */
/**
 * Some utils to convert types
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class WFSTypesConversion extends GMLTypesConversor {
	 
	 /**
	  * Make a conversion between the gvSIG WFSLayerNode and a geotools
	  * FeatureType.
	  * @param schema
	  * gvSIG feature schema that contains a list of attributes
	  * @return
	  */
	 static FeatureType featureTypefromGeotoolsToGvSIG(WFSLayerNode schema) throws FactoryConfigurationError, SchemaException{
		 AttributeType[] types = attributteTypefromGvsigToGeotools(schema);
		 String typeName = schema.getName();
		 FeatureType featureType = FeatureTypeBuilder.newFeatureType(types,typeName);					
		 return featureType;
	 }
	 
	/**
	 * Make a conversion between the gvSIG WFSAttribute and a geotools
	 * AttributeType.
	 * @param schema
	 * gvSIG feature schema that contains a list of attributes
	 * @return
	 */
	 public static AttributeType[] attributteTypefromGvsigToGeotools(WFSLayerNode schema){
		 Vector attributes = schema.getSelectedFields();
		 AttributeType[] attributeTypes = new AttributeType[attributes.size()];
		 for (int i=0 ; i<attributes.size() ; i++){
			 WFSAttribute attribute = (WFSAttribute)attributes.get(i);
		 	AttributeType type = AttributeTypeFactory.newAttributeType(attribute.getName(),
					 WFSUtils.getAttributeJavaType(attribute));
		 	attributeTypes[i] = type;
		 }
		 return attributeTypes;
	 }
	 
	 public static AttributeType[] attributteTypeGeometryfromGvsigToGeotools(WFSLayerNode schema){
		 Vector attributes = schema.getSelectedFields();
		 Vector addedAttributes = new Vector();
		 for (int i=0 ; i<attributes.size() ; i++){
			 WFSAttribute attribute = (WFSAttribute)attributes.get(i);
		 	 if (!(WFSUtils.isGeometry(attribute.getType()))){
		 		 AttributeType type = AttributeTypeFactory.newAttributeType(attribute.getName(),
		 				 WFSUtils.getAttributeJavaType(attribute));
		 		 addedAttributes.add(type);
		 	 }
		 }
		 AttributeType[] attributeTypes = new AttributeType[addedAttributes.size()];
		 for (int i=0 ; i<addedAttributes.size() ; i++){
			 attributeTypes[i] = (AttributeType)addedAttributes.get(i);
		 }
		 return attributeTypes;
	 }
	 

}
