package com.iver.cit.gvsig.gui.dialogs;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import java.util.HashMap;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.gvsig.remoteClient.wfs.WFSStatus;

import com.iver.andami.PluginServices;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.WindowInfo;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.FLyrWFS;
import com.iver.cit.gvsig.fmap.layers.WFSLayerNode;
import com.iver.cit.gvsig.gui.panels.WFSParamsPanel;
import com.iver.cit.gvsig.gui.wizards.WFSWizardData;
import com.iver.cit.gvsig.gui.wizards.WizardListener;
import com.iver.cit.gvsig.gui.wizards.WizardListenerSupport;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSPropsDialog.java,v 1.4 2006/08/30 07:42:29 jorpiell Exp $
 * $Log: WFSPropsDialog.java,v $
 * Revision 1.4  2006/08/30 07:42:29  jorpiell
 * Se le asigna a la capa creada una proyeccin concreta. Si esto no se hace, al exportar a postigis se produce un error.
 *
 * Revision 1.3  2006/08/29 07:56:12  cesar
 * Rename the *View* family of classes to *Window* (ie: SingletonView to SingletonWindow, ViewInfo to WindowInfo, etc)
 *
 * Revision 1.2  2006/08/29 07:13:40  cesar
 * Rename class com.iver.andami.ui.mdiManager.View to com.iver.andami.ui.mdiManager.IWindow
 *
 * Revision 1.1  2006/06/21 12:35:45  jorpiell
 * Se ha aadido la ventana de propiedades. Esto implica aadir listeners por todos los paneles. Adems no se muestra la geomatra en la lista de atributos y se muestran nicamnete los que se van a descargar
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class WFSPropsDialog extends JPanel implements IWindow{
	private WFSParamsPanel wfsParamsTabbedPane = null;
	private WindowInfo m_ViewInfo = null;
	private FLayer fLayer = null;
	private JButton btnApply = null;
	private JButton btnOk = null;
	private JButton btnCancel = null;
	private CommandListener m_actionListener = null;
	private JPanel buttonsPanel = null;
	boolean applied = false;
	

    public WFSPropsDialog(FLayer layer) {
		super();
		initialize(layer);
	}
    
    private void initialize(FLayer layer) {
		setLayout(null);		
        setFLayer(layer);
        wfsParamsTabbedPane = getParamsPanel(((FLyrWFS) layer).getProperties());
        wfsParamsTabbedPane.addWizardListener(new WizardListener(){
			public void wizardStateChanged(boolean finishable) {
				getBtnOk().setEnabled(finishable);
				getBtnApply().setEnabled(finishable);
			}

			public void error(Exception e) {
			}
        	
        })  ;
        //wfsParamsTabbedPane.disableDisagregatedLayers();
        this.add(wfsParamsTabbedPane);
        this.add(getButtonsPanel(), null);
    }
    
    public JPanel getButtonsPanel() {
		if (buttonsPanel == null) {
			m_actionListener = new CommandListener(this);
			buttonsPanel = new JPanel();
	        buttonsPanel.setBounds(5, wfsParamsTabbedPane.getHeight(), 471, 40);
			buttonsPanel.setLayout(null);     
			buttonsPanel.setName("buttonPanel");
			
	        buttonsPanel.add(getBtnOk(), null);
	        buttonsPanel.add(getBtnApply(), null);
	        buttonsPanel.add(getBtnCancel(), null);
		}
		return buttonsPanel;
	}
	
    
    /**
	 * With getParamsPanel we have access to the map config TabbedPane.
	 * If this panel doesn't exist yet (which can occur when an existing project is
	 * recovered) it is been automatically constructed by connecting to the server,
	 * reloading the necessary data, filling up the content and setting the selected
	 * values that were selected when the projet was saved.
	 * 
	 * 
	 * Since a connection to the server is needed when rebuiliding the panel, this
	 * causes a delay for the panel's showing up or a nullPointer error if there is
	 * no path to the server.
	 * 
	 * 
	 * Con getParamsPanel tenemos acceso a juego de pestaas de configuracin
	 * del mapa. Si este panel todava no existe (como puede ser cuando
	 * recuperamos un proyecto guardado) �ste se crea automticamente conectando
	 * al servidor, recuperando los datos necesarios, rellenando el contenido y
	 * dejando seleccionados los valores que estaban seleccionados cuando se
	 * guard� el proyecto.
	 * 
	 * 
	 * Como para reconstruirse requiere una conexi�n con el servidor esto causa
	 * un retardo en la aparici�n en el toc o un error de nullPointer si no
	 * hay conexi�n hasta el servidor.
	 * 
	 * 
	 * @return WMSParamsPanel
	 */
	public WFSParamsPanel getParamsPanel(HashMap info) {
	    if (info!=null){
	    	try {	    		
	    		URL host = (URL) info.get("host");
	    		WFSWizardData dataSource = new WFSWizardData();
	    		dataSource.setHost(host, false);

	    		WFSParamsPanel toc = new WFSParamsPanel();
	    		toc.setWizardData(dataSource);
	    		toc.setListenerSupport(new WizardListenerSupport());
	    		toc.setSelectedFeature((WFSLayerNode)info.get("wfsLayerNode"));
	    		toc.setselectedAttributes((WFSLayerNode)info.get("wfsLayerNode"));
	    		toc.setLayerName(fLayer.getName());
	    		toc.refreshInfo((WFSLayerNode)info.get("wfsLayerNode"));
	    		toc.setStatus((WFSStatus)info.get("status"));
	    		toc.setVisible(true);
	    		applied = false;
	    		return toc;
	    	} catch (DriverException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
	    }
	    return null;        
	}
	
	
	public WindowInfo getWindowInfo() {
		if (m_ViewInfo == null){
			m_ViewInfo=new WindowInfo(WindowInfo.MODALDIALOG);
			m_ViewInfo.setTitle(PluginServices.getText(this,"fit_WFS_layer"));
			m_ViewInfo.setWidth(wfsParamsTabbedPane.getWidth()+ 10);
			m_ViewInfo.setHeight(wfsParamsTabbedPane.getHeight() + 40);
		}
		return m_ViewInfo;
	}

	/**
	 * @return Returns the fLayer.
	 */
	public FLayer getFLayer() {
		return fLayer;
	}

	/**
	 * @param layer The fLayer to set.
	 */
	public void setFLayer(FLayer layer) {
		fLayer = layer;
	}

	public JButton getBtnOk() {
		if (btnOk == null) {
	        btnOk = new JButton("ok");
	        btnOk.setText(PluginServices.getText(this,"ok"));
	        btnOk.setActionCommand("OK");
	        btnOk.addActionListener(m_actionListener);
	        btnOk.setBounds(367, 9, 90, 25);
		}
		return btnOk;
	}
	
	public JButton getBtnApply() {
		if (btnApply == null) {
	        btnApply = new JButton("apply");
	        btnApply.setText(PluginServices.getText(this,"apply"));
	        btnApply.setEnabled(false);
	        btnApply.setActionCommand("APPLY");
	        btnApply.addActionListener(m_actionListener);
	        btnApply.setBounds(267, 9, 90, 25);
		}
		return btnApply;
	}
	
	public JButton getBtnCancel() {
		if (btnCancel == null) {
	        btnCancel = new JButton("cancel");
	        btnCancel.setText(PluginServices.getText(this,"cancel"));
	        btnCancel.setActionCommand("CANCEL");
	        btnCancel.addActionListener(m_actionListener);
	        btnCancel.setBounds(137, 9, 90, 25);
		}
		return btnCancel;
	}
	
	public void close() {
		PluginServices.getMDIManager().closeWindow(this);		
	}
	
    /**
     * Merge two FLayers in one
     * @param group1
     * @param group2
     * @return
     */
    private FLayers mergeFLayers(FLayers group1, FLayers group2)
    {    	
    	FLayer layer;
    	for(int i = 0; i < group2.getLayersCount(); i++)
    	{
    		layer = group2.getLayer( i );
    		if(group1.getLayer( layer.getName()) == null ){
    			group1.addLayer( layer );
    		}
    	}
    	
    	return group1;
    }
	
	 private class CommandListener implements ActionListener {
		 public CommandListener(WFSPropsDialog tp) {
			 //m_tp = tp;
		 }
		 
		 /* (non-Javadoc)
		  * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		  */
		 public void actionPerformed(ActionEvent e) {
			 if (e.getActionCommand() == "CANCEL"){
				 close();
			 }else{	
				String layerName = fLayer.getName();
				com.iver.cit.gvsig.gui.View activeView = 
					(com.iver.cit.gvsig.gui.View) PluginServices.getMDIManager().getActiveWindow();
				
				 fLayer = wfsParamsTabbedPane.getLayer();
				 ((FLyrWFS) fLayer).setHost(wfsParamsTabbedPane.getData().getHost());
				 ((FLyrWFS) fLayer).setWfsDriver(wfsParamsTabbedPane.getData().getDriver());
				 ((FLyrWFS) fLayer).setProjection(activeView.getMapControl().getViewPort().getProjection());	 				 
				 if (e.getActionCommand() == "APPLY"){
					 if (loadLayer(fLayer)){		 
						 com.iver.cit.gvsig.gui.View vista = (com.iver.cit.gvsig.gui.View) PluginServices.getMDIManager().getActiveWindow();
						 MapControl mapCtrl = vista.getMapControl();
				
						 mapCtrl.getMapContext().getLayers().replaceLayer(layerName, fLayer);
						 					
						 mapCtrl.getMapContext().invalidate();
						 applied = true;
						 getBtnApply().setEnabled(!applied); 
					 }
				 }
				 if (e.getActionCommand() == "OK") {
					 if (!applied) {
						 if (loadLayer(fLayer)){
							 com.iver.cit.gvsig.gui.View vista = (com.iver.cit.gvsig.gui.View) PluginServices.getMDIManager().getActiveWindow();
							 MapControl mapCtrl = vista.getMapControl();
							 mapCtrl.getMapContext().getLayers().replaceLayer(layerName, fLayer);
			 				 mapCtrl.getMapContext().invalidate();
						 }
					 }	
					 close();		                
				 }
			 }
		 }
		 private boolean loadLayer(FLayer flayer){
			 try {
				 ((FLyrWFS)fLayer).load();
			 } catch (Exception e1) {
				 // TODO Auto-generated catch block
				 e1.printStackTrace();
				 JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
						 PluginServices.getText(this,"cantLoad"));
				 return false;
			 }
			 return true;
		 }			 
		 
	 }
}

