package com.iver.cit.gvsig.gui.panels;

import java.util.ArrayList;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.gvsig.gui.beans.swing.JButton;
import org.gvsig.remoteClient.wfs.WFSAttribute;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.drivers.geotools.WFSUtils;
import com.iver.cit.gvsig.fmap.layers.WFSLayerNode;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSSelectAttributesPanel.java,v 1.7 2006/09/05 13:01:31 jorpiell Exp $
 * $Log: WFSSelectAttributesPanel.java,v $
 * Revision 1.7  2006/09/05 13:01:31  jorpiell
 * Reducido el tamao de los botones
 *
 * Revision 1.6  2006/07/24 07:30:33  jorpiell
 * Se han eliminado las partes duplicadas y se est usando el parser de GML de FMAP.
 *
 * Revision 1.5  2006/07/21 11:50:31  jaume
 * improved appearance
 *
 * Revision 1.4  2006/06/21 12:35:45  jorpiell
 * Se ha aadido la ventana de propiedades. Esto implica aadir listeners por todos los paneles. Adems no se muestra la geomatra en la lista de atributos y se muestran nicamnete los que se van a descargar
 *
 * Revision 1.3  2006/06/15 07:50:58  jorpiell
 * Aadida la funcionalidad de reproyectar y hechos algunos cambios en la interfaz
 *
 * Revision 1.2  2006/05/26 06:29:42  jorpiell
 * Se ha cambiado el tamao de los botones. Adems, al seleccionar todos los atributos se seleccionan en orden.
 *
 * Revision 1.1  2006/05/25 10:30:13  jorpiell
 * Esta clase se ha renombrado. WFSFields era algo confusa
 *
 * Revision 1.3  2006/05/23 08:09:39  jorpiell
 * Se ha cambiado la forma en la que se leian los valores seleccionados en los paneles y se ha cambiado el comportamiento de los botones
 *
 * Revision 1.2  2006/05/19 12:57:08  jorpiell
 * Modificados algunos paneles
 *
 * Revision 1.1  2006/04/20 16:38:24  jorpiell
 * Ahora mismo ya se puede hacer un getCapabilities y un getDescribeType de la capa seleccionada para ver los atributos a dibujar. Queda implementar el panel de opciones y hacer el getFeature().
 *
 *
 */
/**
 * This panel shows the feature fields list
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class WFSSelectAttributesPanel extends JPanel {
	private static final long serialVersionUID = 3073243334672620143L;
	private JButton addButton = null;
	private JButton delButton = null;
	private JScrollPane jScrollPane3 = null;
	private JScrollPane jScrollPane4 = null;
	private AttributesTable attributesList = null;
	private JList selectedAttributesList = null;
	private JButton addAllButton = null;
	private JButton delAllButton = null;
	private WFSParamsPanel parent = null;

	public WFSSelectAttributesPanel(WFSParamsPanel parent){
		super();
		this.parent = parent;
		initialize();
	}

	public void initialize(){
		this.setLayout(null);
		this.setBounds(5, 17, 464, 374);
		this.setBorder(javax.swing.BorderFactory.createTitledBorder(
				null, PluginServices.getText(this, "select_fields"),
				javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
				javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
		this.add(getAddButton(), null);
		this.add(getDelButton(), null);
		this.add(getJScrollPane3(), null);
		this.add(getJScrollPane4(), null);
		this.add(getAddAllButton(), null);
		this.add(getDelAllButton(), null);
	}

	/**
	 * This method initializes jButton2
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getAddButton() {
		if (addButton == null) {
			addButton = new JButton();
			addButton.setBounds(245, 170, 50, 25);
			addButton.addActionListener(
				new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						addAttribute();
						parent.isApplicable(true);
					}
				});
			addButton.setIcon(new ImageIcon(getClass().getClassLoader().getResource("images/forward.png")));
		}
		return addButton;
	}

	/**
	 * This method initializes btnDelTemps
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getDelButton() {
		if (delButton == null) {
			delButton = new JButton();
			delButton.setBounds(245, 210, 50, 25);
			delButton.addActionListener(
				new java.awt.event.ActionListener() {
					public void actionPerformed(java.awt.event.ActionEvent e) {
						delAttribute();
						parent.isApplicable(true);
					}
				});
			delButton.setIcon(new ImageIcon(getClass().getClassLoader().getResource("images/backward.png")));
		}
		return delButton;
	}

	/**
	 * This method initializes jButton
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getAddAllButton() {
		if (addAllButton == null) {
			addAllButton = new JButton();
			addAllButton.setBounds(new java.awt.Rectangle(245,130,50, 25));
			addAllButton.addActionListener(
					new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							addAllAttributes();
							parent.isApplicable(true);
						}
					});
			addAllButton.setIcon(new ImageIcon(getClass().getClassLoader().getResource("images/forwardDouble.png")));
		}
		return addAllButton;
	}

	/**
	 * This method initializes jButton1
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getDelAllButton() {
		if (delAllButton == null) {
			delAllButton = new JButton();
			delAllButton.setBounds(new java.awt.Rectangle(245,250,50,25));
			delAllButton.addActionListener(
					new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							delAllAttributes();
							parent.isApplicable(true);
						}
					});
			delAllButton.setIcon(new ImageIcon(getClass().getClassLoader().getResource("images/backwardDouble.png")));
		}
		return delAllButton;
	}


	/**
	 * This method initializes jScrollPane3
	 *
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane3() {
		if (jScrollPane3 == null) {
			jScrollPane3 = new JScrollPane();
			jScrollPane3.setBounds(10, 30, 226, 334);
			jScrollPane3.setViewportView(getAttributesList());
		}
		return jScrollPane3;
	}

	/**
	 * This method initializes jScrollPane4
	 *
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane4() {
		if (jScrollPane4 == null) {
			jScrollPane4 = new JScrollPane();
			jScrollPane4.setBounds(304, 30, 186, 334);
			jScrollPane4.setViewportView(getSelectedAttributesList());
		}
		return jScrollPane4;
	}

	/**
	 * This method initializes lstTemps
	 *
	 * @return javax.swing.JList
	 */
	public AttributesTable getAttributesList() {
		if (attributesList == null) {
			attributesList = new AttributesTable();
		}
		return attributesList;
	}

	/**
	 * This method initializes selectedAttributesList
	 *
	 * @return javax.swing.JList
	 */
	public JList getSelectedAttributesList() {
		if (selectedAttributesList == null) {
			selectedAttributesList = new JList();
			selectedAttributesList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
				public void valueChanged(javax.swing.event.ListSelectionEvent e) {
//					selectedAttributesList.callStateChanged(isCorretlyConfigured());
				}
			});
		}
		return selectedAttributesList;
	}

	public void refresh(WFSLayerNode feature){
		setFields(feature);
	}

	/**
	 *
	 * @param feature
	 */
	private void setFields(WFSLayerNode feature){
		getSelectedAttributesList().setListData(new Object[0]);
		Vector fields = feature.getFields();
		getAttributesList().addAttributes(fields.toArray());
		for (int i=0 ; i<fields.size() ;i++){
			WFSAttribute field = (WFSAttribute)fields.get(i);
			if (WFSUtils.isGeometry(field.getType())){
				Vector geometry = new Vector();
				geometry.add(field);
				getSelectedAttributesList().setListData(geometry.toArray());
				feature.setGeometry(field);
			}
		}

	}

	public void setSelectedFields(WFSLayerNode feature){
		Vector selectedFields = feature.getSelectedFields();
		Vector addedFields = new Vector();
		Object[] selectableAttributes = getAttributesList().getAllValues();
		for (int i=0 ; i<selectedFields.size() ; i++){
			WFSAttribute selectedAttribute = (WFSAttribute)selectedFields.get(i);
			for (int j=0; j < selectableAttributes.length; j++) {
				WFSAttribute attribute = (WFSAttribute)selectableAttributes[j];
				if (attribute.getName().equals(selectedAttribute.getName())){
					addedFields.add(attribute);
				}
			}
		}
		getSelectedAttributesList().setListData(addedFields.toArray());
	}

	/**
	 * Adds the selected items from the time list to the selected
	 * features list.
	 */
	private void addAttribute(){
		ArrayList times = new ArrayList();
		for (int i = 0; i < getSelectedAttributesList()
				.getModel().getSize(); i++) {
			times.add(getSelectedAttributesList().getModel()
					.getElementAt(i));
		}

		Object[] obj = getAttributesList().getSelectedValues();
		for (int i = 0; i < obj.length; i++) {
			if (!times.contains(obj[i]))
				times.add(obj[i]);
		}
		getSelectedAttributesList()
				.setListData(times.toArray());

	}

	/**
	 * Removes the selected items from the selected features list.
	 */
	private void delAttribute(){
		ArrayList features = new ArrayList();
		Object[] obj = getSelectedAttributesList()
				.getSelectedValues();
		for (int i = 0; i < getSelectedAttributesList()
				.getModel().getSize(); i++)
			features.add(getSelectedAttributesList().getModel()
					.getElementAt(i));

		for (int j = 0; j < obj.length; j++){
			if (features.contains(obj[j])){
				WFSAttribute field = (WFSAttribute)obj[j];
				if (!(WFSUtils.isGeometry(field.getType()))){
					features.remove(obj[j]);
				}
			}
		}
		obj = new Object[features.size()];
		for (int i = 0; i < obj.length; i++) {
			obj[i] = features.get(i);
		}
		getSelectedAttributesList()
				.setListData(features.toArray());
	}

	/**
	 * Removes all the selected attributes
	 */
	private void delAllAttributes() {
		getSelectedAttributesList().setListData(new Object[0]);
		Object[] obj = getAttributesList().getAllValues();
		for (int i = 0; i < obj.length; i++) {
			WFSAttribute field = (WFSAttribute)obj[i];
			if (WFSUtils.isGeometry(field.getType())){
				Vector geometry = new Vector();
				geometry.add(field);
				getSelectedAttributesList().setListData(geometry.toArray());
			}
		}
	}

	/**
	 * Adds all the selectable attributes
	 *
	 */
	private void addAllAttributes() {
		ArrayList attributes = new ArrayList();
		for (int i = 0; i < getSelectedAttributesList()
				.getModel().getSize(); i++) {
			attributes.add(getSelectedAttributesList().getModel()
					.getElementAt(i));
		}

		Object[] obj = getAttributesList().getAllValues();
		for (int i = 0; i < obj.length; i++) {
			if (!attributes.contains(obj[i]))
				attributes.add(obj[i]);
		}
		getSelectedAttributesList()
				.setListData(attributes.toArray());

	}

	/**
	 *
	 * @return the selected fields.
	 */
	public WFSAttribute[] getSelectedFields(){
		int numSelected = getSelectedAttributesList().getModel().getSize();
		WFSAttribute[] fields = new WFSAttribute[numSelected];
		for (int i=0 ; i<fields.length ; i++){
			fields[i] = ((WFSAttribute)getSelectedAttributesList().getModel().getElementAt(i));
		}
		return fields;
	}
}  //  @jve:decl-index=0:visual-constraint="10,22"
