/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */

/* CVS MESSAGES:
*
* $Id: ImportWebMapContextExtension.java,v 1.5 2006/08/29 07:56:15 cesar Exp $
* $Log: ImportWebMapContextExtension.java,v $
* Revision 1.5  2006/08/29 07:56:15  cesar
* Rename the *View* family of classes to *Window* (ie: SingletonView to SingletonWindow, ViewInfo to WindowInfo, etc)
*
* Revision 1.4  2006/08/29 07:13:43  cesar
* Rename class com.iver.andami.ui.mdiManager.View to com.iver.andami.ui.mdiManager.IWindow
*
* Revision 1.3  2006/07/21 10:31:05  jaume
* *** empty log message ***
*
* Revision 1.2  2006/05/03 11:10:54  jaume
* *** empty log message ***
*
* Revision 1.1  2006/05/03 07:51:21  jaume
* *** empty log message ***
*
* Revision 1.5  2006/05/02 16:12:12  jorpiell
* Se ha cambiado la interfaz Extension por dos clases: una interfaz (IExtension) y una clase abstract(Extension). A partir de ahora todas las extensiones deben heredar de Extension
*
* Revision 1.4  2006/05/02 15:57:43  jaume
* Few better javadoc
*
* Revision 1.3  2006/04/20 17:11:54  jaume
* Attempting to export
*
* Revision 1.2  2006/04/19 16:34:29  jaume
* *** empty log message ***
*
* Revision 1.1  2006/04/19 07:57:29  jaume
* *** empty log message ***
*
* Revision 1.1  2006/04/12 17:10:53  jaume
* *** empty log message ***
*
*
*/
package com.iver.cit.gvsig.wmc;

import java.awt.Component;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import com.iver.andami.PluginServices;
import com.iver.andami.plugins.Extension;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.cit.gvsig.ProjectExtension;
import com.iver.cit.gvsig.fmap.drivers.UnsupportedVersionException;
import com.iver.cit.gvsig.fmap.exceptions.ImportMapContextException;
import com.iver.cit.gvsig.fmap.layers.FLyrWMS;
import com.iver.cit.gvsig.gui.panels.WebMapContextFileChooserAccessory;
import com.iver.cit.gvsig.project.Project;
import com.iver.cit.gvsig.project.ProjectFactory;
import com.iver.cit.gvsig.project.ProjectView;

public class ImportWebMapContextExtension extends Extension {

	public void initialize() {

	}

	public void execute(String actionCommand) {
		if (actionCommand.equals("IMPORT")) {
			JFileChooser fc = new JFileChooser();
			fc.setFileFilter(new FileFilter() {
				public boolean accept(File f) {
					return f.isDirectory() || f.getAbsolutePath().toLowerCase().endsWith(WebMapContext.FILE_EXTENSION);
				}

				public String getDescription() {
					return PluginServices.getText(this, "ogc_mapcontext_file")+" (*.cml)";
				}
			});
			IWindow v = PluginServices.getMDIManager().getActiveWindow();

			// If the current active view is a gvSIG's view, we'll keep its name to
			// show it at the JFileChooser's accessory.
			String currentViewName;
			if (v instanceof com.iver.cit.gvsig.gui.View)
				currentViewName = ((com.iver.cit.gvsig.gui.View) v).getModel().getName();
			else
				currentViewName = null;

			// Create an accessory for the Web Map Context's file chooser.
			WebMapContextFileChooserAccessory acc = new WebMapContextFileChooserAccessory(currentViewName);

			// Add the accessory to the file chooser
			fc.setAccessory(acc);

			// Nothing else than Web Map Context files allowed
			fc.setAcceptAllFileFilterUsed(false);

			if (fc.showOpenDialog((Component) PluginServices.getMainFrame()) == JFileChooser.APPROVE_OPTION){
				readMapContextFile(fc.getSelectedFile(), acc.getSelectedView());
			}
			fc = null;
		}
	}

	/**
	 * Reads a WebMapContext (.cml) file and loads its layers into the destination
	 * view.
	 * @param wmcFile
	 * @param dstView
	 */
	private void readMapContextFile(File wmcFile, ProjectView dstView) {
		WebMapContext wmc = new WebMapContext();
		try {
			wmc.readFile(wmcFile);
		} catch (UnsupportedVersionException e) {
			JOptionPane.showMessageDialog(
					(Component) PluginServices.getMainFrame(),
					e.getMessage(),
					PluginServices.getText(this, "unsupported_version"),
					JOptionPane.ERROR_MESSAGE);
			return;
		} catch (ImportMapContextException e) {
			e.printStackTrace();
			if (e.isCritical()) {
				// import impossible: show message and quit
				JOptionPane.showMessageDialog(
						(Component) PluginServices.getMainFrame(),
						e.getMessage(),
						PluginServices.getText(this, "problems_encountered_while_importing"),
						JOptionPane.ERROR_MESSAGE);
				return;
			} else {
				JOptionPane.showMessageDialog(
						(Component) PluginServices.getMainFrame(),
						e.getMessage() + "\n\n" +
						PluginServices.getText(this, "edit_layer_properties_to_fix_them"),
						PluginServices.getText(this, "problems_encountered_while_importing"),
						JOptionPane.ERROR_MESSAGE);
			}
		}

		if (dstView == null) {
			// Since the destination view is null, a new one will be added to
			// the project.
			dstView = ProjectFactory.createView(null);
			dstView.setName(wmc.title);
			dstView.setComment("Created from WebMapContext file: "+wmcFile.getName());

			ProjectExtension pe = (ProjectExtension) PluginServices.getExtension(ProjectExtension.class);
			Project theProject = pe.getProject();
			theProject.addView(dstView);

			// show the view in a new window
			com.iver.cit.gvsig.gui.View v = null;
			v = new com.iver.cit.gvsig.gui.View();
			v.setModel(dstView);
			PluginServices.getMDIManager().addWindow(v);
		}

		for (int i = 0; i < wmc.layerList.size(); i++) {
			// WMS layers
			if (wmc.layerList.get(i) instanceof FLyrWMS) {
				FLyrWMS layer = (FLyrWMS) wmc.layerList.get(i);
				dstView.getMapContext().getLayers().addLayer(layer);
			}
		}
	}

	public boolean isEnabled() {
		return true;
	}

	public boolean isVisible() {
		return true;
	}

}
