/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.core.mdiManager;

import java.awt.Component;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Iterator;
import java.util.TreeMap;

import javax.swing.JComponent;
import javax.swing.JInternalFrame;

import com.iver.andami.ui.mdiManager.SingletonDialogAlreadyShownException;
import com.iver.andami.ui.mdiManager.SingletonWindow;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.andami.ui.mdiManager.WindowInfo;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision: 1.1 $
 */
public class SingletonWindowSupport {
	private static int singletonViewInfoID = 0;
	/** Hashtable que asocia contenido con vistas */
	private HashMap contentWindowInfo = new HashMap();
	private WindowInfoSupport vis;
	private FrameWindowSupport frameWindowSupport;
	private HashMap contentFrame = new HashMap();
	
	/**
	 * DOCUMENT ME!
	 *
	 * @param vis DOCUMENT ME!
	 * @param fvs
	 *
	 * @see com.iver.andami.ui.mdiManager.MDIManager#init(com.iver.andami.ui.mdiFrame.MDIFrame)
	 */
	public SingletonWindowSupport(WindowInfoSupport vis, FrameWindowSupport fvs) {
		this.vis = vis;
		this.frameWindowSupport = fvs;
	}

	/**
	 * Devuelve una referencia a la vista si ya est mostrada o null si la
	 * vista no ha sido aadida o ya fue cerrada
	 *
	 * @param windowClass DOCUMENT ME!
	 * @param model DOCUMENT ME!
	 * @param wi DOCUMENT ME!
	 *
	 * @return true si la vista existe ya y false si la vista no existe
	 *
	 * @throws SingletonDialogAlreadyShownException DOCUMENT ME!
	 */
	public boolean registerWindow(Class windowClass, Object model, WindowInfo wi) {
		//Se comprueba si la vista est siendo mostrada
		SingletonViewInfo svi = new SingletonViewInfo(windowClass, model);

		if (contentWindowInfo.containsKey(svi)) {
			if (wi.isModal()) {
				throw new SingletonDialogAlreadyShownException();
			}

			wi.setWindowInfo((WindowInfo)contentWindowInfo.get(svi));
			
			return true;
		} else {
			//La vista singleton no estaba mostrada
			//Se asocia el modelo con la vista
			contentWindowInfo.put(svi, wi);
			return false;
		}
	}

	public void openSingletonView(SingletonWindow sv, JComponent frame){
		SingletonViewInfo svi = new SingletonViewInfo(sv.getClass(), sv.getWindowModel());
		contentFrame.put(svi, frame);
	}
	
	public boolean contains(SingletonWindow sw){
		SingletonViewInfo svi = new SingletonViewInfo(sw.getClass(), sw.getWindowModel());
		return contentFrame.containsKey(svi);
	}
	
	/**
	 * DOCUMENT ME!
	 *
	 * @param s
	 */
	public void closeView(SingletonWindow s) {
		SingletonViewInfo svi = new SingletonViewInfo(s.getClass(), s.getWindowModel());
		WindowInfo viewInfo = (WindowInfo) contentWindowInfo.get(svi);
		JInternalFrame c = (JInternalFrame) contentFrame.get(svi); 
		viewInfo.setWidth(c.getWidth());
		viewInfo.setHeight(c.getHeight());
		viewInfo.setX(c.getX());
		viewInfo.setY(c.getY());
        
		contentFrame.remove(svi);
	}
	
	/**
	 * Representa una vista singleton manteniendo el modelo y la clase de la
	 * vista que lo muestra
	 *
	 * @author Fernando Gonzlez Corts
	 */
	public class SingletonViewInfo {
		
		public int id;
		
		/** Clase de la vista */
		public Class clase;

		/** Modelo que representa la vista */
		public Object modelo;
		
		/**
		 * Creates a new SingletonView object.
		 *
		 * @param clase Clase de la vista
		 * @param modelo Modelo que representa la vista
		 */
		public SingletonViewInfo(Class clase, Object modelo) {
			this.clase = clase;
			this.modelo = modelo;
			this.id = singletonViewInfoID;
			singletonViewInfoID++;
		}

		/**
		 * @see java.lang.Object#equals(java.lang.Object)
		 */
		public boolean equals(Object obj) {
			if (obj.getClass() != SingletonViewInfo.class) {
				throw new IllegalArgumentException();
			}

			SingletonViewInfo s = (SingletonViewInfo) obj;

			if ((clase == s.clase) && (modelo == s.modelo)) {
				return true;
			} else {
				return false;
			}
		}
	}

	private JInternalFrame getFrame(SingletonViewInfo svi){
		WindowInfo vi = (WindowInfo) contentWindowInfo.get(svi);
		return (JInternalFrame) contentFrame.get(svi);
	}
	
	public JInternalFrame getFrame(Class viewClass, Object model){
		SingletonViewInfo svi = new SingletonViewInfo(viewClass, model);
		return getFrame(svi);
	}

	/**
	 * @param model
	 * @return
	 */
	public JInternalFrame[] getFrames(Object model) {
		ArrayList ret = new ArrayList();
		
		ArrayList keys = contentFrame.getKeys();
		for (int i = 0; i < keys.size(); i++) {
			SingletonViewInfo svi = (SingletonViewInfo) keys.get(i);
			
			if (svi.modelo == model){
				ret.add(contentFrame.get(svi));
			}
		}
		
		return (JInternalFrame[]) ret.toArray(new JInternalFrame[0]);
	}

	/**
	 * @param view
	 * @return
	 */
	public JInternalFrame getFrame(SingletonWindow sv) {
		SingletonViewInfo svi = new SingletonViewInfo(sv.getClass(), sv.getWindowModel());
		return getFrame(svi);
	}

	/**
	 * @param sv
	 * @param i
	 */
	public void setX(SingletonWindow sv, int x) {
		JInternalFrame o = (JInternalFrame) contentFrame.get(new SingletonViewInfo(sv.getClass(), sv.getWindowModel()));

        if (o == null) return;
        o.setLocation(x, o.getY());
	}

	/**
	 * @param sv
	 * @param i
	 */
	public void setY(SingletonWindow sv, int y) {
		JInternalFrame o = (JInternalFrame) contentFrame.get(new SingletonViewInfo(sv.getClass(), sv.getWindowModel()));

        if (o == null) return;
        
        o.setLocation(o.getX(), y);
	}

	/**
	 * @param sv
	 * @param i
	 */
	public void setHeight(SingletonWindow sv, int height) {
		JInternalFrame o = (JInternalFrame) contentFrame.get(new SingletonViewInfo(sv.getClass(), sv.getWindowModel()));

        if (o == null) return;
            
        o.setSize(o.getWidth(), height);
	}

	/**
	 * @param sv
	 * @param i
	 */
	public void setWidth(SingletonWindow sv, int width) {
		JInternalFrame o = (JInternalFrame) contentFrame.get(new SingletonViewInfo(sv.getClass(), sv.getWindowModel()));

        if (o == null) return;
        o.setSize(width, o.getHeight());
	}

	/**
	 * @param sv
	 * @param string
	 */
	public void setTitle(SingletonWindow sv, String title) {
		JInternalFrame o = (JInternalFrame) contentFrame.get(new SingletonViewInfo(sv.getClass(), sv.getWindowModel()));

        if (o == null) return;
        o.setTitle(title);
	}
	
	private class HashMap {
	    private ArrayList keys = new ArrayList();
	    private ArrayList values = new ArrayList();
	    
	    public void put(SingletonViewInfo key, Object value) {
	        int index = -1;
	        for (int i = 0; i < keys.size(); i++) {
	            if (keys.get(i).equals(key)){
	                index = i;
	                break;
	            }
            }
	        
	        if (index != -1){
	            keys.add(index, key);
	            values.add(index, value);
	        }else{
	            keys.add(key);
	            values.add(value);
	        }
	    }
	    
	    public boolean containsKey(SingletonViewInfo key){
	        for (int i = 0; i < keys.size(); i++) {
	            if (keys.get(i).equals(key)){
	                return true;
	            }
	        }
	        
	        return false;
	    }
	    
	    public Object get(SingletonViewInfo key){
	        for (int i = 0; i < keys.size(); i++) {
	            if (keys.get(i).equals(key)){
	                return values.get(i);
	            }
	        }
	        
	        return null;
	    }
	    
	    public void remove(SingletonViewInfo key){
	        for (int i = 0; i < keys.size(); i++) {
	            if (keys.get(i).equals(key)){
	                keys.remove(i);
	                values.remove(i);
	            }
	        }
	    }
	    
	    public ArrayList getKeys(){
	        return keys;
	    }
	}
}
