/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;

import org.cresques.geo.GeoPoint;
import org.cresques.geo.Geodetic;
import org.cresques.geo.ProjPoint;
import org.cresques.geo.Projection;

import org.cresques.px.Extent;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;

import java.io.IOException;


/**
 * Soporte para ficheros georeferenciados con World File (.tfw)
 * @author "Luis W. Sevilla" <sevilla_lui@gva.es>
 * @author "Jos Morell Rama" <morell_josram@gva.es>
 */
public class TifGeoRefFile extends GeoRasterFile {
    geo.raster.GeoRasterFile geoRasterFile = null;
    Extent v = null;

    /**
     * @param name
     */
    public TifGeoRefFile(IProjection proj, String fName) {
        super(proj, null); // TODO aqui van las proyecciones
        fName = DataSource.normalize(fName);
        super.setName(fName);
        System.out.println("jmGeoRaster: " + fName);

        geoRasterFile = geo.raster.GeoRasterFile.createFile(fName);

        load();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.GeoRasterFile#load()
     */
    public GeoFile load() {
        try {
            geoRasterFile.open();

            if (!(proj instanceof Geodetic) && (proj instanceof Projection) &&
                    (geoRasterFile.getOriginX() >= -180.0) &&
                    (geoRasterFile.getOriginX() <= 180.0)) {
                GeoPoint gPt = new GeoPoint(geoRasterFile.getOriginX(),
                                            geoRasterFile.getOriginY());
                ProjPoint pt = (ProjPoint) getProjection().createPoint(0.0, 0.0);
                pt = (ProjPoint) ((Geodetic) getProjection()).fromGeo(gPt, pt);

                GeoPoint gPt2 = new GeoPoint(geoRasterFile.getOriginX() +
                                             geoRasterFile.getCellIncrementX(),
                                             geoRasterFile.getOriginY() +
                                             geoRasterFile.getCellIncrementY());
                ProjPoint pt2 = (ProjPoint) ((Geodetic) getProjection()).createPoint(0.0,
                                                                                     0.0);
                pt2 = (ProjPoint) ((Geodetic) getProjection()).fromGeo(gPt2, pt2);
                geoRasterFile.setOriginX(pt.getX());
                geoRasterFile.setOriginY(pt.getY());
                geoRasterFile.setCellIncrementX(pt2.getX() - pt.getX());
                geoRasterFile.setCellIncrementY(pt2.getY() - pt.getY());

                //geoRasterFile.computeExtent();
            }

            extent = new Extent(geoRasterFile.getExtent());
            System.out.println("jmGeoRaster: Extent=" + extent);
            geoRasterFile.setOriginRaster(new Point(0, 0));
            geoRasterFile.setGraphics2Doffset(new Point(0, 0));
        } catch (IOException e) {
            e.printStackTrace();
            geoRasterFile = null;
        }

        return this;
    }

    public int getWidth() {
        return geoRasterFile.getWidth();
    }

    public int getHeight() {
        return geoRasterFile.getHeight();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.GeoRasterFile#setView(org.cresques.px.Extent)
     */
    public void setView(Extent e) {
        v = new Extent(e);
    }

    public Extent getView() {
        return v;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.GeoRasterFile#setTransparency(boolean)
     */
    public void setTransparency(boolean t) {
        //transparency = t;	
    }

    public void setTransparency(int t) {
        //transparency = t;	
    }

    /* (non-Javadoc)
     * @see org.cresques.io.GeoRasterFile#updateImage(int, int, org.cresques.geo.ReProjection)
     */
    public Image updateImage(int width, int height, ICoordTrans rp) {
        double viewportScale = (double) width / v.width();
        Dimension sz = new Dimension(width, height);

        //return geoRasterFile.window(v.toRectangle2D(), viewportScale);
        return ((geo.raster.GeoRasterFile) geoRasterFile).updateImage(sz,
                                                                      v.toRectangle2D(),
                                                                      viewportScale);
    }

    /* (non-Javadoc)
     * @see org.cresques.io.GeoRasterFile#reProject(org.cresques.geo.ReProjection)
     */
    public void reProject(ICoordTrans rp) {
        // TODO Auto-generated method stub
    }

    /* (non-Javadoc)
     * @see org.cresques.io.GeoRasterFile#close()
     */
    public void close() {
    }

    /* (non-Javadoc)
     * @see org.cresques.io.GeoRasterFile#updateImage(int, int, org.cresques.cts.ICoordTrans, java.awt.Image, int)
     */
    public Image updateImage(int width, int height, ICoordTrans rp, Image img,
                             int flags) {
        // TODO Auto-generated method stub
        return updateImage(width, height, rp) ;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.GeoRasterFile#updateImage(int, int, org.cresques.cts.ICoordTrans, java.awt.Image, int)
     */
    public Image updateImage(int width, int height, ICoordTrans rp, Image img,
                             int origBand, int destBand) {
        // TODO Auto-generated method stub
        return updateImage(width, height, rp) ;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.GeoRasterFile#getData(int, int)
     */
    public Object getData(int x, int y, int band) {
        // TODO Auto-generated method stub
        return null;
    }

    public int getBlockSize() {
        return 0;
    }

    public byte[] getWindow(int ulX, int ulY, int sizeX, int sizeY, int band) {
        return null;
    }
    
    public void setExtentTransform(double originX, double originY, double w, double f, double psX, double psY) {}
}
