/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.cts;

import org.cresques.cts.IProjection;
import org.cresques.cts.ProjectionPool;


/**
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 */
public class CSSelectionModel {
    public static int NONE = 0x01;
    public static int GEODETIC = 0x02;
    public static int TRANSVERSAL = 0x03;
    public String[] datumList = {
                                    "WGS 84", "European 1950", "Datum 73",
                                    "North American Datum 1927",
                                    "North American Datum 1983",
                                    "La Canoa" /*,
    "Nouvelle Triangulation Francaise",
    */
                                };
    private String[] projList = {
                                    "Geodesica",
                                    "(UTM) Universal Transversal Mercator",
                                    "Datum 73 / Modified Portuguese Grid",
                                    "WGS 84 / LCC Canada",
                                    "NAD83 / NRCan LCC Canada"
                                };
    private int[] projType = { GEODETIC, TRANSVERSAL, NONE, NONE, NONE };
    private String[] huseList = {  };
    private int selectedDatum = 0;
    private int selectedProj = 0;
    private int selectedHuse = 0;

    /**
     *
     */
    public CSSelectionModel() {
        super();
        huseList = new String[60];

        for (int i = 1; i <= 60; i++)
            huseList[i - 1] = "Huso " + Integer.toString(i);
    }

    public String[] getDatumList() {
        return datumList;
    }

    public String[] getProjectionList() {
        return projList;
    }

    public String[] getHuseList() {
        return huseList;
    }

    public void setSelectedDatum(int opNr) {
        selectedDatum = opNr;
    }

    public void setSelectedDatum(Object item) {
        String[] list = datumList;

        for (int i = 0; i < list.length; i++)
            if (list[i].compareTo((String) item) == 0) {
                selectedDatum = i;
            }
    }

    public int getSelectedDatum() {
        return selectedDatum;
    }

    public void setSelectedProj(int opNr) {
        selectedProj = opNr;
    }

    public void setSelectedProj(Object item) {
        String[] list = projList;

        for (int i = 0; i < list.length; i++)
            if (list[i].compareTo((String) item) == 0) {
                selectedProj = i;
            }
    }

    public int getSelectedProj() {
        return selectedProj;
    }

    public int getSelectedProjType() {
        return projType[selectedProj];
    }

    public void setSelectedHuse(int opNr) {
        selectedHuse = opNr;
    }

    public void setSelectedHuse(Object item) {
        String[] list = huseList;

        for (int i = 0; i < list.length; i++)
            if (list[i].compareTo((String) item) == 0) {
                selectedHuse = i;
            }
    }

    public int getSelectedHuse() {
        return selectedHuse;
    }

    public void setProjection(IProjection proj) {
        String key = proj.getAbrev();
        key = key.substring(key.length() - 5);

        if (key.endsWith(":4326")) {
            setSelectedDatum(0);
            setSelectedProj(0);
            setSelectedHuse(-1);
        } else if (key.endsWith(":4230")) {
            setSelectedDatum(1);
            setSelectedProj(0);
            setSelectedHuse(-1);
        } else if (key.endsWith(":4267")) {
            setSelectedDatum(2);
            setSelectedProj(0);
            setSelectedHuse(-1);
        } else if (key.endsWith(":4269")) {
            setSelectedDatum(3);
            setSelectedProj(0);
            setSelectedHuse(-1);
        } else if (key.endsWith(":4247")) {
            setSelectedDatum(5);
            setSelectedProj(0);
            setSelectedHuse(-1);
        } else if (key.startsWith("326")) {
            setSelectedDatum(0);
            setSelectedProj(1);
            setSelectedHuse(Integer.parseInt(key.substring(3)) - 1);
        } else if (key.startsWith("230")) {
            setSelectedDatum(1);
            setSelectedProj(1);
            setSelectedHuse(Integer.parseInt(key.substring(3)) - 1);
        } else if (key.startsWith("267")) {
            setSelectedDatum(2);
            setSelectedProj(1);
            setSelectedHuse(Integer.parseInt(key.substring(3)) - 1);
        } else if (key.startsWith("269")) {
            setSelectedDatum(3);
            setSelectedProj(1);
            setSelectedHuse(Integer.parseInt(key.substring(3)) - 1);
        } else if (key.startsWith("247")) {
            setSelectedDatum(3);
            setSelectedProj(1);
            setSelectedHuse(Integer.parseInt(key.substring(3)) - 1);
        } else if (key.startsWith("27492")) { // Datum 73 / Modified Portuguese Grid
            setSelectedDatum(-1);
            setSelectedProj(2);
            setSelectedHuse(-1);
        } else if (key.startsWith("42101")) { // WGS 84 / LCC Canada
            setSelectedDatum(-1);
            setSelectedProj(3);
            setSelectedHuse(-1);
        } else if (key.startsWith("42304")) { // NAD83 /  / LCC Canada
            setSelectedDatum(-1);
            setSelectedProj(4);
            setSelectedHuse(-1);
        } else {
            System.err.println("CAGADA EN EL PARSING DE LA PROYECCION: " + key);
        }
    }

    public IProjection getProjection() {
        if (selectedProj == 2) {
            return ProjectionPool.get("EPSG:27492");
        } else if (selectedProj == 3) {
            return ProjectionPool.get("EPSG:42101");
        } else if (selectedProj == 4) {
            return ProjectionPool.get("EPSG:42304");
        }

        IProjection proj = null;
        String datum = "326";

        if (selectedDatum == 0) {
            datum = "326";
        } else if (selectedDatum == 1) {
            datum = "230";
        } else if (selectedDatum == 2) {
            datum = "267";
        } else if (selectedDatum == 3) {
            datum = "269";
        } else if (selectedDatum == 5) {
            datum = "247";
        }

        if (selectedProj == 0) {
            return ProjectionPool.get("EPSG:4" + datum);
        } else if (selectedProj == 1) {
            String huse = Integer.toString(selectedHuse + 1);

            if (selectedHuse < 9) {
                huse = "0" + huse;
            }

            if ((selectedDatum == 2) || (selectedDatum == 3)) {
                if (selectedHuse < 3) {
                    huse = "03";
                }

                if (selectedHuse > 23) {
                    huse = "23";
                }
            }

            if (selectedDatum == 53) {
                if (selectedHuse < 18) {
                    huse = "18";
                }

                if (selectedHuse > 22) {
                    huse = "22";
                }
            }

            return ProjectionPool.get("EPSG:" + datum + huse);
        }

        return proj;
    }
}
