/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.ui.raster;

import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.TreeMap;

import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.cresques.filter.IRasterOperationsRegistrablePanels;
import org.cresques.filter.Transparency.TransparencyRange;
import org.cresques.io.GeoRasterFile;
import org.cresques.ui.DefaultTabPanel;
import org.cresques.ui.BrightnessContrast.EnhancedBrightnessContrastPanel;


/**
 * Panel que contiene todos los paneles de los tabs del cuadro de dialogo
 * de bandas y transparencias
 *
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class FilterRasterDialogPanel extends DefaultTabPanel
    implements MouseListener, FocusListener, KeyListener, ChangeListener, ComponentListener {
    final private static long serialVersionUID = -3370601314380922368L;

    /**
     * Tamao en X del panel interior.El tab de dentro es proporcional
     */
    protected int sizePanelX = 460;
    
    /**
     * Tamao en Y del panel interior. El tab de dentro es proporcional
     */
    protected int sizePanelY = 275;
    
    /**
     * Propiedades guardadas en una matriz Nx2 con la forma propiedad/valor.
     */
    public Object[][] props = null;
    
    /**
     * Nmero de bandas.
     */
    protected int nbands = 0;
    
    /**
     * Lista de georrasterfiles correspondiente a los ficheros de bandas
     */
    protected GeoRasterFile[] grf = null;
    private TreeMap shadePanel = new TreeMap();
    
    //Paneles que se registran dentro de esta misma clase
    private RasterTransparencyPanel 			pTrans = null;
    private BandSetupPanel						pSetup = null;
    private InfoPanel 							pInfo = null;
    private EnhancedBrightnessContrastPanel 	pEnhancedBrightnessContrast = null;
    
	    
    /**
     * Constructor. Inicializa los paneles y propiedades
     * @param props        Propiedades
     */
    public FilterRasterDialogPanel(Object[][] props) {
        super();
        init(props);
    }

    /**
     * Constructor
     */
    public FilterRasterDialogPanel() {
        super();
    }

    /**
     * Inicializa las propiedades
     * @param props        Propiedades. Guardadas en una matriz Nx2 con la forma proiedad/valor
     */
    public void init(Object[][] props) {
        contentPane.setPreferredSize(new java.awt.Dimension(sizePanelX + this.difWidth,
                                                            sizePanelY + this.difHeight));
        this.tabbedPane.setPreferredSize(new java.awt.Dimension(sizePanelX + this.difWidth,
                                                                sizePanelY + difHeight - 5));
        this.pButton.setPreferredSize(new java.awt.Dimension(sizePanelX + this.difWidth, 25));
        //this.buttonPane.setPreferredSize(new java.awt.Dimension(200,25));
        
        this.props = props;
        initPanels();
        
    }

    /**
     * This method initializes this
     *
     * @return void
     */
    protected void initPanels() {
        this.setBounds(0, 0, 355, 230);
        pTrans = new RasterTransparencyPanel(this);
        pSetup = new BandSetupPanel();
        pInfo = new InfoPanel(this);
        pEnhancedBrightnessContrast = new EnhancedBrightnessContrastPanel(this);
        
        this.addTab(pInfo.getName(), pInfo);
        this.addTab(pSetup.getName(), pSetup);
        this.addTab(pTrans.getName(), pTrans);
        this.addTab(pEnhancedBrightnessContrast.getName(), pEnhancedBrightnessContrast);
   
        //pTrans.getOpacitySlider().addChangeListener(this);
        //pTrans.addFocusListener(this);
        this.getTab().addFocusListener(this);
    }

    /**
     * Asigna la visibilidad de un tab a verdadero o falso. La
     * seleccin del tab se hace por el identificador.
     * @param tab	Identificador del tab. Variable nom del mismo
     * @param active	True o false para visible o invisible.
     */
    public void setTabVisible(String tab, boolean active){
    	if(active == false){
	    	for(int i=0; i<this.getTab().getTabCount(); i++){
	    		if(tab.equals( ((JPanel)this.getTab().getComponentAt(i)).getName() )){
	    			shadePanel.put(tab, this.getTab().getComponentAt(i));
	    			this.getTab().removeTabAt(i);
	    		}
	    	}
    	}else
    		this.addTab(tab, ((JPanel)shadePanel.get(tab)) );
    }
    
    /**
     * Selecciona el panel indicado por index
     * @param index        panel seleccionado
     */
    public void setSelectedIndex(int index) {
        tabbedPane.setSelectedIndex(index);
    }

    /**
     * Obtiene un panel a travs del nombre de su clase
     * @param name Nombre de la clase del panel
     */
    public JPanel getPanelByClassName(String name){
    	for(int i=0;i<super.getTab().getTabCount() ;i++){
    		JPanel p = (JPanel)super.getTab().getComponentAt(i);
    		if(p.getClass().getName().endsWith(name))
    			return p;
    	}
    	return null;
    }

    /**
     * Asigna valores para los rangos.
     * @param ranges
     */
    public void setRanges(ArrayList ranges) {      
      if(ranges != null){    	  
    	  TransparencyByPixelPanel panel = (TransparencyByPixelPanel)pTrans.getPTranspByPixel();
    	  pTrans.getTransparencyCheck().setSelected(true);
    	  panel.setControlEnabled(true);
    	  panel.setEntries(ranges);
	      for(int i=0;i<ranges.size();i++)
	    	  panel.getListModel().addElement( ((TransparencyRange)ranges.get(i)).getStrEntry() );
      }
    }

    /**
     * Aade las bandas de los georrasterfile a los paneles 
     * que lo necesitan
     * @param files
     */
    public void addFiles(GeoRasterFile[] files){
    	for(int i=0;i<this.getTab().getTabCount();i++){
    		if(this.getTab().getComponentAt(i) instanceof IRasterOperationsRegistrablePanels)
    		 	((IRasterOperationsRegistrablePanels)getTab().getComponentAt(i)).addFiles(files);
    	}
    		
        if(pSetup != null)
        	pSetup.addFiles(files);
        if(pInfo != null)
            pInfo.addFiles(files);
        
        	
 
    }
    
    public void addInfoFiles(GeoRasterFile[] files){
    	if(pInfo != null){
    		pInfo.setBands(pSetup.getAssignedBand(GeoRasterFile.RED_BAND), pSetup.getAssignedBand(GeoRasterFile.GREEN_BAND), pSetup.getAssignedBand(GeoRasterFile.BLUE_BAND));
    		pInfo.addFiles(files);
    	}
    		
    	 
    }
    
    /**
     * Elimina las bandas de los georrasterfile a los paneles
     * @param file Banda a eliminar
     */
    public void removeFile(String file) {
    	for(int i=0;i<this.getTab().getTabCount();i++){
    		if(this.getTab().getComponentAt(i) instanceof IRasterOperationsRegistrablePanels)
    		 	((IRasterOperationsRegistrablePanels)(getTab().getComponentAt(i))).removeFile(file);
    	}
        if(pSetup != null)
        	pSetup.removeFile(file);
    }
    
    public void focusGained(FocusEvent e) {
        //pTrans.updateTextBox();
    }

    public void focusLost(FocusEvent e) {
        //checkOpacityText();
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public void keyTyped(KeyEvent e) {
    }

    public void keyPressed(KeyEvent e) {
    }

    public void keyReleased(KeyEvent e) {
    }

    public void stateChanged(ChangeEvent e) {
    	 contentPane.setPreferredSize(new java.awt.Dimension(sizePanelX + this.difWidth,
                 sizePanelY + this.difHeight));
    	 this.tabbedPane.setPreferredSize(new java.awt.Dimension(sizePanelX + this.difWidth,
                 sizePanelY + difHeight - 5));
    	 this.pButton.setPreferredSize(new java.awt.Dimension(sizePanelX + this.difWidth, 25));

    }
    
    public void componentResized(ComponentEvent e){
    	super.componentResized(e);
    	contentPane.setPreferredSize(new java.awt.Dimension(sizePanelX,
                sizePanelY));
   	 	this.tabbedPane.setPreferredSize(new java.awt.Dimension(sizePanelX,
                sizePanelY));
   	 	this.pButton.setPreferredSize(new java.awt.Dimension(sizePanelX , 25));
     
    	for(int i=0;i<super.getTab().getTabCount() ;i++){
			JPanel p = (JPanel)super.getTab().getComponentAt(i);
			
			if(p instanceof IResize){
				((IResize)p).setComponentSize(this.getWidth() - 20, this.getHeight() - 108);	
			}	
    	}
    }
}
