package com.iver.cit.gvsig.fmap.drivers.gml;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Date;

import org.apache.log4j.Logger;
import org.gvsig.remoteClient.gml.factories.IGeometriesFactory;

import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.cit.gvsig.fmap.core.DefaultFeature;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.ShapeFactory;
import com.iver.cit.gvsig.fmap.core.v02.FConverter;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.io.gml2.GMLReader;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: FMAPGeometryFactory.java,v 1.1 2006/08/10 12:03:43 jorpiell Exp $
 * $Log: FMAPGeometryFactory.java,v $
 * Revision 1.1  2006/08/10 12:03:43  jorpiell
 * Se usa el nuevo driver de GML de remoteServices
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class FMAPGeometryFactory implements IGeometriesFactory{
	private static Logger logger = Logger.getLogger(FMAPGeometryFactory.class);
	
	public Object createPoint2D(double x, double y){
		return ShapeFactory.createPoint2D(x,y);
	}

	public Object createPoint2D(Point2D point2D) {
		return createPoint2D(point2D.getX(),point2D.getY());
	}
	
	public Object createMultipoint2D(double[] x, double[] y){
		return ShapeFactory.createMultipoint2D(x,y);
	}
	
	public Object createSimpleFeature(String element, Object geometry, ArrayList params, ArrayList values) {
		Object[] attr = null;
		Value[] value = getValues(values);
		IFeature feature = new DefaultFeature((IGeometry)geometry,value);
		return new FeatureWithAttributes(feature,params,value);
	}

	public Object createGeometry(String gmlGeometry) {
		GMLReader reader = new GMLReader();
		try {
			Geometry geom = reader.read(gmlGeometry,new GeometryFactory());
			return ShapeFactory.createGeometry(FConverter.jts_to_java2d(geom));
		} catch (Exception e){
			logger.error("Can't parse: " + gmlGeometry,e);
		}
		return null;
	}
	
	/**
	 * Gets the attributes
	 * @param attr
	 * @return
	 */
	private Value[] getValues(ArrayList attr) {	
		Value[] values = new Value[attr.size()];
		
		for (int i=0;i<attr.size();i++){
			if (attr.get(i)!=null){
				if (attr.get(i) instanceof Double){
					values[i]=ValueFactory.createValue(((Double)attr.get(i)).doubleValue());
				}else if (attr.get(i) instanceof String){
					values[i]=ValueFactory.createValue(String.valueOf(attr.get(i)));
				}else if (attr.get(i) instanceof Long){
					values[i]=ValueFactory.createValue(((Long)attr.get(i)).longValue());
				}else if (attr.get(i) instanceof Integer){
					values[i]=ValueFactory.createValue(((Integer)attr.get(i)).intValue());
				}else if (attr.get(i) instanceof Float){
					values[i]=ValueFactory.createValue(((Float)attr.get(i)).floatValue());
				}else if (attr.get(i) instanceof Short){
					values[i]=ValueFactory.createValue(((Short)attr.get(i)).shortValue());
				}else if (attr.get(i) instanceof Boolean){
					values[i]=ValueFactory.createValue(((Boolean)attr.get(i)).booleanValue());
				}else if (attr.get(i) instanceof Date){
					values[i]=ValueFactory.createValue(((Date)attr.get(i)));
				}					
			}else{
				values[i]=ValueFactory.createValue("");
			}
		}
		return values;
	}
	
	public class FeatureWithAttributes{
		private IFeature feature;
		private ArrayList attributeName;
		private Value[] attributeValue;
		
		public FeatureWithAttributes(IFeature feature, ArrayList attributeName, Value[] attributeValue) {
			super();		
			this.feature = feature;
			this.attributeName = attributeName;
			this.attributeValue = attributeValue;
		}

		/**
		 * @return Returns the attributeName.
		 */
		public ArrayList getAttributeName() {
			return attributeName;
		}

		/**
		 * @return Returns the feature.
		 */
		public IFeature getFeature() {
			return feature;
		}

		/**
		 * @return Returns the attributeValue.
		 */
		public Value[] getAttributeValue() {
			return attributeValue;
		}
		
	}

}
