package com.iver.cit.gvsig.fmap.drivers.gml;

import java.awt.geom.Point2D;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.NoSuchElementException;
import java.util.Vector;

import org.cresques.cts.ICoordTrans;
import org.geotools.data.FeatureReader;
import org.geotools.feature.AttributeType;
import org.geotools.feature.Feature;
import org.geotools.feature.IllegalAttributeException;

import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.DefaultFeature;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.ShapeFactory;
import com.iver.cit.gvsig.fmap.core.v02.FConverter;
import com.iver.cit.gvsig.fmap.drivers.IFeatureIterator;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GMLFeaturesIterator.java,v 1.2 2006/07/24 07:36:40 jorpiell Exp $
 * $Log: GMLFeaturesIterator.java,v $
 * Revision 1.2  2006/07/24 07:36:40  jorpiell
 * Se han hecho un cambio en los nombres de los metodos para clarificar
 *
 * Revision 1.1  2006/07/19 12:29:39  jorpiell
 * Aadido el driver de GML
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class GMLFeaturesIterator implements IFeatureIterator{
	protected FeatureReader featureReader = null;
	private Vector featureVector = new Vector();
	private boolean parserException = false;
	private boolean finished = false;
	private int position = -1;
	private int length = -2;
	protected ICoordTrans coordTrans;	
	
	public GMLFeaturesIterator(FeatureReader featureReader){
		this.featureReader = featureReader;		
	}
	
	/**
	 * Gets the next feature
	 * @return
	 * @throws DriverException
	 */
	public IFeature next() throws DriverException {
		if (!finished){
			Feature feature = null;
			try {
				feature = featureReader.next();
			} catch (NoSuchElementException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			} catch (IllegalAttributeException e) {
				e.printStackTrace();
			}
			Object[] attr = null;
			attr = feature.getAttributes(attr);
			Value[] values = getValues(attr);
			
			try{
				IGeometry g = ShapeFactory.createGeometry(FConverter.jts_to_java2d(feature.getDefaultGeometry()));
				if (coordTrans != null){
					g.reProject(coordTrans);
				}
				IFeature outPutFeature = new DefaultFeature(g,values,feature.getID());
				featureVector.add(outPutFeature);
				return outPutFeature;
			}catch (NullPointerException e){
				//The geometry can't be created
			}
		}else{
			return (DefaultFeature)featureVector.get(position);			
		}
		return null;
	}
	
	/**
	 * Returns if there is a new feature to retrieve
	 * @return
	 */
	public boolean hasNext(){
		if (length == -2){
			try {
				if (featureReader.hasNext()){
					position++;
					return true;
				}else{
					featureReader.close();
					finished = true;
					length = featureVector.size() - 1;
					return false;
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}else{
			if (position != length){
				position++;
				return true;
			}else{
				return false;
			}
		}
		return false;
	}
	
	/**
	 * Gets the attributes
	 * @param attr
	 * @return
	 */
	private Value[] getValues(Object[] attr) {	
		Value[] values = null;
		if (attr.length - 1 > 0){
			values = new Value[attr.length - 1];
		}else{
			values = new Value[0];
		}
		for (int i=1;i<attr.length;i++){
			if (attr[i]!=null){
				if (attr[i] instanceof Double){
					values[i-1]=ValueFactory.createValue(((Double)attr[i]).doubleValue());
				}else if (attr[i] instanceof String){
					values[i-1]=ValueFactory.createValue(String.valueOf(attr[i]));
				}else if (attr[i] instanceof Long){
					values[i-1]=ValueFactory.createValue(((Long)attr[i]).longValue());
				}else if (attr[i] instanceof Integer){
					values[i-1]=ValueFactory.createValue(((Integer)attr[i]).intValue());
				}else if (attr[i] instanceof Float){
					values[i-1]=ValueFactory.createValue(((Float)attr[i]).floatValue());
				}else if (attr[i] instanceof Short){
					values[i-1]=ValueFactory.createValue(((Short)attr[i]).shortValue());
				}else if (attr[i] instanceof Boolean){
					values[i-1]=ValueFactory.createValue(((Boolean)attr[i]).booleanValue());
				}else if (attr[i] instanceof Date){
					values[i-1]=ValueFactory.createValue(((Date)attr[i]));
				}					
			}else{
				values[i-1]=ValueFactory.createValue("");
			}
		}
		return values;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.drivers.IFeatureIterator#closeIterator()
	 */
	public void closeIterator() throws DriverException {
		// TODO Auto-generated method stub		
	}
	
	/**
	 * This method must be executed before to start an iteration
	 * process
	 */
	public void startIteration(){
		position = -1;		
	}
	
	/**
	 * @return the coordTrans
	 */
	public ICoordTrans getCoordTrans() {
		return coordTrans;
	}

	/**
	 * @return Returns the parserException.
	 */
	public boolean isParserException() {
		return parserException;
	}

	/**
	 * @param parserException The parserException to set.
	 */
	public void setParserException(boolean parserException) {
		this.parserException = parserException;
	}
	
	/**
	 * Return the attributes type
	 *
	 */
	public AttributeType[] getAttributeTypes(){
		try{
			return featureReader.getFeatureType().getAttributeTypes();
		}catch(NullPointerException e){
			return new AttributeType[0];
		}
	}
	
	/**
	 * Returns the attributes name
	 * @return
	 */
	public ArrayList getAttributeNames(){
		AttributeType[] attr = getAttributeTypes();
		ArrayList arrayFields = new ArrayList();
		for (int i=0 ; i<attr.length ; i++){
			if (!(GMLUtils.isGeometry(attr[i].getType()))){
				arrayFields.add(attr[i].getName());
			}
		}
		return arrayFields;
	}

	/**
	 * @param coordTrans The coordTrans to set.
	 */
	public void setCoordTrans(ICoordTrans coordTrans) {
		this.coordTrans = coordTrans;
	}

}
