package com.iver.cit.gvsig.fmap.drivers.gml;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Vector;

import org.geotools.data.FeatureReader;
import org.geotools.data.wfs.WFSFeatureReader;
import org.geotools.data.wfs.WFSTransactionState;
import org.geotools.factory.FactoryConfigurationError;
import org.geotools.feature.AttributeType;
import org.geotools.feature.AttributeTypeFactory;
import org.geotools.feature.FeatureType;
import org.geotools.feature.FeatureTypeBuilder;
import org.geotools.feature.SchemaException;
import org.geotools.xml.gml.GMLFeatureCollection;
import org.xml.sax.SAXException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
/* CVS MESSAGES:
*
* $Id: GMLFeaturesParser.java,v 1.2 2006/07/24 07:36:40 jorpiell Exp $
* $Log: GMLFeaturesParser.java,v $
* Revision 1.2  2006/07/24 07:36:40  jorpiell
* Se han hecho un cambio en los nombres de los metodos para clarificar
*
* Revision 1.1  2006/07/19 12:29:39  jorpiell
* Aadido el driver de GML
*
*
*/
/**
 * It parses a getFeature request file
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class GMLFeaturesParser{
	private static int buffer  = 100000000;
	private static int timeout = 100000000;
	
	/**
	 * Return an iterrator of features
	 * @param file
	 * File to parse
	 * @param currentSRS
	 * Curerent SRS (to reproject)
	 * @return
	 * Iterator
	 * @throws Exception
	 */
	public static GMLFeaturesIterator getFeatureReader(File file) throws Exception{
		FileInputStream is = new FileInputStream(file);
			
		FeatureReader featureReader = getFeatureReader(is,
				buffer,
				timeout,
				null,
				null);			
			
		return new GMLFeaturesIterator(featureReader);
	}
	
	/**
	 * Gets a feature reader
	 * @param is
	 * WFS getFeature XML File
	 * @param bufferSize
	 * buffer size
	 * @param timeout
	 * Request timeout
	 * @param transaction
	 * @param ft
	 * Feature schema
	 * @return
	 * @throws SAXException 
	 */
	public static FeatureReader getFeatureReader(InputStream is,
	        int bufferSize, int timeout, WFSTransactionState transaction, FeatureType ft) throws SAXException{
    	return WFSFeatureReader.getFeatureReader(is, bufferSize,
    			timeout, transaction, ft);
    }

	
	
}
