package com.iver.cit.gvsig.fmap.edition.writers.gml;

import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.cresques.cts.IProjection;
import org.geotools.feature.FeatureType;

import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.drivers.ITableDefinition;
import com.iver.cit.gvsig.fmap.drivers.LayerDefinition;
import com.iver.cit.gvsig.fmap.drivers.gml.GMLTypesConversor;
import com.iver.cit.gvsig.fmap.edition.EditionException;
import com.iver.cit.gvsig.fmap.edition.IRowEdited;
import com.iver.cit.gvsig.fmap.edition.ISpatialWriter;
import com.iver.cit.gvsig.fmap.edition.writers.AbstractWriter;
import com.iver.utiles.StringUtilities;
import com.vividsolutions.jts.geom.Geometry;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GMLWriter.java,v 1.5 2006/08/25 13:04:07 fjp Exp $
 * $Log: GMLWriter.java,v $
 * Revision 1.5  2006/08/25 13:04:07  fjp
 * Comprobar que se puede editar antes de ponerte en edicion
 *
 * Revision 1.4  2006/08/10 12:03:43  jorpiell
 * Se usa el nuevo driver de GML de remoteServices
 *
 * Revision 1.3  2006/08/02 11:22:56  fjp
 * Empieza a funcionar con PostGIS lo de aadir campos
 *
 * Revision 1.2  2006/07/21 08:57:28  jorpiell
 * Se ha aadido la exportacin de capas de puntos
 *
 * Revision 1.1  2006/07/19 12:29:39  jorpiell
 * Aadido el driver de GML
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class GMLWriter extends AbstractWriter implements ISpatialWriter{
	private File m_File = null;	
	private FeatureType featureType = null;
	private GMLFileCreator gmlFile = null;	
	private GMLSchemaCreator gmlSchemaFileCreator = null;
		
	/**
	 * Sets the output file
	 * @param f
	 */
	public void setFile(File f) {
		m_File = f;
	}
	
	/**
	 * Sets the schema definition
	 * @param lyrDef
	 * @throws EditionException 
	 * @throws Exception 
	 */
	public void setSchema(LayerDefinition lyrDef) throws EditionException{
		try {
			featureType = GMLTypesConversor.featureTypefromGvSIGToGeotools(lyrDef);
			
			String schemaPath = m_File.getName().toLowerCase().replaceAll("\\.gml", ".xsd");
						
			gmlFile = new GMLFileCreator(m_File,schemaPath);
			gmlFile.setAttibutes(lyrDef.getFieldsDesc());
			
			schemaPath = m_File.getAbsolutePath().toLowerCase().replaceAll("\\.gml", ".xsd");
			
			gmlSchemaFileCreator = new GMLSchemaCreator(new File(schemaPath));
			gmlSchemaFileCreator.createFile(lyrDef);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new EditionException(e.toString());
		}
	}
	
	/**
	 * Sets the geometry extend
	 * @param extend
	 * @param proj
	 */
	public void setBoundedBy(Rectangle2D extend,IProjection proj){
		gmlFile.setBoundedBy(extend,proj);
	}
	
	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.edition.IWriter#canWriteAttribute(int)
	 */
	public boolean canWriteAttribute(int sqlType) {
		return true;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.edition.ISpatialWriter#canWriteGeometry(int)
	 */
	public boolean canWriteGeometry(int gvSIGgeometryType) {
		return true;
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.edition.ISpatialWriter#setFlatness(double)
	 */
	public void setFlatness(double flatness) {
		// TODO Auto-generated method stub
		
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.edition.IWriter#preProcess()
	 */
	public void preProcess() throws EditionException {
		if (gmlFile == null){
			new EditionException("The driver must be initialized using the setSchema method");
		}
		try {
			gmlFile.initialize();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			new EditionException(e.toString());
		}
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.edition.IWriter#process(com.iver.cit.gvsig.fmap.edition.IRowEdited)
	 */
	public void process(IRowEdited row) throws EditionException {
		Geometry geom =  ((IFeature)row.getLinkedRow()).getGeometry().toJTSGeometry();
		try {			
			gmlFile.insertFeature(geom,row.getAttributes());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new EditionException(e.toString()); 
		}					
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.edition.IWriter#postProcess()
	 */
	public void postProcess() throws EditionException {
		try {
			gmlFile.writeFile();
			gmlSchemaFileCreator.writeFile();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new EditionException(e.toString()); 
		}		
	}

	/*
	 *  (non-Javadoc)
	 * @see com.iver.cit.gvsig.fmap.edition.IWriter#initialize(com.iver.cit.gvsig.fmap.drivers.ITableDefinition)
	 */
	public void initialize(ITableDefinition tableDefinition) throws EditionException {
		super.initialize(tableDefinition);
		
	}

	/*
	 *  (non-Javadoc)
	 * @see com.hardcode.driverManager.Driver#getName()
	 */
	public String getName() {
		return "GML Writer";
	}

	public boolean canAlterTable() {
		// TODO Auto-generated method stub
		return false;
	}
	
	public boolean canSaveEdits() {
		if (m_File.canWrite()) return true;
		return false;
	}



}
