/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.rendering;

import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.instruction.FieldNotFoundException;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;

import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.SLDTags;
import com.iver.cit.gvsig.fmap.core.SLDUtils;
import com.iver.cit.gvsig.fmap.core.v02.FConstant;
import com.iver.cit.gvsig.fmap.core.v02.FSymbol;
import com.iver.cit.gvsig.fmap.layers.XMLException;

import com.iver.utiles.StringUtilities;
import com.iver.utiles.XMLEntity;

import java.awt.Color;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.TreeMap;

import org.geotools.filter.ExpressionBuilder;
import org.geotools.filter.Filter;
import org.geotools.styling.FeatureTypeStyle;
import org.geotools.styling.NamedLayer;
import org.geotools.styling.Rule;
import org.geotools.styling.SLDTransformer;
import org.geotools.styling.Style;
import org.geotools.styling.StyleBuilder;
import org.geotools.styling.StyleFactory;
import org.geotools.styling.StyledLayerDescriptor;
import org.geotools.styling.Symbolizer;


/**
 * Leyenda Vectorial por intervalos.
 *
 * @author Vicente Caballero Navarro
 */
public class VectorialIntervalLegend implements IntervalLegend, VectorialLegend {
    public final static int EQUAL_INTERVALS = 0;
    public final static int NATURAL_INTERVALS = 1;
    public final static int QUANTILE_INTERVALS = 2;
    private TreeMap symbols = new TreeMap(new Comparator() {
                public int compare(Object o1, Object o2) {
                    if ((o1 != null) && (o2 != null)) {
                       if (o1 instanceof NullIntervalValue &&
                                o2 instanceof NullIntervalValue) {
                            return 0;
                        }

                        if (o2 instanceof NullIntervalValue) {
                            return 1;
                        }

                        if (o1 instanceof NullIntervalValue) {
                            return -1;
                        }

                        FInterval i2 = (FInterval) o2;
                        FInterval i1 = (FInterval) o1;

                        if (i1.getMin() > i2.getMin()) {
                            return 1;
                        }

                        if (i1.getMin() < i2.getMin()) {
                            return -1;
                        }
                    }

                    return 0;
                }
            }); // Para poder ordenar
    private ArrayList keys = new ArrayList(); // En lugar de un HashSet, para tener acceso por ndice
    private int index = 0;
    private String fieldName;
    private int fieldId;
    private String labelFieldName;
    private String labelFieldHeight;
    private String labelFieldRotation;
    private FSymbol defaultSymbol;
    private DataSource dataSource;
    private Color startColor = Color.red;
    private Color endColor = Color.blue;
    private int shapeType;
    private int intervalType = NATURAL_INTERVALS;
    private boolean useDefaultSymbol = false;
    // private boolean bWithHeightText;

    /**
     * Crea un nuevo VectorialIntervalLegend.
     */
    public VectorialIntervalLegend() {
        //defaultSymbol = LegendFactory.DEFAULT_POLYGON_SYMBOL;
    }

    /**
     * Crea un nuevo VectorialIntervalLegend.
     *
     * @param type tipo de shape.
     */
    public VectorialIntervalLegend(int type) {
        setShapeType(type);
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.UniqueValueLegend#addSymbol(java.lang.Object,
     *      FSymbol)
     */
    public void addSymbol(Object key, FSymbol symbol) {
        //TODO guardar los intervalos.
        Object resul;
        resul = symbols.put(key, symbol);

        /*if (resul != null) {
           System.err.println("Error: la clave " + key +
                   " ya exista. Resul = " + resul);
           System.err.println("symbol nuevo:" + symbol.m_Descrip +
                   " Sviejo= " + ((FSymbol) resul).m_Descrip);
           } else {
         */
        keys.add(key);

        //}
    }

    /*
     * @see com.iver.cit.gvsig.fmap.rendering.Legend#getSymbol(java.lang.Object)
     *
                   public FStyle2D getSymbol(Object value) {
                       return (FStyle2D) symbols.get(value);
                   }
                   // TODO transformar la funcin anterior en la siguiente
     *
     */

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#getSymbol(int)
     */
    public FSymbol getSymbol(int recordIndex) throws DriverException {
        try {
            Value val = dataSource.getFieldValue(recordIndex, fieldId);
            IInterval interval = getInterval(val);
            FSymbol theSymbol = getSymbolByInterval(interval);

            if (theSymbol != null) {
                return theSymbol;
            } else if (useDefaultSymbol) {
                return getDefaultSymbol();
            }

            return null;
        } catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
            throw new DriverException(e);
        }
    }

    /**
     * Devuelve un smbolo a partir de una IFeature.
     * OJO!! Cuando usamos un feature iterator de base de datos
     * el nico campo que vendr rellenado es el de fieldID.
     * Los dems vendrn a nulos para ahorra tiempo de creacin.
     *
     * @param feat IFeature.
     *
     * @return Smbolo.
     */
    public FSymbol getSymbolByFeature(IFeature feat) {
        Value val = feat.getAttribute(fieldId);
        IInterval interval = getInterval(val);
        FSymbol theSymbol = getSymbolByInterval(interval);

        if (theSymbol != null) {
            return theSymbol;
        } else {
            return getDefaultSymbol();
        }
    }

    /**
     * Devuelve el smbolo a partir del intervalo.
     *
     * @param key intervalo.
     *
     * @return smbolo.
     */
    public FSymbol getSymbolByInterval(IInterval key) {
        if (key == null) {
            return null;
        }

        if (symbols.containsKey(key)) {
            return (FSymbol) symbols.get(key);
        }

        return null;
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.ClassifiedLegendInfo#getDescriptions()
     */
    public String[] getDescriptions() {
        String[] descriptions = new String[symbols.size()];
        FSymbol[] auxSym = getSymbols();

        for (int i = 0; i < descriptions.length; i++)
            descriptions[i] = auxSym[i].getDescription();

        return descriptions;
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.ClassifiedLegendInfo#getValues()
     */
    public Object[] getValues() {
        return (Object[]) symbols.keySet().toArray(new Object[0]);
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.IntervalLegend#setIntervalSymbol(com.iver.cit.gvsig.fmap.rendering.FInterval,
     *      org.geotools.renderer.style.Style2D)
     */
    public void setIntervalSymbol(IInterval interval, FSymbol symbol) {
        /*symbols.put(interval, symbol);
           values.put(new Integer(index), interval);
           index++;
         */
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.IntervalLegend#changeInterval(int,
     *      com.iver.cit.gvsig.fmap.rendering.FInterval)
     */
    public void changeInterval(int index, IInterval newInterval) {
        /*Object value = values.remove(new Integer(index));
           Object symbol = symbols.remove(value);
           values.put(new Integer(index), newInterval);
           symbols.put(newInterval, symbol);
         */
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.ClassifiedLegend#clear()
     */
    public void clear() {
        index = 0;
        keys.clear();
        symbols.clear();
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.ClassifiedLegend#getSymbols()
     */
    public FSymbol[] getSymbols() {
        return (FSymbol[]) symbols.values().toArray(new FSymbol[0]);
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.ClassifiedLegend#getFieldName()
     */
    public String getFieldName() {
        return fieldName;
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#setDefaultSymbol(com.iver.cit.gvsig.fmap.rendering.styling.FStyle2D)
     */
    public void setDefaultSymbol(FSymbol s) {
        defaultSymbol = s;
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.ClassifiedLegend#setFieldName(String)
     */
    public void setFieldName(String str) {
        fieldName = str;
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.Legend#setLabelField(int)
     */
    public void setLabelField(String fieldName) {
        labelFieldName = fieldName;
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.Legend#getLabelField()
     */
    public String getLabelField() {
        return labelFieldName;
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.Legend#getDefaultSymbol()
     */
    public FSymbol getDefaultSymbol() {
        return defaultSymbol;
    }
    
    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @see com.iver.cit.gvsig.fmap.rendering.Legend#getSLDString()
     */
    public String getSLDString(String layerName) {
	    
    	try{
	    	
			XmlBuilder xmlBuilder = new XmlBuilder();
			xmlBuilder.writeHeader();
			xmlBuilder.openTag(SLDTags.SLD_ROOT, SLDTags.VERSION_ATTR, SLDTags.VERSION_1_0_0);
			xmlBuilder.openTag(SLDTags.NAMEDLAYER);
			xmlBuilder.writeTag(SLDTags.NAME,layerName);
			xmlBuilder.openTag(SLDTags.USERSTYLE);
			xmlBuilder.openTag(SLDTags.FEATURETYPESTYLE);
			xmlBuilder.writeTag(SLDTags.FEATURETYPENAME,fieldName);
			
			FSymbol[] symbols = this.getSymbols(); 
			Object[] values = this.getValues(); 
			
			FInterval interval;
			for(int i = 0; i < symbols.length; i++ )
			{
				interval = (FInterval)values[i];								
				//interval = (FInterval)getInterval(ValueFactory.createValue(valueDbl.doubleValue()));
				xmlBuilder.openTag(SLDTags.RULE);
				xmlBuilder.openTag(SLDTags.FILTER);
				xmlBuilder.openTag(SLDTags.AND);
				xmlBuilder.openTag(SLDTags.PROPERTYISGREATEROREQUALTHAN);
				xmlBuilder.writeTag(SLDTags.PROPERTYNAME,fieldName);
				xmlBuilder.writeTag(SLDTags.LITERAL, ""+interval.getMin());
				xmlBuilder.closeTag();
				xmlBuilder.openTag(SLDTags.PROPERTYISLESSOREQUALTHAN);
				xmlBuilder.writeTag(SLDTags.PROPERTYNAME,fieldName);
				xmlBuilder.writeTag(SLDTags.LITERAL, ""+interval.getMax());
				xmlBuilder.closeTag();				
				xmlBuilder.closeTag();										
				xmlBuilder.writeRaw(symbols[i].toSLD());				
				xmlBuilder.closeTag();
				xmlBuilder.closeTag();
			}
			
			xmlBuilder.closeTag();
			xmlBuilder.closeTag();
			xmlBuilder.closeTag();
			xmlBuilder.closeTag();			
			return xmlBuilder.getXML(); 	    					
	        	       
	    }catch(Exception e)
	    {
	    	e.printStackTrace(); 
	    	return null;
	    }
    }
        
    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @see com.iver.cit.gvsig.fmap.rendering.Legend#getXMLEntity()
     */
    public XMLEntity getXMLEntity() {
        XMLEntity xml = new XMLEntity();
        xml.putProperty("className", this.getClass().getName());

        if (getDefaultSymbol() == null) {
            xml.putProperty("useDefaultSymbol", 0);
        } else {
            xml.putProperty("useDefaultSymbol", 1);
            xml.addChild(getDefaultSymbol().getXMLEntity());
        }

        xml.putProperty("fieldName", fieldName);
        xml.putProperty("index", index);
        xml.putProperty("labelfield", labelFieldName);
        xml.putProperty("labelfield", labelFieldName);
        xml.putProperty("labelFieldHeight", labelFieldHeight);
        xml.putProperty("labelFieldRotation", labelFieldRotation);
        
        xml.putProperty("intervalType", intervalType);
        xml.putProperty("numKeys", keys.size());

        if (keys.size() > 0) {
            xml.putProperty("tipoValueKeys", keys.get(0).getClass().getName());

            String[] sk = new String[keys.size()];

            for (int i = 0; i < keys.size(); i++) {
                sk[i] = ((IInterval) keys.get(i)).toString();
            }

            xml.putProperty("keys", getValues());

            for (int i = 0; i < keys.size(); i++) {
                xml.addChild(getSymbols()[i].getXMLEntity());
            }
        }

        xml.putProperty("startColor", StringUtilities.color2String(startColor));
        xml.putProperty("endColor", StringUtilities.color2String(endColor));

        ///xml.putProperty("numSymbols", symbols.size());
        ///xml.putProperty("indexs", getIndexs());
        ///xml.putProperty("values", getValues());
        return xml;
    }

    /**
     * Inserta los atributos del XMLEntity.
     *
     * @param xml XMLEntity.
     */
    public void setXMLEntity03(XMLEntity xml) {
        fieldName = xml.getStringProperty("fieldName");
        index = xml.getIntProperty("index");
        labelFieldName = xml.getStringProperty("labelfield");

        if (xml.contains("intervalType")) { //TODO Esta condicin es para poder cargar la versin 0.3, se puede eliminar cuando ya no queramos soportar esta versin.
            intervalType = xml.getIntProperty("intervalType");
        }

        int useDefaultSymbol = xml.getIntProperty("useDefaultSymbol");

        if (useDefaultSymbol == 1) {
            setDefaultSymbol(FSymbol.createFromXML03(xml.getChild(0)));
        } else {
            setDefaultSymbol(null);
        }

        int numKeys = xml.getIntProperty("numKeys");

        if (numKeys > 0) {
            String className = xml.getStringProperty("tipoValueKeys");
            String[] sk = xml.getStringArrayProperty("keys");
            IInterval auxInterval;

            for (int i = 0; i < numKeys; i++) {
                auxInterval = FInterval.create(sk[i]);
                symbols.put(auxInterval,
                    FSymbol.createFromXML03(xml.getChild(i + useDefaultSymbol)));
                keys.add(auxInterval);
                System.out.println("auxInterval =" + auxInterval + "Symbol =" +
                    FSymbol.createFromXML03(xml.getChild(i + useDefaultSymbol))
                           .getDescription());
            }
        }

        startColor = StringUtilities.string2Color(xml.getStringProperty(
                    "startColor"));
        endColor = StringUtilities.string2Color(xml.getStringProperty(
                    "endColor"));
    }

    /**
     * Inserta los atributos del XMLEntity.
     *
     * @param xml XMLEntity.
     */
    public void setXMLEntity(XMLEntity xml) {
        fieldName = xml.getStringProperty("fieldName");
        index = xml.getIntProperty("index");
        labelFieldName = xml.getStringProperty("labelfield");
        if (xml.contains("labelFieldHeight")) {
            setLabelHeightField(xml.getStringProperty("labelFieldHeight"));
        }

        if (xml.contains("labelFieldRotation")) {
            setLabelRotationField(xml.getStringProperty("labelFieldRotation"));
        }

        if (xml.contains("intervalType")) { //TODO Esta condicin es para poder cargar la versin 0.3, se puede eliminar cuando ya no queramos soportar esta versin.
            intervalType = xml.getIntProperty("intervalType");
        }

        int useDefaultSymbol = xml.getIntProperty("useDefaultSymbol");

        if (useDefaultSymbol == 1) {
            setDefaultSymbol(FSymbol.createFromXML(xml.getChild(0)));
            this.useDefaultSymbol=true;
        } else {
            setDefaultSymbol(null);
            this.useDefaultSymbol=false;
        }

        int numKeys = xml.getIntProperty("numKeys");

        if (numKeys > 0) {
            String className = xml.getStringProperty("tipoValueKeys");
            String[] sk = xml.getStringArrayProperty("keys");
            IInterval auxInterval;

            for (int i = 0; i < numKeys; i++) {
                auxInterval = FInterval.create(sk[i]);
                symbols.put(auxInterval,
                    FSymbol.createFromXML(xml.getChild(i + useDefaultSymbol)));
                keys.add(auxInterval);
                System.out.println("auxInterval =" + auxInterval + "Symbol =" +
                    FSymbol.createFromXML(xml.getChild(i + useDefaultSymbol))
                           .getDescription());
            }
        }

        startColor = StringUtilities.string2Color(xml.getStringProperty(
                    "startColor"));
        endColor = StringUtilities.string2Color(xml.getStringProperty(
                    "endColor"));
    }

    /**
     * @see com.iver.cit.gvsig.fmap.rendering.Legend#cloneLegend()
     */
    public Legend cloneLegend() throws XMLException {
        return (Legend) LegendFactory.createFromXML(getXMLEntity());
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#setDataSource(com.hardcode.gdbms.engine.data.DataSource)
     */
    public void setDataSource(DataSource ds)
        throws FieldNotFoundException, DriverException {
        try {
            dataSource = ds;
            ds.start();
            fieldId = ds.getFieldIndexByName(fieldName);
            ds.stop();
        } catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
            throw new DriverException(e);
        }
    }

    /**
     * Devuelve el intervalo a partir del valor.
     *
     * @param v valor.
     *
     * @return intervalo.
     */
    public IInterval getInterval(Value v) {
        /*if (v instanceof NullValue){
           System.out.println("Si");
           }*/
        for (int i = 0; i < keys.size(); i++) {
            if (((IInterval) keys.get(i)).isInInterval(v)) {
                return (IInterval) keys.get(i);
            }
        }

        return null;
    }

    /**
     * Devuelve el color final.
     *
     * @return color final.
     */
    public Color getEndColor() {
        return endColor;
    }

    /**
     * Inserta el color final.
     *
     * @param endColor Color final.
     */
    public void setEndColor(Color endColor) {
        this.endColor = endColor;
    }

    /**
     * Devuelve el color inicial.
     *
     * @return Color inicial.
     */
    public Color getStartColor() {
        return startColor;
    }

    /**
     * Inserta el color inicial.
     *
     * @param startColor Color inicial.
     */
    public void setStartColor(Color startColor) {
        this.startColor = startColor;
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#getShapeType()
     */
    public int getShapeType() {
        return shapeType;
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#setShapeType(int)
     */
    public void setShapeType(int shapeType) {
        if (this.shapeType != shapeType) {
            switch (shapeType) {
                case FShape.POINT:
                    defaultSymbol = new FSymbol(FConstant.SYMBOL_TYPE_POINT);

                    break;

                case FShape.LINE:
                    defaultSymbol = new FSymbol(FConstant.SYMBOL_TYPE_LINE);

                    break;

                case FShape.POLYGON:
                    defaultSymbol = new FSymbol(FConstant.SYMBOL_TYPE_FILL);

                    break;

                default:
                    defaultSymbol = new FSymbol(shapeType);
            }

            this.shapeType = shapeType;
        }
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#getLabelHeightField()
     */
    public String getLabelHeightField() {
        return labelFieldHeight;
    }

    /**
     * Inserta el alto del campo.
     *
     * @param str alto.
     */
    public void setLabelHeightField(String str) {
        labelFieldHeight = str;
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#getLabelRotationField()
     */
    public String getLabelRotationField() {
        return labelFieldRotation;
    }

    /**
     * Inserta la rotacin del campo.
     *
     * @param str Rotacin.
     */
    public void setLabelRotationField(String str) {
        labelFieldRotation = str;
    }

    /**
     * Inserta el tipo de clasificacin de los intervalos.
     *
     * @param tipoClasificacion Tipo de clasificacin.
     */
    public void setIntervalType(int tipoClasificacion) {
        intervalType = tipoClasificacion;
    }

    /**
     * Devuelve el tipo de clasificacin de los intervalos.
     *
     * @return Tipo de clasificacin.
     */
    public int getIntervalType() {
        return intervalType;
    }

    /**
     * Inserta si se representan el resto de valores o no.
     *
     * @param b True si se tienen que representar el resto de valores.
     */
    public void useDefaultSymbol(boolean b) {
        useDefaultSymbol = b;
    }

    /**
     * Devuelve si se utiliza o no el resto de valores para representarse.
     *
     * @return True si se utiliza el resto de valores.
     */
    public boolean isUseDefaultSymbol() {
        return useDefaultSymbol;
    }

    /**
     * Devuelve si ha sido modificado el etiquetado de la capa.
     *
     * @return True si el etiquetado de la capa ha sido modificado.
     */
    /* public boolean isBWithHeightText() {
        return bWithHeightText;
    } */

    /**
     * Introduce si el etiquetado de la capa ha sido modificado.
     *
     * @param withHeightText Boolean que indica si el etiquetado de la capa ha
     *        sido modificado.
     */
    /* public void setBWithHeightText(boolean withHeightText) {
        bWithHeightText = withHeightText;
    } */

    /**
     * Elimina un smbolo a partir de su clave.
     *
     * @param obj clave.
     */
    public void delSymbol(Object obj) {
        keys.remove(obj);
        symbols.remove(obj);
    }

    /* (non-Javadoc)
     * @see com.iver.cit.gvsig.fmap.rendering.VectorialLegend#getUsedFields()
     */
    public String[] getUsedFields() {
        ArrayList usedFields = new ArrayList();
        if (getFieldName() != null)
            usedFields.add(getFieldName());        
        if (getLabelField() != null)
            usedFields.add(getLabelField());
        if (getLabelHeightField() != null)
            usedFields.add(getLabelHeightField());
        if (getLabelRotationField() != null)
            usedFields.add(getLabelRotationField());
                
        return (String[]) usedFields.toArray(new String[0]);

    }
}
