package org.gvsig.remoteClient.gml.schemas;

import java.io.IOException;

import org.gvsig.remoteClient.gml.GMLException;
import org.gvsig.remoteClient.utils.CapabilitiesTags;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: XMLSimpleType.java,v 1.1 2006/08/10 12:00:49 jorpiell Exp $
 * $Log: XMLSimpleType.java,v $
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 *
 */
/**
 * A XS simple data type.
 * 
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class XMLSimpleType implements IXMLType{
	public static final String STRING = "XS:STRING";
	public static final String INTEGER = "XS:INTEGER";
	public static final String DOUBLE = "XS:DOUBLE";
	public static final String FLOAT = "XS:FLOAT";
	public static final String BOOLEAN = "XS:BOOLEAN";
	
	private String type = null;
	private int totalDigits = 0;
	private int fractionDigits = 0;
	
	
	
	public XMLSimpleType(String type) {
		super();
		this.type = type;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.remoteClient.gml.IXMLType#getName()
	 */
	public String getName() {
		return type;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.remoteClient.gml.IXMLType#getType()
	 */
	public int getType() {
		return IXMLType.SIMPLE;
	}
	
	/**
	 * @return Returns the fractionDigits.
	 */
	public int getFractionDigits() {
		return fractionDigits;
	}
	/**
	 * @param fractionDigits The fractionDigits to set.
	 */
	public void setFractionDigits(int fractionDigits) {
		this.fractionDigits = fractionDigits;
	}
	/**
	 * @return Returns the totalDigits.
	 */
	public int getTotalDigits() {
		return totalDigits;
	}
	/**
	 * @param totalDigits The totalDigits to set.
	 */
	public void setTotalDigits(int totalDigits) {
		this.totalDigits = totalDigits;
	}

	/**
	 * @param type The type to set.
	 */
	public void setType(String type) {
		this.type = type;	
	}	

	public Object getObject(String value)throws GMLException{
		try{
			if (type.equals(STRING)){
				return value;
			}else if (type.equals(INTEGER)){
				return new Integer(value);
			}else if (type.equals(DOUBLE)){
				return new Double(value);
			}else if (type.equals(FLOAT)){
				return new Float(value);
			}else if (type.equals(BOOLEAN)){
				return new Boolean(value);
			}
		}catch (Exception e){
			throw new GMLException(GMLException.EXC_ATTRIB_WITH_INVALID_FORMAT);
		}
		return value;
	}
	
	private void parseSimpleType(XMLSchemaParser parser) throws IOException, XmlPullParserException
	{   
		int currentTag;
		boolean end = false;
		
		parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.SIMPLETYPE);
		currentTag = parser.next();
		
		while (!end) 
		{
			switch(currentTag)
			{
			case KXmlParser.START_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.RESTRICTION)==0)
				{
					for (int i=0 ; i<parser.getAttributeCount() ; i++){
						if (parser.getAttributeName(i).compareTo(CapabilitiesTags.BASE) == 0){
							setType(parser.getAttributeValue(i));
						}
					}
					parseRestriction(parser);
				}   
				break;
			case KXmlParser.END_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.SIMPLETYPE) == 0)
					end = true;
				break;
			case KXmlParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = parser.next();
			}	
		}
	}
	
	private void parseRestriction(XMLSchemaParser parser) throws IOException, XmlPullParserException
	{   
		int currentTag;
		boolean end = false;
		
		parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.RESTRICTION);
		currentTag = parser.next();
		
		while (!end) 
		{
			switch(currentTag)
			{
			case KXmlParser.START_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.TOTAL_DIGITS)==0)
				{
					for (int i=0 ; i<parser.getAttributeCount() ; i++){
						if (parser.getAttributeName(i).compareTo(CapabilitiesTags.VALUE) == 0){
							setTotalDigits(Integer.parseInt(parser.getAttributeValue(i)));
						}
					}
				}else if (parser.getName().compareTo(CapabilitiesTags.FRACTION_DIGITS)==0){
					for (int i=0 ; i<parser.getAttributeCount() ; i++){
						if (parser.getAttributeName(i).compareTo(CapabilitiesTags.VALUE) == 0){
							setFractionDigits(Integer.parseInt(parser.getAttributeValue(i)));
						}
					}
				}
				break;
			case KXmlParser.END_TAG:
				if (parser.getName().compareTo(CapabilitiesTags.RESTRICTION) == 0)
					end = true;
				break;
			case KXmlParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = parser.next();
			}	
		}
	}


	
	
	
	
}
