package org.gvsig.remoteClient.wfs;

import java.io.IOException;
import java.util.TreeMap;

import org.gvsig.remoteClient.utils.CapabilitiesTags;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSAttribute.java,v 1.2 2006/07/12 06:23:31 jorpiell Exp $
 * $Log: WFSAttribute.java,v $
 * Revision 1.2  2006/07/12 06:23:31  jorpiell
 * Soportados tipos ms complejos
 *
 * Revision 1.1  2006/05/25 10:20:57  jorpiell
 * Se ha cambiado el nombre de la clase WFSField por la clase WFSAttribute, porque resultaba confuso
 *
 * Revision 1.3  2006/05/23 13:23:22  jorpiell
 * Se ha cambiado el final del bucle de parseado y se tiene en cuenta el online resource
 *
 * Revision 1.1  2006/04/20 16:39:16  jorpiell
 * Aadida la operacion de describeFeatureType y el parser correspondiente.
 *
 *
 */
/**
 * This class is used to represent a feature field.
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class WFSAttribute {
	private String name = null;
	private Type type = null;
	private int minOccurs = -1;
	private int maxOccurs = -1;
	

	public WFSAttribute() {
		super();
		this.type = new Type();
	}
	
	public String toString(){
		return name;
	}
	/**
	 * @return Returns the maxOcurs.
	 */
	public int getMaxOccurs() {
		return maxOccurs;
	}
	/**
	 * @param maxOcurs The maxOcurs to set.
	 */
	public void setMaxOccurs(int maxOccurs) {
		this.maxOccurs = maxOccurs;
	}
	/**
	 * @return Returns the minOcurs.
	 */
	public int getMinOccurs() {
		return minOccurs;
	}
	/**
	 * @param minOcurs The minOcurs to set.
	 */
	public void setMinOccurs(int minOccurs) {
		this.minOccurs = minOccurs;
	}
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}
	/**
	 * @return Returns the type.
	 */
	public String getType() {
		return type.getType();
	}
	/**
	 * @param type The type to set.
	 */
	public void setType(String type) {
		if (this.type == null){
			this.type = new Type();
		}
		this.type.setType(type);
	}
	
	/**
     * <p>Parses the contents of the parser
     * to extract the information about an WFS Layer</p>
	 * @throws IOException 
	 * @throws XmlPullParserException 
     * 
     */
    public void parse(WFSSchemaParser parser) throws XmlPullParserException, IOException{
    	
    	for (int i=0 ; i<parser.getAttributeCount() ; i++){
    		if (parser.getAttributeName(i).compareTo(CapabilitiesTags.ELEMENT_NAME) == 0){
    			setName(parser.getAttributeValue(i));
    		}else if (parser.getAttributeName(i).compareTo(CapabilitiesTags.ELEMENT_TYPE) == 0){
    			setType(parser.getAttributeValue(i));
    		}else if (parser.getAttributeName(i).compareTo(CapabilitiesTags.ELEMENT_MAXOCCURS) == 0){
    			setMaxOccurs(Integer.parseInt(parser.getAttributeValue(i)));
    		}else if (parser.getAttributeName(i).compareTo(CapabilitiesTags.ELEMENT_MINOCCURS) == 0){
    			setMinOccurs(Integer.parseInt(parser.getAttributeValue(i)));
    		}
    	}		
	}
    
    public class Type{
    	private String type = "";
    	private int totalDigits = 0;
    	private int fractionDigits = 0;
    	
    	 public void parse(WFSSchemaParser parser) throws XmlPullParserException, IOException{
    	    	
    	    	
    	}
    	
		/**
		 * @return Returns the fractionDigits.
		 */
		public int getFractionDigits() {
			return fractionDigits;
		}
		/**
		 * @param fractionDigits The fractionDigits to set.
		 */
		public void setFractionDigits(int fractionDigits) {
			this.fractionDigits = fractionDigits;
		}
		/**
		 * @return Returns the totalDigits.
		 */
		public int getTotalDigits() {
			return totalDigits;
		}
		/**
		 * @param totalDigits The totalDigits to set.
		 */
		public void setTotalDigits(int totalDigits) {
			this.totalDigits = totalDigits;
		}
		/**
		 * @return Returns the type.
		 */
		public String getType() {
			return type;
		}
		/**
		 * @param type The type to set.
		 */
		public void setType(String type) {
			this.type = type;
		}    
		
		private void parseSimpleType(WFSSchemaParser parser) throws IOException, XmlPullParserException
		{   
			int currentTag;
			boolean end = false;
			
			parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.SIMPLETYPE);
			currentTag = parser.next();
			
			while (!end) 
			{
				switch(currentTag)
				{
				case KXmlParser.START_TAG:
					if (parser.getName().compareTo(CapabilitiesTags.RESTRICTION)==0)
					{
						for (int i=0 ; i<parser.getAttributeCount() ; i++){
				    		if (parser.getAttributeName(i).compareTo(CapabilitiesTags.BASE) == 0){
				    			setType(parser.getAttributeValue(i));
				    		}
						}
						parseRestriction(parser);
					}   
					break;
				case KXmlParser.END_TAG:
					if (parser.getName().compareTo(CapabilitiesTags.SIMPLETYPE) == 0)
						end = true;
					break;
				case KXmlParser.TEXT:                   
					break;
				}
				if (!end){
					currentTag = parser.next();
				}	
			}
		}
		
		private void parseRestriction(WFSSchemaParser parser) throws IOException, XmlPullParserException
		{   
			int currentTag;
			boolean end = false;
			
			parser.require(KXmlParser.START_TAG, null, CapabilitiesTags.RESTRICTION);
			currentTag = parser.next();
			
			while (!end) 
			{
				switch(currentTag)
				{
				case KXmlParser.START_TAG:
					if (parser.getName().compareTo(CapabilitiesTags.TOTAL_DIGITS)==0)
					{
						for (int i=0 ; i<parser.getAttributeCount() ; i++){
				    		if (parser.getAttributeName(i).compareTo(CapabilitiesTags.VALUE) == 0){
				    			setTotalDigits(Integer.parseInt(parser.getAttributeValue(i)));
				    		}
						}
					}else if (parser.getName().compareTo(CapabilitiesTags.FRACTION_DIGITS)==0){
						for (int i=0 ; i<parser.getAttributeCount() ; i++){
				    		if (parser.getAttributeName(i).compareTo(CapabilitiesTags.VALUE) == 0){
				    			setFractionDigits(Integer.parseInt(parser.getAttributeValue(i)));
				    		}
						}
					}
					break;
				case KXmlParser.END_TAG:
					if (parser.getName().compareTo(CapabilitiesTags.RESTRICTION) == 0)
						end = true;
					break;
				case KXmlParser.TEXT:                   
					break;
				}
				if (!end){
					currentTag = parser.next();
				}	
			}
		}
		
    	
    }

	public void parseType(WFSSchemaParser parser) throws IOException, XmlPullParserException {
		this.type.parseSimpleType(parser);		
	}
    
    

    


}
