package org.gvsig.remoteClient.wfs;

import java.io.File;
import java.net.URL;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Set;
import java.util.Vector;

import org.gvsig.remoteClient.OGCProtocolHandler;
import org.gvsig.remoteClient.utils.Utilities;
import org.gvsig.remoteClient.wms.ICancellable;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSProtocolHandler.java,v 1.7 2006/06/21 12:53:03 jorpiell Exp $
 * $Log: WFSProtocolHandler.java,v $
 * Revision 1.7  2006/06/21 12:53:03  jorpiell
 * Se tienen en cuanta el nmero de features
 *
 * Revision 1.6  2006/06/14 08:46:07  jorpiell
 * Se tiene en cuanta la opcion para refrescar las capabilities
 *
 * Revision 1.5  2006/06/14 07:54:18  jorpiell
 * Se parsea el online resource que antes se ignoraba
 *
 * Revision 1.4  2006/05/30 13:58:03  jaume
 * cancelable downloads
 *
 * Revision 1.3  2006/05/23 13:23:13  jorpiell
 * Se ha cambiado el final del bucle de parseado y se tiene en cuenta el online resource
 *
 * Revision 1.2  2006/04/20 16:39:16  jorpiell
 * Aadida la operacion de describeFeatureType y el parser correspondiente.
 *
 * Revision 1.1  2006/04/19 12:51:35  jorpiell
 * Aadidas algunas de las clases del servicio WFS
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public abstract class WFSProtocolHandler extends OGCProtocolHandler {
	/**
	 * Encoding used to parse different xml documents.
	 */
	protected String encoding = "UTF-8";
	/**
     * WFS metadata
     */
    protected ServiceInformation serviceInfo = new ServiceInformation();
	protected Hashtable features = new Hashtable();
	protected String currentFeature = null;
	
	/**
     * <p>Builds a GetCapabilities request that is sent to the WFS
     * the response will be parse to extract the data needed by the
     * WFS client</p>
     */
    public void getCapabilities(WFSStatus status,boolean override, ICancellable cancel) {
           URL request = null;
            try {
                request = new URL(buildCapabilitiesRequest(status));
            }
            catch(Exception e) {
                e.printStackTrace();
            }
            try {
            	if (override){
    				Utilities.removeURL(request);
            	}
            	File f = Utilities.downloadFile(request,"wfs_capabilities.xml", cancel);
            	if (f == null)
    				return;
            	clear();
            	parseCapabilities(f);
            } catch(Exception e) {
            	e.printStackTrace();
            }
    }    

    private void clear() {
		features.clear();
		serviceInfo.clear();
	}
    
    /**
     * @return
     */
    private String buildCapabilitiesRequest(WFSStatus status) {
    	StringBuffer req = new StringBuffer();
		String symbol = null;
		
		String onlineResource;
		if (status == null || status.getOnlineResource() == null)
			onlineResource = getHost();
		else 
			onlineResource = status.getOnlineResource();
		symbol = getSymbol(onlineResource);
		
		req.append(onlineResource).append(symbol).append("REQUEST=GetCapabilities&SERVICE=WFS&");
		req.append("VERSION=").append(getVersion()).append("&EXCEPTIONS=XML");
		return req.toString();
    }   
    
    /**
     * Builds the GetCapabilitiesRequest according to the OGC WFS Specifications
     * without a VERSION, to get the highest version than a WFS supports.
     */
    public static String buildCapabilitiesSuitableVersionRequest(String _host, String _version)
    {
		String req = new String();		
        String symbol = getSymbol(_host);
        req = req + _host + symbol + "REQUEST=GetCapabilities&SERVICE=WFS&";		
        if((_version != null) && (_version.length()>0 ))
        {
        	req += ("&VERSION=" + _version);
        }
		req += ("&EXCEPTIONS=XML");		
		
		return req;   	
    }
    
	/**
     * <p>Builds a describeFeatureType request that is sent to the WFS
     * the response will be parse to extract the data needed by the
     * WFS client</p>
     * @param status
	 * WFS client status. Contains all the information to create
	 * the query. In this case, the only the feature name is needed.
     */
    public void describeFeatureType(WFSStatus status) {
        URL request = null;
        this.currentFeature = status.getFeatureName();
        try {
            request = new URL(buildDescribeFeatureTypeRequest(status));
        }
        catch(Exception e) {
            e.printStackTrace();
        }
        try {
        	File f = Utilities.downloadFile(request, "wfs_describeFeatureType.xml", null);
        	parseDescribeFeatureType(f);
        	
        } catch(Exception e) {
        	e.printStackTrace();
        }
    }     
    
    private String buildDescribeFeatureTypeRequest(WFSStatus status) {
    	StringBuffer req = new StringBuffer();
		String symbol = null;
		
		String onlineResource;
		if(serviceInfo.getOnlineResource(WFSOperation.DESCRIBEFEATURETYPE) != null){
			onlineResource = serviceInfo.getOnlineResource(WFSOperation.DESCRIBEFEATURETYPE);
		}else {
			onlineResource = getHost();
		}
		symbol = getSymbol(onlineResource);
		
		req.append(onlineResource).append(symbol).append("REQUEST=DescribeFeatureType&SERVICE=WFS&");
		req.append("TYPENAME=").append(status.getFeatureName()).append("&");
		req.append("VERSION=").append(getVersion()).append("&EXCEPTIONS=XML");
		return req.toString();
    }
    
    /**
     * parses the data retrieved by the DescribeCoverage XML document
     */
    public abstract boolean parseDescribeFeatureType(File f);
    
    /**
     * <p>Builds a getFeature request that is sent to the WFS
     * the response will be parse to extract the data needed by the
     * WFS client</p>
     * @param status
	 * WFS client status. Contains all the information to create
	 * the query. 
	 * @return File
	 * GML file
     */    
    public File getFeature(WFSStatus status){
    	URL request = null;
        try {
            request = new URL(buildGetFeatureRequest(status));
        }
        catch(Exception e) {
            e.printStackTrace();
        }
        try {
        	File f = Utilities.downloadFile(request, "wfs_getFeature.xml", null);
        	return f;
        	
        } catch(Exception e) {
        	e.printStackTrace();
        	return null;
        }
    }
    
	/**
     * <p>Builds a getFeature request that is sent to the WFS
     * the response will be parse to extract the data needed by the
     * WFS client</p>
     * @param status
	 * WFS client status. Contains all the information to create
	 * the query. 
     */
    private String buildGetFeatureRequest(WFSStatus status){
    	StringBuffer req = new StringBuffer();
		String symbol = null;
		
		String onlineResource;
		if(serviceInfo.getOnlineResource(WFSOperation.GETFEATURE) != null){
			onlineResource = serviceInfo.getOnlineResource(WFSOperation.GETFEATURE);
		}else {
			onlineResource = getHost();
		}
		symbol = getSymbol(onlineResource);
		
		req.append(onlineResource).append(symbol).append("REQUEST=GetFeature&SERVICE=WFS&");
		req.append("TYPENAME=").append(status.getFeatureName()).append("&");
		String[] fields = status.getFields();
		if (fields.length > 0){
			req.append("PROPERTYNAME=");
			req.append(fields[0]);
			for (int i=1 ; i<fields.length ; i++){
				req.append("," + fields[i]);
			}	
			req.append("&");
		}		
		req.append("VERSION=").append(getVersion()).append("&EXCEPTIONS=XML");
		req.append("&MAXFEATURES=").append(status.getBuffer());
		return req.toString();
    	

    }    
    
    /**
     * Just for not repeat code. Gets the correct separator according 
     * to the server URL
     * @param h
     * @return
     */
    private static String getSymbol(String h) {
        String symbol;
        if (h.indexOf("?")==-1) 
            symbol = "?";
        else if (h.indexOf("?")!=h.length()-1)
            symbol = "&";
        else
            symbol = "";
        return symbol;
    }  
    
    /**
     * Returns the service Information
     * @return
     */
    public ServiceInformation getServiceInformation(){
    	return serviceInfo;
    }
    
    /**
	 * @return Returns the features.
	 */
	public Hashtable getFeatures() {
		return features;
	}

	/**
	 * @return Returns the currentFeature.
	 */
	public String getCurrentFeature() {
		return currentFeature;
	}
	
	/**
	 * Sets the fields of the current feature
	 * @param fields
	 */
	public void setFields(Vector fields){
		WFSFeature feature = (WFSFeature) features.get(currentFeature);
		feature.setFields(fields);
		features.put(feature.getName(),feature);		
	}
	
	/**
	 * Sets the fields of the current feature
	 * @param fields
	 */
	public void setFields(Hashtable fields){
		WFSFeature feature = (WFSFeature) features.get(currentFeature);
		Vector vFields = new Vector();
		Set keys = fields.keySet();
		for (int i=0 ; i<keys.size() ; i++){
			vFields.add(fields.get(keys.toArray()[i]));
		}
		feature.setFields(vFields);
		features.put(feature.getName(),feature);		
	}
	

	/**
	 * @param currentFeature The currentFeature to set.
	 */
	public void setCurrentFeature(String currentFeature) {
		this.currentFeature = currentFeature;
	}      
    
    public class ServiceInformation {

        public String online_resource = null;
        public String version;
        public String name;
        public String scope;
        public String title;
        public String abstr;
        public String keywords;
        public String fees;
        public String operationsInfo;
        public String personname;
        public String organization;
        public String function;
        public String addresstype;
        public String address;
        public String place;
        public String province;
        public String postcode;
        public String country;
        public String phone;
        public String fax;
        public String email;
        public Vector formats;
        public HashMap operations; 
        
        public ServiceInformation() {  	
        	clear();     
        }

		public void clear() {
			version = new String();
            name = new String();
            scope = new String();
            title = new String();
            abstr = new String();
            keywords = new String();
            fees = new String();
            operationsInfo = new String();
            personname = new String();
            organization = new String();
            function = new String();
            addresstype = new String();
            address = new String();
            place = new String();
            province = new String();
            postcode = new String();
            country = new String();
            phone = new String();
            fax = new String();
            email = new String();
            formats = new Vector();       	
            operations = new HashMap();   			
		}

		/**
		 * @return Returns the online_resource.
		 */
		public String getOnline_resource() {
			return online_resource;
		}
		
		public void addOperation(int operation){
			operations.put(new Integer(operation),new WFSOperation(operation));
		}
		
		public void addOperation(int operation,String onlineResource){
			operations.put(new Integer(operation),new WFSOperation(operation,onlineResource));
		}
		
		public String getOnlineResource(int operation){
			WFSOperation op = (WFSOperation)operations.get(new Integer(operation));
			if ((op == null) ||
					(op.getOnlineResource() == null) || 
				(op.getOnlineResource().equals(""))){
				return null;
			}
			return op.getOnlineResource();
				
		}

     }
	
	public class WFSOperation{
		public static final int GETCAPABILITIES = 0;
		public static final int DESCRIBEFEATURETYPE = 1;
		public static final int GETFEATURE = 2;
		
		private int operationName;
		private String onlineResource;
		
		public WFSOperation(int operationName) {
			super();
			this.operationName = operationName;
		}		

		public WFSOperation(int operationName, String onlineResource) {
			super();			
			this.operationName = operationName;
			this.onlineResource = onlineResource;
		}	
		
		/**
		 * @return Returns the onlineResource.
		 */
		public String getOnlineResource() {
			return onlineResource;
		}
		/**
		 * @param onlineResource The onlineResource to set.
		 */
		public void setOnlineResource(String onlineResource) {
			this.onlineResource = onlineResource;
		}
		/**
		 * @return Returns the operationName.
		 */
		public int getOperationName() {
			return operationName;
		}
		/**
		 * @param operationName The operationName to set.
		 */
		public void setOperationName(int operationName) {
			this.operationName = operationName;
		}		
	}
    

}
